/* =============================================================================
 *
 * Copyright (C) 2003 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ===========================================================================*/

//======================================================================
// File Name	:	$RCSfile: apEfmBondingconfig.h,v $
// Description	:	Configuration data structure and information for the
//                  AP in packet Transfer Mode supporting bonding.
//======================================================================

#ifndef _APEFMBONDINGCONFIG_H_
#define _APEFMBONDINGCONFIG_H_

#include <netpro/apconf.h>
 #include <netpro/apclasscfg.h>
 #include <netpro/apipccfg.h>
 #include <netpro/apsyscfg.h>

/* Define the number of IPC queues that will be used between the host and the
 * AP
 */
#define AP_IPC_Q_TO_HOST        4

#include <netpro/apsyscfg.h>

#include <netpro/apprehdr.h>    // egress port specification
#include <sysconf.h>
#include <netpro/efmBonding.h>    //efmBonding related stuff
#include <ethdriver/std_ap/apethconfig.h>
typedef struct
{
    /* Each ethernet AP port has a "port" id. By convention this is the
     * address of the AP input queue. This is used to identify the source of
     * a packet
     */
    void                *pPortAddr;

} apBondingEthConfig_t;

//SN--
#ifndef IS_VX185
// -------------------------------------------
// Structure holding Index to RX fragment ring
// -------------------------------------------
typedef struct apRxFragRingIndex_s
{
    unsigned short   wr_index;      // Write Index of fragmentation ring
    unsigned short   rd_index;      // Read Index of fragmentation ring
} apRxFragRingIndex_t;
#endif
//SN--

// ------------------------------------------------------
// Structure holding Indexes to the two RX fragment rings
// ------------------------------------------------------
typedef struct apRxFragRingIndexes_s
{
    apRxFragRingIndex_t   ring0;
    apRxFragRingIndex_t   ring1;
} apRxFragRingIndexes_t;


/* Define the event bits used by the ethernet AP to indicate conditions
 * to the host
 */
//#define AP_EVT_NO_BUFFER_BIT    9       // No buffer available for Rx
//#define AP_EVT_RX_LOSS_BIT      10      // Rx packet lost
//#define AP_EVT_TX_LOSS_BIT      12      // Transmit queue overflow
//#define AP_EVT_TX_URUN_BIT      13      // Transmit underrun

/* Define the statistics held by the EFM bonding external APs */

typedef struct efmBondingLineStats_s
{
    unsigned long       rxFrags;        // Correctly received Fragments
    unsigned long       rxErrors;       // CRC error received fragments
    unsigned long       rxFragTooLong;  // received frags with size more than 512 bytes
    unsigned long       rxFragTooShort; // received frags with size less than 64 bytes
    unsigned long       rxPhyOverrun;   // number of times overrun observed from phy
    unsigned long       rxBufferOverrun;// number of times overrun observed in Frag buffers 

    unsigned long       txFragSent;     // Number of Fragments sent out by Ext AP
    unsigned long       txFreed2BondMgr; // Packets(Bonding Info) returned to Bonding Manager

} efmBondingLineStats_t;

typedef struct frag_ring_config_info_s
{
    unsigned long start_ptr; /* base addr of memory block allocated by host for the ring */
    unsigned long end_ptr;   /* must be computed as: end_ptr = start_ptr + [(ring_size - 1) * sizeof(ring elem)]*/
                             // end pointer is not used at a moment
    unsigned short ring_size; /* # of fragment BD elements in the ring */
    unsigned short reserved;
}  frag_ring_config_info_t;

typedef struct efmBondingCfg_s
{
    unsigned long line0_Txrate;   /* TX line rate of line0 in Kbps */
    unsigned long line1_Txrate;   /* TX line rate of line1 in Kbps */
    unsigned short line0weight;    /* Fragment weight for Line rate connected to AP0*/
    unsigned short line1weight;    /* Fragment weight for Line rate connected to AP1*/
    unsigned short lineRxDiffTimeTicks; /* ticks for calculation of differential timer */
    unsigned short reserved;      /* ATTENTION! DO NOT remove this element */
    unsigned short lineCtlInfo;    /* Bitmap holding control information about the lines
                                        Bit0 : Flag used by HOST to indicate AP if line0 is Up or not 
                                        Bit1:  Flag used by HOST to indicate AP if line1 is Up or not 
                                        Bit 2-15: Unused
                                    */
    unsigned long fragRingIndexesAddr;   /* Base address of apRxFragRingIndexes_t structure */
    unsigned long eDMAChannelBaseAddr;   /* Start address of eDMA channel BD control info. */
    frag_ring_config_info_t rxFragRing0; /* Information regarding fragment ring0 for reass */
    frag_ring_config_info_t rxFragRing1; /* Information regarding fragment ring1 for reass */
} efmBondingCfg_t;

typedef struct efmReassemStats_s
{
    unsigned long       rxLossStart;        // number of times "start of packet" lost 
                                            // Defn: If a fragment is received with the StartofPacket bit 
                                            // deasserted while the packet assembly function was
                                            // between frames (i.e. waiting for a Start of Packet), 
                                            // discard the offending fragment

    unsigned long       rxLossEnd;          // number of times "end of packet" lost 
                                            // Defn: If a fragment is received with the StartOfPacket bit 
                                            // asserted while the packet assembly function was midframe
                                            // (i.e. waiting for an End of Packet), abort frame transfer,
                                            // starting the next frame with the Start of Packet fragment just received.

    unsigned long       badFragmentReceived;// number of times bad frag received i.e greater than 50% SSID
                                            // Defn: If nextFragmentSequenceNumber is outside the range 
                                            // (expectedFragmentSequenceNumber through 
                                            // expectedFragmentSequenceNumber + (maxSequenceNumber+1)/2)
                                            // then increase BadFragmentReceived 
                                            // Discard the fragment, do not increment ExpectedFragmentSequenceNumber

    unsigned long       lostFragment;       // number of times fragment lost
                                            // Defn:
                                            //  - diff timer expired and exp SSID not rcvd
                                            //  - frag rcvd on both lines and not matching exp SSID

    unsigned long       rxPktTooLong;       // received Pkt with size more than 1536 bytes
    unsigned long       rxPktSuccess;       // Successfully re-assembled fragments

} efmReassemStats_t;

#if 0
/* Define the efm bonding configuration structure. This is a composition of
 * the globally-defined configuration structures.
 *
 * At initialization all parts are used. When updated during execution only
 * the efm bonding parts are used.
 */
typedef struct
{
    apSysConfig_t       sys;            // System (global) configuration
    apEthConfig_t       eth;            // Ethernet configuration
    efmBondingCfg_t     bondingCfg;     // PTM bonding configuration
} ethBondingConfig_t;

#endif



typedef struct
{
#if !(defined(IS_VX185) && (IS_VX185))
    apClassConfig_t     classif;        // Classifier configuration
#endif
    apSysConfig_t       sys;            // System (global) configuration
    apEthConfig_t       eth;            // Ethernet configuration
    efmBondingCfg_t             bondingCfg;             //BONDING configuration
    apMacConfig_t       macAddrList;    // List and a counter of configured MAC Addresses
    apIpcConfig_t       ipc;            // Host IPC config

} ethConfig_bonding_t;

#ifdef AP_ASSEMBLER
#define RT_PADDING_TILL_CLINE       (CACHE_LINE_SIZE - (sizeof typedef apFlowEntry_t & (CACHE_LINE_SIZE - 1)))
#else
#define RT_PADDING_TILL_CLINE       (CACHE_LINE_SIZE - (sizeof(apFlowEntry_t) & (CACHE_LINE_SIZE - 1)))
#endif

typedef struct ethConfig_bonding_s_chaligned
{
        ethConfig_bonding_t     ethConfigMain;
        unsigned char   reserved[RT_PADDING_TILL_CLINE];
} ethConfig_bonding_t_chaligned __attribute__((aligned(CACHE_LINE_SIZE)));

#endif /* _APEFMBONDINGCONFIG_H_ */
