/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

// ===========================================================================
//
// File name    : $RCSfile: apahbdma.h,v $
// Description  : AP AHB DMA definitions for internal ports
// ===========================================================================

#ifndef _APAHBDMA_H_
#define _APAHBDMA_H_

/* These are the definitions for the AP AHB DMA controller ports from the
 * AP side.
 *
 * All port addresses are offsets from a instance-defined address
 *
 * Note that all addresses and lengths are programmable in byte increments,
 * but the most efficient transfers will occur if the following are
 * true:
 *      1. The DMEM address is on a 4 byte boundary
 *      2. The AHB address is on a four byte boundary
 *      3. The transfer length is a AHB burst size
 *         (1, 4, 8 or 16 words [32 bit words]), or a multiple of 16 words
 *
 *      Note that there is an incremental burst mode, but this should only
 *      be used for transfers to/from the PCI bus as internal slaves (in
 *      general) do not understand this type of transfer.
 */
#define AP_AHB_CTRL_OFF     0x0
#define AP_AHB_START_BIT    0           // Start DMA channel. Will be cleared
                                        // when operation is complete
#define AP_AHB_DIR_BIT      1           // Set for AHB -> DMEM
#if 0 //AP_DDR_16_BIT_SUPPORT
/* These bits are made 0 to support 16 bit DDR SDRAM by disabling them. The 0th bit is Start bit which is anyway 
   mandatory for DMA */
#define AP_AHB_16_BURST_BIT 	0           // Use 16word AHB bursts
#define AP_AHB_INC_BURST_BIT    0       // Use incremental bursts (min 2 words)
#define AP_AHB_INC_WLENGTH_BIT  0
#else // default 32 bit support

#define AP_AHB_16_BURST_BIT 	2           // Use 16word AHB bursts
#define AP_AHB_INC_BURST_BIT    3       // Use incremental bursts (min 2 words)
#define AP_AHB_INC_WLENGTH_BIT  4
#endif
#define AP_AHB_ABORT_BIT    8           // Abort current operations. Bit
                                        // reset by H/W when complete

#define AP_AHB_STATUS_OFF   0x1         // AHB status. Need to define bits

#define AP_AHB_MSW_OFF      0x2         // MSW of AHB memory address
#define AP_AHB_LSW_OFF      0x3         // LSW of AHB memory address
#define AP_AHB_DMEM_OFF     0x4         // DMEM address
#define AP_AHB_LENGTH_OFF   0x5         // Transfer length (bytes)

#define AP_AHB_CHANS    4               // Hardware supports 4 channels
#define AP_AHB_CHAN_LEN 8               // Each channel is 8 I/O ports

#endif /* _APAHBDMA_H_ */
