/* Copyright (C) 2006 Ikanos Communications 
 * See IKANOS_PROP_LICENSE.txt for license information. */


//======================================================================
// File Name	:	$RCSfile: apclasscfg.h,v $
// Description	:	AP classifier module configuration information
// CVS ID	:	$Id: apclasscfg.h,v 1.3 2005/04/08 19:28:16 malati Exp $
//======================================================================

#ifndef _APCLASSCFG_H_
#define _APCLASSCFG_H_

#include <netpro/apconf.h>
#include <netpro/apbridge.h>
#include <netpro/apflow.h>
#ifdef AP_DENY
#include <netpro/apfwconfig.h>
#endif /* AP_DENY */
#include <netpro/apflow_ipv6.h>

#if defined(AP2AP_PPPOERELAY_FASTPATH) && (AP2AP_PPPOERELAY_FASTPATH)
#include <netpro/appppoeRelay.h>
#endif

#if defined(IS_VX185) && IS_VX185
#include <netpro/vx185/parserhw.h>
#endif
/* The AP classifier module is present in both the ethernet AP and the
 * buffer manager AP. It performs two major functions:
 *
 * - It acts as a part of an ethernet bridge by forwarding frames that have
 * a known egress port to that egress port under the directory of a
 * host-created bridging table. Unknown frames are passed to the
 * host to perform a "learning" function which may result in the frame being
 * placed into the bridiging table for the port.
 *
 * - It acts as a IP router, modifying frames (TTL, NAT, ethernet header if
 * necessary) and queuing them to an egress port under the direction of a
 * host-created routing table. Unknown frames are passed to the host for
 * firewall checking and may lead to a new entry in the flow table (RFT) for
 * the port.
 *
 * The classifier can perform these tasks at the same time by taking advantage
 * of the fact that frames that are to be routed will have a destination
 * ethernet MAC address that matches that of the port.
 *
 * The operation of the classifier is controlled by the following
 * configuration structures, the flow table, and the bridge table.
 *
 * All classifier information can be safely modified during AP execution,
 * although care should be taken to allow for transient conditions that may
 * occur if the configuration changes while a packet is being processed.
 *
 * These tend to be harmless as long as the host doesn't make assumptions for
 * a short while after the configuration has been changed.
 */


/* Define the configuration of the classifier
 */
typedef struct
{
    /* The MAC address of the port is required to determine whether a
     * received packet is addressed to this port (and should be routed), or
     * is for some other address (and may need to be bridged)
     */
    // macAddr has been transferred into apMacConfig_t structure 
    //unsigned char       macAddr[AP_MAC_ADDR_LEN];

    /* The bridging table is specified by a base address for the bridging
     * table and a hash mask. Basically the 16bit hash is calculated and then
     * folded to be in the range of the mask.
     *
     * This allows tables with less than 64k entries to be used, saving a
     * considerable amount of memory.
     */
    apBridgeEntry_t_chaligned  *pBridgeTable;
    unsigned short      bridgeMask;

    /* The flow table is specified as a base adress for the flow table and
     * a hash mask. The use of the mask is the same as that for the bridging
     * table mask.
     */
    apFlowEntry_t_chaligned       *pFlowTable;
    unsigned short      flowMask;
#if defined(AP2AP_PPPOERELAY_FASTPATH) && (AP2AP_PPPOERELAY_FASTPATH)
    /*
     * The pppoe relay flow table is specified as a base adress for the pppoe relayflow
     * table and a hash mask. The use of the mask is the same as that for the bridging
     * table mask.
     */
    struct t_ap2ap_pppoerelay_session_aligned   *pppoeRelayFlowTable;
    unsigned short      pppoeRelayFlowmask;
#endif

    /* The IPV6 flow table is specified as a base adress for the IPV6 flow table and
     * a IPV6 hash mask.
     */
#if defined(IPV6) && (IPV6)
    apIpv6FlowEntry_t_chaligned   *pIpv6FlowTable;
    unsigned short      flowIpv6Mask;
#endif

    /* The following information controls the handling of VLAN and S-VLAN for
     * the port.
     *
     * Note that some of the operations are only performed if the packet has
     * bridge processing applied.
     */
    unsigned short      control;
#define AP_ALL_ROUTE_BIT        3       /* Treat all received packets as
                                         * routeable (ie. ignore the
                                         * destination MAC address)
                                         */
    /* The state of bridging (as far as the AP is concerned can be represented
     * as three states
     *
     * - Disabled. All non-routed packets are dropped
     * - Listening. All non-routed packets are forwarded to the host CPU
     * - Operational. All non-routed packets are bridge processed
     */
    unsigned char     bridgeState;

#ifdef AP_DENY
    /* Firewall related configuration information:
     * - Firewall enable/disable status
     */
    apFwConfig_t   fwConfig;
#endif /* AP_DENY */

    /* When the AP receives packets that it cannot process it will forward the
     * packet to a single egress port. Note that the egress port may specify
     * that the packet be deleted
     */
    apEgressPort_t      bridgeEgress;   /* Unrecognized bridge packets */

    //apEgressPort_t      ttlZeroEgress;  /* Dest for packets with TTL = zero */
    //apEgressPort_t      ipVerEgress;    /* Dest for packets ver != IPv4 */
    //apEgressPort_t      ipCsumEgress;   /* Dest for packets with IP csum err */
    //apEgressPort_t      ipFragEgress;   /* Dest for packets that are frags */
    apEgressPort_t      unknownEgress;  /* Dest for packets that have no
                                         * flow table entry
                                         */
#ifdef AP_DENY
    apEgressPort_t      fwAttacksEgress;/* Dest for attack related packets */
#endif /* AP_DENY */
    apEgressPort_t      bridgeDrop;     /* Packet should be dropped to BM */
	/* Actions for bridge mark */
#if defined(IS_VX185) && IS_VX185
    unsigned char	qosEnabledBitmap;	/* stores the list of all interfaces on which qdisc is attached */

#define CL_TOS_DSCP2Q_MAP_BIT	0
#define CL_COS2Q_MAP_BIT	1
#define CL_MARK_ACTION_BIT	2
    unsigned char	globalBridgeAction[MAX_BRIDGE_AP_PORTS];
    apbridgeAction_t    bridgeAction[MAX_BRIDGE_AP_PORTS][MAX_BRIDGE_AP_PORTS]; 
#else    
apbridgeAction_t    bridgeAction[MAX_BRIDGE_AP_PORTS]; /* Actions for bridge
mark */
#endif    
unsigned short      defaultVlanId;  /* this stores the default vlan id*/
#if !defined(IS_VX185) || !IS_VX185
#if defined(POLICY_BASED_VLAN) && (POLICY_BASED_VLAN)
    apVlanClass_t       vlanClassTbl[MAX_VLAN_CLASS_ENTRIES];
#endif
#endif
} apClassConfig_t;

/* The classifier code also maintains statistics with respect to the actions
 * performed by the classifier
 *
 */
typedef struct
{
    unsigned long       bridgeDrop;     /* Rx pkts dropped by bridge */
    unsigned long       bridgeFwd;      /* Rx pkts forwarded via bridge */

    unsigned long       route2Host;     /* Rx pkts forwarded by routing */
    unsigned long       routeFwd;       /* Rx pkts forwarded by routing */

#ifdef AP_DENY
    unsigned long       fwAttacksEgress;/* Rx pkts using fwAttacksEgress */
#endif /* AP_DENY */
} apClassStats_t;

// This number defines how many MAC addresses can be configured for AP
#if defined(IS_VX185) && IS_VX185
	#define MAX_NUM_SUPPORTED_MAC_ADDRESSES PARSER_MAX_NUM_MACADDR_LIST
#else

#ifdef BUFFMANAGER
    #if (defined(ETH_NO_CLASSIFICATION) && (ETH_NO_CLASSIFICATION))
        #define MAX_NUM_SUPPORTED_MAC_ADDRESSES 5
    #else
        #define MAX_NUM_SUPPORTED_MAC_ADDRESSES 16
    #endif //ETH_NO_CLASSIFICATION
#elif (defined(PER) && (PER))
    #if defined(ETH_NO_CLASSIFICATION) && (ETH_NO_CLASSIFICATION)
        #define MAX_NUM_SUPPORTED_MAC_ADDRESSES 9
    #else
        #define MAX_NUM_SUPPORTED_MAC_ADDRESSES 8
    #endif //ETH_NO_CLASSIFICATION
#elif defined(ETHERNETDRV) && (ETHERNETDRV)
#ifdef ETH_MULTIMAC
   #define MAX_NUM_SUPPORTED_MAC_ADDRESSES 4 //Multi-Mac Support for GMII2 
#else
   #define MAX_NUM_SUPPORTED_MAC_ADDRESSES 1  
#endif
#else
    #define MAX_NUM_SUPPORTED_MAC_ADDRESSES 1
#endif

#endif	//IS_VX185


typedef struct
{
     /* The MAC address of the port is required to determine whether a
     * received packet is addressed to this port (and should be routed), or
     * is for some other address (and may need to be bridged)
     */
    unsigned char    macAddr[MAX_NUM_SUPPORTED_MAC_ADDRESSES * AP_MAC_ADDR_LEN];

     /* Number of configured MAC addresses AP should compare with
     * before deciding if go for a routing ( in case of match) or
     * bridging process
     */
    unsigned char       num_configured_MAC_Addrs;
} apMacConfig_t;

typedef struct commonEntry_s
{
    union
    {
        struct apFlowEntry_s		routeEntry;

        struct apNewBridgeEntry_s	bridgeEntry;

#if defined(IPV6) && (IPV6)
        struct apIpv6FlowEntry_s    routeIpv6Entry;
#endif /* IPv6 */
#if defined(AP2AP_PPPOERELAY_FASTPATH) && (AP2AP_PPPOERELAY_FASTPATH)
        struct t_ap2ap_pppoerelay       pppoerelayEntry;
#endif
    } cEntry;
} commonEntry_t;


#endif /* _APCLASSCFG_H_ */
