/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

// ===========================================================================
//
// File name    : apbmdebug.h
/* ===========================================================================
 * File name    : apbmdebug.h
 * Description  : Debug definitions for BM AP
 * ===========================================================================
 */

#ifndef _AP_DEBUG_H_
#define _AP_DEBUG_H_

#include <sysconf.h>

// Definitions for DUMP_INFO_ENCHANCED feature
// *** ATTENTION !!! USAGE of symbolic constants below is mandatory ( and case sensitive )
// Abbreviations and notes:
// MSR - Register carring 16 Most Significant bits of address
//       ( It should be used for SDRAM or Local Memory space ONLY)
//       In case if User wants to print block of DMEM it should be substitute with special constant
//       "NO_REG"
// LSR - Register carring 16 Least Significant bits of address
//       Together MSR and LSR combine 32 bit ADDRESS which would be interpreted by Host.
//       If SDRAM or Local Memory block "has been ordered" to be uploaded, 
//       Host simple will copy this block;
//       If DMem block "has been ordered" to be uploaded, Host will organize dmemStructExchange
//       between DMem and SDRAM to read DMem block
// OFFSET - ( ADDRESS + OFFSET ) is start address of the memory block
//       Host will extract information from. Each OFFSET unit is equal 0x100 bytes.
//       Value can be between 0  and 7.
//       For example, value 3 will give the offset 0x300 from ADDRESS.
// SIZE - number of 16 bytes blocks would be extracted and printed by Host from given ADDRESS
// DATATYPE - This constant means what data has been uploaded;
//       Stack part ofapplication can parse uploaded data and present it in desired format
//       In case of uploading memory block without any meaning, use constant "NOTYPE"
// DUMPLEVEL - This constant will be used bu Host which will print corresponded dumpInfo 
//       if the constant value is smaller or equal than currently set in Host
//       2 special cases in Host(Host will not "pay attention" to dumplevel in dumpInfo):
//        - Host will NOT print dumpInfo data if setup value in Host is 0
//        - Host will CUSTOM dumpInfo data ONLY  if setup value in Host is 7

#if 0
///////////      Code example:          /////////////////////////////////////////////////
//   AP code fragment presented below executes DMA transfer of apPreheader from DMEM
//   into SDRAM. We want to present an apPreheader before start of DMA.
//   Register r0 contains start address of apPreheader.
//   In this case 64 bytes ( 4 * 16) of DMem would be uploaded into SDRAM (exchange area)
/////////////////////////////////////////////////////////////////////////////////////////

	lw	r0, class_buf

_CHECK_DMA(CLASS_DMA)
	lbl	ra, _AHB_ADDR(CLASS_DMA, DMEM)	// Write DMEM address
	out	r0, ra
	DUMP_INFO_ENCHANCED( APPREHEADER_STRUCT_DUMP, 20, DUMP_LEVEL_1, NO_REG, R0, 0, 4 )

	lbl	r2, sizeof typedef apPreHeader_t
	lbl	ra, _AHB_ADDR(CLASS_DMA, LENGTH)	// Write transfer length
	out	r2, ra

	lw	r1, r0[typedef apClassInp_t.pCluster]
	lbl	ra, _AHB_ADDR(CLASS_DMA, MSW)	// Write AHB address
	out	r1, ra

	lw	r1, r0[typedef apClassInp_t.pCluster + 2]
	lbl	ra, _AHB_ADDR(CLASS_DMA, LSW)
	out	r1, ra

	lbl	rb, _EXP(AP_AHB_START) | _EXP(AP_AHB_16_BURST)
	lbl	ra, _AHB_ADDR(CLASS_DMA, CTRL) 	// Start write operation
	out	rb, ra
	nop
 #endif


#define	NO_REG                  0x10
#define	R0						0
#define	R1						1
#define	R2						2
#define	R3						3
#define	R4						4
#define	R5						5
#define	R6						6
#define	R7						7
#define	R8						8
#define	R9						9
#define	R10						10
#define	R11						11
#define	R12						12
#define	R13						13
#define	R14						14
#define	R15						15
#define	EXT_RAM_REQ_BIT                                 0x8

// dumpId element inside of below structure is 16 bits wide.
// It's combined from 3 different fields:
// - bits 0..7   - dumpId itself
// - bits 8..12  - datatype in case of DUMP_INFO_ENCHANCED usage
// - bits 13..15 - dump level. It would be possible to filter unnecessary dump prints based on the level
#define DUMP_ID_MASK                                    0x00ff
#define DUMP_INFO_STRUCTURE_MASK                        0x1f00
#define DUMP_LEVEL_MASK                                 0xe000
#define DUMP_LEVEL_BITS_SHIFT                           13
#define NOTYPE                                          0x0000
#define APPREHEADER_STRUCT_DUMP                         0x0100
#define APIPV6_HEADER_STRUCT_DUMP                       0x0200

#define DUMP_LEVEL_0_NOPRINT                            0       // For Host - DO NOT PRINT dumps
#define DUMP_LEVEL_1                                    1
#define DUMP_LEVEL_2                                    2
#define DUMP_LEVEL_3                                    3
// Levels 4, 5 and 6 are reserved
#define DUMP_LEVEL_7_CUSTOM                             7       // For Host - print customized AP dumps ONLY

// More to come

typedef struct
{
    unsigned short      dumpId;         // Identifier of Dump
    unsigned short      reg_0;
    unsigned short      reg_1;
    unsigned short      reg_2;
    unsigned short      reg_3;
    unsigned short      reg_4;
    unsigned short      reg_5;
    unsigned short      reg_6;
    unsigned short      reg_7;
    unsigned short      reg_8;
    unsigned short      reg_9;
    unsigned short      reg_10;
    unsigned short      reg_11;
    unsigned short      reg_12;
    unsigned short      reg_13;
    unsigned short      reg_14;
    unsigned short      reg_15;
    unsigned short      dumpMemBlockInfo;   // Request info to print block of memory
                                            // ( AP Dmem or Local Mem or SDRAM)
} dumpStruct_s;

#if 1	//CACHE::
// Important for CACHE WRITE BACK MODE
// Below definitions are created to have all entries on different cache lines
// In this case a situation when Host and AP are updating elements of different structures
// on the SAME cache line and Host is doing cache flush will be resolved properly
// WITHOUT AP portion data loss.

#ifdef AP_ASSEMBLER
    #define RT_PADDING_TILL_CLINE       (CACHE_LINE_SIZE - (sizeof typedef dumpStruct_s & (CACHE_LINE_SIZE - 1)))
#else
    #define RT_PADDING_TILL_CLINE       (CACHE_LINE_SIZE - (sizeof(dumpStruct_s) & (CACHE_LINE_SIZE - 1)))
#endif

typedef struct dumpStruct_s_chaligned
{
        dumpStruct_s    dumpStructMain;
        unsigned char   reserved[RT_PADDING_TILL_CLINE];
} dumpStruct_s_chaligned __attribute__((aligned(CACHE_LINE_SIZE)));

#ifdef AP_ASSEMBLER
assert (sizeof typedef dumpStruct_s_chaligned & (CACHE_LINE_SIZE - 1)), 0
#else
// will do it later
//assert (0 == (sizeof(dumpStruct_s_chaligned) & (CACHE_LINE_SIZE - 1)));
#endif
#endif		//CACHE::

#if !(defined(DUMP_INFO_FEATURE) && (DUMP_INFO_FEATURE))
#define	DUMP_INFO( dumpId, level )
#else
#define	DUMP_INFO( dumpId, level )     sw     r14, reg14StorePlace                             \
                                       lpc    r14, dumpLabel_## dumpId ## 0                    \
                                       sw     r15, reg15StorePlace                             \
                                       sw     r14, returnAddrStore                             \
                                       li     r15, Breakpoint_Dump_Routine                     \
                                       li     r14, 0                                           \
                                       sw     r14, dumpMemBlockInfo                            \
                                       jmp    if  id r15                                       \
                                       li     r15, dumpId | ( level << DUMP_LEVEL_BITS_SHIFT)  \
                                                                                               \
dumpLabel_## dumpId ## 0:              lw     r14, reg14StorePlace

#endif

#if !(defined(DUMP_INFO_FEATURE) && (DUMP_INFO_FEATURE))
#define	DUMP_INFO_ENCHANCED( dataType, dumpId, level, msr, lsr, offset, size )
#else
#define	DUMP_INFO_ENCHANCED( dataType, dumpId, level, msr, lsr, offset, size )                 \
                                       sw     r14, reg14StorePlace                             \
                                       lpc    r14, dumpLabel_## dumpId ##_## dataType          \
                                       sw     r15, reg15StorePlace                             \
                                       sw     r14, returnAddrStore                             \
                                       li     r15, Breakpoint_Dump_Routine                     \
                                       li     r14, (size<<12) | (offset<<9) | (msr<<4) | (lsr << 0)        \
                                       sw     r14, dumpMemBlockInfo                            \
                                       jmp    if id  r15                                       \
                                       li     r15, (dumpId | dataType | ( level << DUMP_LEVEL_BITS_SHIFT)) \
                                                                                               \
dumpLabel_## dumpId ##_## dataType:    lw     r14, reg14StorePlace

#endif

#endif /* _APBM_DEBUG_H_ */
