/* =====================================================================
 * Copyright (C) 2006 Ikanos Communications 
 * See IKANOS_PROP_LICENSE.txt for license information.
 * =====================================================================
 */

/* ===========================================================================
 * File name    : apbmdebug.h
 * Description  : Debug definitions for BM AP
 * ===========================================================================
 */

#ifndef _AP_DUMPDESCRIPTIONS_H_
#define _AP_DUMPDESCRIPTIONS_H_

#define MAX_DUMP_DESCRIPTION_LENGTH    32
#define MIN_CUSTOM_DUMP_ID             32
typedef struct
{
    unsigned char       dumpId;    // Values: 0..31 are used for standard checked-in dumps
                                   // Custom dumps should use value 31..255
    unsigned char       dumpDescription[MAX_DUMP_DESCRIPTION_LENGTH];
} dumpDescription_s;

unsigned char apNames[MAX_AP_ID + 1][8] = {
			   ""            // apId 0 doesn't exist
			   , "MAC1"
			   , "MAC2"
			   , "MAC3"
			   , "SPA"
			   , "BMU"
			   , "ATM"
			   , "EAP/PERI"
#              if (MAX_AP_ID > 7)
			   , "VDSL"
#              if (MAX_AP_ID > 8)
			   , "CLAP1"
#              if (MAX_AP_ID > 9)
			   , "CLAP2"
#              endif
#              endif
#              endif
	};

dumpDescription_s ethApDumpDescriptorsArray[] = {
              { 0, "ETHDescription 0" },
              { 1, "R0&R1 - Cluster addr to RX pkt" },
              { 2, "ETHDescription 2" },
			  // Customized dumps are below. 
			  // dumpId value should be bigger or equal to MIN_CUSTOM_DUMP_ID
              { 33, "txInfo structure in R6" }
	};
dumpDescription_s spaApDumpDescriptorsArray[] = {
              { 0, "SPADescription 0" },
              { 1, "SPADescription 1" },
              { 2, "SPADescription 2" },
              { 3, "SPADescription 3" },
              { 4, "SPADescription 4" }
	};
dumpDescription_s bmApDumpDescriptorsArray[] = {
              { 0, "BMDescription 0" },
              { 1, "BMDescription 1" },
              { 2, "BMDescription 2" },
              { 3, "BMDescription 3" },
              { 4, "BMDescription 4" }
	};
dumpDescription_s atmApDumpDescriptorsArray[] = {
              { 0, "ATMDescription 0" },
              { 2, "ATMDescription 1" },
              { 2, "ATMDescription 2" },
              { 3, "ATMDescription 3" },
              { 4, "ATMDescription 4" }
	};
dumpDescription_s eapApDumpDescriptorsArray[] = {
              { 0, "EAPDescription 0" },
              { 1, "EAPDescription 1" },
              { 2, "EAPDescription 2" },
              { 3, "EAPDescription 3" },
              { 4, "EAPDescription 4" }
	};
dumpDescription_s vdslApDumpDescriptorsArray[] = {
              { 0, "VDSLDescription 0" },
              { 1, "VDSLDescription 1" },
              { 2, "VDSLDescription 2" },
              { 3, "VDSLDescription 3" },
              { 4, "VDSLDescription 4" }
	};
dumpDescription_s clApDumpDescriptorsArray[] = {
              { 0, "CLDescription 0" },
              { 1, "CLDescription 1" },
              { 2, "CLDescription 2" },
              { 3, "updateParserTag2APPreHdr" },
			  // Customized dumps are below. 
			  // dumpId value should be bigger or equal to MIN_CUSTOM_DUMP_ID
              { 32, "SDMA Descriptor to use in R7" },
              { 33, "Custom description 33" },
};

unsigned int  *apsDumpDescriptorsArray[MAX_AP_ID + 1] = {
			  NULL                            // apId 0 doesn't exist
			  , (unsigned int *) ethApDumpDescriptorsArray     // MAC1_ID 
			  , (unsigned int *) ethApDumpDescriptorsArray     // MAC2_ID 
			  , (unsigned int *) ethApDumpDescriptorsArray     // MAC3_ID 
			  , (unsigned int *) spaApDumpDescriptorsArray     // SPA_ID  
			  , (unsigned int *) bmApDumpDescriptorsArray      // BMU_ID  
			  , (unsigned int *) atmApDumpDescriptorsArray     // ATM_ID  
			  , (unsigned int *) eapApDumpDescriptorsArray     // EAP_ID /PERI 
#             if (MAX_AP_ID > 7)
			  , (unsigned int *) vdslApDumpDescriptorsArray    // VDSL_ID 
#             if (MAX_AP_ID > 8)
			  , (unsigned int *) clApDumpDescriptorsArray      // CLAP1_ID
#             if (MAX_AP_ID > 9)
			  , (unsigned int *) clApDumpDescriptorsArray      // CLAP2_ID
#             endif
#             endif
#             endif
};

unsigned int apsDumpDescriptorsArraysSizes[MAX_AP_ID + 1] = {
			  0                          // apId 0 doesn't exist
			  , sizeof(ethApDumpDescriptorsArray)/sizeof(dumpDescription_s)
			  , sizeof(ethApDumpDescriptorsArray)/sizeof(dumpDescription_s)
			  , sizeof(ethApDumpDescriptorsArray)/sizeof(dumpDescription_s)
			  , sizeof(spaApDumpDescriptorsArray)/sizeof(dumpDescription_s)
			  , sizeof(bmApDumpDescriptorsArray)/sizeof(dumpDescription_s)
			  , sizeof(atmApDumpDescriptorsArray)/sizeof(dumpDescription_s)
			  , sizeof(eapApDumpDescriptorsArray)/sizeof(dumpDescription_s)
#             if (MAX_AP_ID > 7)
			  , sizeof(vdslApDumpDescriptorsArray)/sizeof(dumpDescription_s)
#             if (MAX_AP_ID > 8)
			  , sizeof(clApDumpDescriptorsArray)/sizeof(dumpDescription_s)
#             if (MAX_AP_ID > 9)
			  , sizeof(clApDumpDescriptorsArray)/sizeof(dumpDescription_s)
#             endif
#             endif
#             endif
};

#endif /* _AP_DUMPDESCRIPTIONS_H_ */

