/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

// ===========================================================================
//
// File name    : $RCSfile: apeth.h,v $
// Description  : The I/O layout and bit definitions for the AP ethernet
//                hardware and data source simulation
// ===========================================================================

#ifndef _APETH_H_
#define _APETH_H_

/* The next register controls the operation of the ethernet DMA
 * controller.
 *
 * The AP_ETH_RX_START_BIT, AP_ETH_TX_START_BIT, and AP_ETH_TX_ABORT_BIT
 * bits are set by AP firmware and will be cleared by hardware when
 * the operation is complete.
 *
 * The AP_ETH_RX_START_BIT and AP_ETH_TX_START_BIT bit states are also
 * connected to input flags to allow the AP firmware to determine
 * whether the DMA is complete
 */
#define AP_ETH_CTRL_OFF         0x4             // Control register
#define AP_ETH_RX_START_BIT     0               // Bit to start Rx DMA.
#define AP_ETH_TX_START_BIT     1               // Bit to start Tx DMA
#define AP_ETH_TX_EOF_BIT       2               // Bit to send Tx EOF
#define AP_ETH_RX_ABORT_BIT     8               // Bit to abort Rx DMA
#define AP_ETH_TX_ABORT_BIT     9               // Bit to abort Tx DMA
#define AP_ETH_TX_UNDER_BIT     10              // Tx underrun
#define AP_ETH_TX_PAUSE         11              // Tx sending pause frame
#define AP_ETH_RX_PAUSE         12              // Rx received pause frame

/* The next two registers define the receive status (32 bits)
 * These need to be filled in with the correct definitions by
 * the AP ETH hardware implementor (and this comment removed)
 *
 * These values are only valid after the completion of a DMA operation
 */
#define AP_ETH_RX_MSW_OFF       0x0             // Rx status MSB

#define AP_ETH_RX_EOF_BIT       15              // Rx EOF. !!!! May not be here
#define AP_ETH_RX_TOSS_BIT      14              // Packet tossed at MAC level
#define AP_ETH_RX_ARC_ENT_MASK  0x3e00          // ARC entry #
#define AP_ETH_RX_ARC_STS_MASK  0x01e0          // ARC status
#define AP_ETH_RX_PAUSE_BIT     4               // Packet was pause packet
#define AP_ETH_RX_VLAN_BIT      3               // Packet had VLAN header
#define AP_ETH_RX_BCAST_BIT     2               // Packet has broadcast MAC addr
#define AP_ETH_RX_MCAST_BIT     1               // Packet has broadcast MAC addr
#define AP_ETH_RX_HALTED_BIT    0               // Reception interrupted

#define AP_ETH_RX_LSW_OFF       0x1             // Rx status LSB
#define AP_ETH_RX_HALTED_2_BIT  11              // Reception interrupted (copy)
#define AP_ETH_RX_GOOD_BIT      10              // Reception was good
#define AP_ETH_RX_PAR_BIT       9               // MAC FIFO Rx parity error
#define AP_ETH_RX_LONG_BIT      7               // Packet too long error. This
                                                // only allows for a single
                                                // VLAN hdr, so watch for SVLAN
#define AP_ETH_RX_OVER_BIT      6               // Overrun in Rx FIFO
#define AP_ETH_RX_CRC_BIT       5               // CRC error in packet
#define AP_ETH_RX_ALIGN_BIT     4               // Bit alignment error
#define AP_ETH_RX_INT_BIT       2               // "Rx interrupt" generated
#define AP_ETH_RX_CTL_BIT       1               // Ethernet control fram rx'd
#define AP_ETH_RX_LEN_BIT       0               // Length error (802.3)


/* This register defines the transmit control bits. These effect the
 * way the frame is transmitted.
 *
 * These are to be defined by the hardware implementor
 */
#define AP_ETH_TX_CTRL_OFF      0x5             // Tx  control bits
#define AP_ETH_TX_NO_CRC_BIT    0               /* Do not insert CRC - the
                                                 * last 4 bytes of the data
                                                 * will by used as the CRC
                                                 */
#define AP_ETH_TX_NO_PAD_BIT    1               /* Do not insert pad bytes
                                                 * when packets are less than
                                                 * 64 (60 + CRC) bytes long
                                                 */

/* The following registers control the DMA of received data (ie data
 * entering the ethernet port
 *
 * The address and length written must be aligned on a 4-byte boundary. When
 * reception is complete the length register will contain the number of
 * bytes actually received.
 *
 * The length register is updated as bytes are received. This number
 * will probably increment by 4 as 32 bit words are written. It is
 * important that the number read never exceeds the number of bytes
 * received.
 *
 * The value of unused bytes in the final 32bit word is not defined
 */
#define AP_ETH_RX_ADDR_OFF      0x2             // Rx DMEM transfer addr
#define AP_ETH_RX_LENGTH_OFF    0x3             /* Rx transfer count
                                                 * Write must be 4 byte multiple
                                                 * Read will be number of bytes
                                                 * received so far
                                                 */

/* The following registers control the transfer of data to be written out of
 * the ethernet port.
 *
 * The address must be a multiple of 4 bytes. The length must be a multiple
 * of 4 bytes except for the last transfer in the frame when it should be
 * the exact number of bytes.
 */
#define AP_ETH_TX_ADDR_OFF      0x6             // Tx DMEM transfer addr
#define AP_ETH_TX_LENGTH_OFF    0x7             // Tx transfer count (bytes)

#endif /* _APETH_H_ */
