/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

#ifndef _APEXTERNAL_H_
#define _APEXTERNAL_H_

// This file presents contains AP register offsets from Host point of view

/* Note: Only port 0x0 - 0x7 and 0xc - 0xf should be assumed to be accessable
 * from the host processor
 */

// Port 0 is the AP control port.
#define AP_CTRL_OFFSET                  0x300

/* Port 1 is the current value of the program counter. This is not
 * really useful within an AP program, but this register is also
 * available from the external (CSR) interface to determine the
 * PC of a BRK instruction.
 */
#define AP_PC_OFFSET                    0x304

/* Port 2 represents the "interrupt" state of the AP. These bits
 * are set by the AP and cleared by the host.
 *
 * Port 3 is the AP_INTERRUPT_MASK. These bits can only be set or cleared
 * by the host.
 *
 * If a bit is set in this register and in the same bit is set in the
 * AP_INTERRUPT_MASK register then an interrupt will be seen by the ICU
 * for this AP
 */
#define AP_INTERRUPT_STATUS_OFFSET      0x308

#define AP_INTERRUPT_MASK_OFFSET        0x30C

/* Port 4 and 5 are registers that act as 2 * 16 bit memory locations. By
 * convention these are used to pass a configuration block to the AP from
 * the host CPU during initialization. After initialization they are
 * used in an application specific manner
 */
#define AP_GP_REGISTER_1_OFFSET         0x310

#define AP_GP_REGISTER_2_OFFSET         0x314

/* Port 6 is a register whose bits can be only by set by the host and
 * cleared by the AP. This allows the host to indicate events to the AP
 */
#define AP_EVENT_IN_OFFSET              0x318

/* Port 7 is similar to port 6, except the bits can only be cleared by
 * the host and set by the AP. This allows the AP to indicate events to
 * the host
 */
#define AP_EVENT_OUT_OFFSET             0x31C

/* The following registers are related to the AP internal timer. This
 * timer is driven by the AP clock, with a selectable prescaler
 */
#define AP_TMR_VALUE_OFFSET             0x320

#define AP_TMR_LIMIT_OFFSET             0x324

#define AP_TMR_CTRL_OFFSET              0x328
#define AP_TMR_STATUS_OFFSET            0x32C

#define AP_PM_ADDR_REGISTER_OFFSET      0x330

#define AP_PM_DATA_REGISTER_OFFSET      0x340


/* The following registers are related to the AP input queue.
 */
#define INPQ1_CTRL_OFFSET				0x0500
#define INPQ1_STATUS_OFFSET				0x0504
#define INPQ1_DATA_OFFSET				0x0510

#define INPQ2_CTRL_OFFSET				0x0600
#define INPQ2_STATUS_OFFSET				0x0604
#define INPQ2_DATA_OFFSET				0x0610

#define INPQ3_CTRL_OFFSET				0x0700
#define INPQ3_STATUS_OFFSET				0x0704
#define INPQ3_DATA_OFFSET				0x0710

// Related to Buffer Manager AP ONLY
/* The following registers are related to the AP output queue.
 */
#define OUTQ1_CTRL_OFFSET				0x0800
#define OUTQ1_STATUS_OFFSET				0x0804
#define OUTQ1_DATA_OFFSET				0x0810

#define OUTQ2_CTRL_OFFSET				0x0900
#define OUTQ2_STATUS_OFFSET				0x0904
#define OUTQ2_DATA_OFFSET				0x0910

/* The Vx175/Vx185 and later APs have additional GP registers
 * for use with the split classifier concept
 */
#define AP_GP_REGISTER_3_OFFSET         0x380
#define AP_GP_REGISTER_4_OFFSET         0x384
#define AP_GP_REGISTER_5_OFFSET         0x388
#define AP_GP_REGISTER_6_OFFSET         0x38c
#define AP_GP_REGISTER_7_OFFSET         0x390
#define AP_GP_REGISTER_8_OFFSET         0x394
#define AP_GP_REGISTER_9_OFFSET         0x398
#define AP_GP_REGISTER_10_OFFSET        0x39c
// Address of register which contains deviceId information
#define DEVICE_IDENTIFICATION_REG_ADDR  0xb900003C
#define DEVICE_IDENTIFICATION_REG_ADDR_VX185  	0xb9000018

#endif /* _APEXTERNAL_H_ */
