/* Copyright (C) 2006 Ikanos Communications 
 * See IKANOS_PROP_LICENSE.txt for license information. */

#ifndef _APFW_H_
#define _APFW_H_

#define TRUE		1
#define FALSE		0

/* IP Options information */
#define	IPOPT_EOL	0		/* end of option list */
#define	IPOPT_NOP	1		/* no operation */

#define	IPOPT_RR	7		/* record packet route */
#define	IPOPT_TS	68		/* timestamp */
#define	IPOPT_SECURITY	130		/* provide s,c,h,tcc */
#define	IPOPT_LSRR	131		/* loose source route */
#define	IPOPT_SATID	136		/* satnet id */
#define	IPOPT_SSRR	137		/* strict source route */
#define	IPOPT_RA	148		/* router alert */

/* TCP Flags */
#define	TH_FIN		0x01
#define	TH_SYN		0x02
#define	TH_RST		0x04
#define	TH_PSH		0x08
#define	TH_ACK		0x10
#define	TH_URG		0x20

/*
 * Offsets to fields in options other than EOL and NOP.
 */
#define	IPOPT_OPTVAL	0		/* option ID */
#define	IPOPT_OLEN	1		/* option length */
#define IPOPT_OFFSET	2		/* offset within option */
#define	IPOPT_MINOFF	4		/* min value of above */

/* Offset of IP and TCP header fields */
#define IP_LEN_OFF	0x2		/* Offset to total length field */
#define TCP_HDRLEN_OFF	0xc		/* Offset to tcp data offset field */

#define IP_SRC_OFF	0xc		/* Offset to source address */
#define IP_DST_OFF	0x10		/* Offset to destination address */
#define IP_PROTOCOL_OFF	0x9		/* Offset to IP protocol */
#define TCP_LEN_RES_FLAG_OFF	0xc	/* Offset to TCP flags */

#define TCP_FLAGS_MASK	0xff		/* SYN/FIN/RST flags */
#define IP_TCP_PROTOCOL	6		/* TCP packet */

/*
 * This macro sets the attack type in pre-header field and en-queues
 * the packet to host.
 */
#define ENQUE_TO_HOST(ulAttackType)                        \
        lbl     r1, ulAttackType                           \
        li	r5, class_fw_attack                        \
        jmp if id	r5                                 \
        sw      r1, r0[typedef apClassInp_t.header.flags2] \
	nop

/* TCP Flags attack */
#define TH_NOFLAGS          0
#define TH_ALLFLAGS         TH_PSH|TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG
#define TH_FIN_SYN_RST_ACK  TH_FIN|TH_SYN|TH_RST|TH_ACK
#define TH_SYN_ACK          TH_SYN|TH_ACK
#define TH_FIN_RST          TH_FIN|TH_RST
#define TH_SYN_FIN          TH_SYN|TH_FIN
#define TH_FIN_RST_PSH_URG  TH_FIN|TH_RST|TH_PSH|TH_URG
#define TH_FIN_ACK          TH_FIN|TH_ACK

/* List of attacks detected by AP */
#define ATTACK_TCP_HDR_FRAG       1
#define ATTACK_TCP_NO_FLAGS       2
#define ATTACK_TCP_ALL_FLAGS      3
#define ATTACK_TCP_NOSENSE_FLAGS  4
#define ATTACK_TCP_SYNACK_RSTFIN  5
#define ATTACK_TCP_SYN_FIN        6
#define ATTACK_TCP_SYN_NOACK      7
#define ATTACK_TCP_FIN_NOACK      8
#define ATTACK_IP_BAD_OPTION      9
#define ATTACK_IP_UNSUPP_OPTION   10
#define ATTACK_LAND               11
#define ATTACK_MAX                ATTACK_LAND

/* Log message format */
typedef struct {
    unsigned long  ulFacility;   /* Module logging the message */
    unsigned long  ulSeverity;   /* Priority of the message */
    unsigned long  ulMsgFormat;  /* Message format: WELF/None */
    unsigned long  ulFilterId;   /* Filter id to match */
    unsigned short uDataId;      /* Id to interpret data following hdr */
} apLogMsgHdr_t;

/* In addition to facility, severity, message format and filter id 
 * following data is needed to log an attack:
 * - Attack Type: Indicates to host the attack detected
 * - Packet Pointer: Packet that is detected as illegal
 */
typedef struct {
    unsigned long ulAttackType;
    unsigned char *pPacket;
} apAttackData_t;
#endif /* _APFW_H_ */
