/* =====================================================================
 * Copyright (C) 2006 Ikanos Communications 
 * See IKANOS_PROP_LICENSE.txt for license information.
 * =====================================================================
 */

/* ===========================================================================
 * File name    : $RCSfile: apgige.h,v $
 * Description  : The I/O layout and bit definitions for the AP ethernet
 *                hardware and data source simulation
 * ===========================================================================
 */
#include <sysconf.h>

#ifndef _APGIGE_H_
#define _APGIGE_H_

/* The next register controls the operation of the ethernet DMA
 * controller.
 *
 * The AP_ETH_RX_START_BIT and AP_ETH_TX_START_BIT
 * bits are set by AP firmware and will be cleared by hardware when
 * the operation is complete.
 *
 * The AP_ETH_RX_START_BIT and AP_ETH_TX_START_BIT bit states are also
 * connected to input flags to allow the AP firmware to determine
 * whether the DMA is complete
 */
#define AP_ETH_CTRL_OFF         0x4             // Control register
#define AP_ETH_RX_START_BIT     0               // Bit to start Rx DMA.
#define AP_ETH_TX_START_BIT     1               // Bit to start Tx DMA
#define AP_ETH_TX_EOF_BIT       2               // Bit to send Tx EOF
#define AP_ETH_TX_CRC_BIT       3               // Append CRC on Tx
#if defined(IS_VX185) && (IS_VX185)
#define AP_ETH_TX_UNDER_BIT     6               // Tx underrun
#else
#define AP_ETH_TX_UNDER_BIT     10              // Tx underruna
#endif

#define AP_ETH_TX_AHB_DMEM_BIT  7               // Tx to DMEM or SDRAM

/* The next register holds the receive status (16 bits)
 * These need to be filled in with the correct definitions by
 * the AP ETH hardware implementor (and this comment removed)
 *
 * These values are only valid after the completion of a DMA operation
 */
#define AP_ETH_RX_STATUS_OFF    0x0             // Rx status MSB

#define AP_ETH_RX_EOF_BIT       15              // Rx EOF. !!!! May not be here
#define AP_ETH_RX_FILTER_BIT    14              // Packet to be discarded
#define AP_ETH_RX_CRC_BIT       13              // CRC (or other Rx) error
//RVRV#define AP_ETH_RX_ORUN_BIT      7               // Overrun during this packet
#define AP_ETH_RX_ORUN_BIT      12              // Overrun during this packet
#define AP_ETH_RX_ORUN_MASK     0x0f00          // Mask for overrun count
#define AP_ETH_RX_ORUN_SHFT     8               // Shift for overrun count

/* The following registers control the DMA of received data (ie data
 * entering the ethernet port
 *
 * The address and length written must be aligned on a 4-byte boundary. When
 * reception is complete the length register will contain the number of
 * bytes actually received.
 *
 * The length register is updated as bytes are received. This number
 * will probably increment by 4 as 32 bit words are written. It is
 * important that the number read never exceeds the number of bytes
 * received.
 *
 * The value of unused bytes in the final 32bit word is not defined
 */
#define AP_ETH_RX_ADDR_OFF      0x2             // Rx DMEM transfer addr
#define AP_ETH_RX_LENGTH_OFF    0x3             /* Rx transfer count
                                                 * Write must be 4 byte multiple
                                                 * Read will be number of bytes
                                                 * received so far
                                                 */

/* The following registers control the transfer of data to be written out of
 * the ethernet port.
 *
 * The address must be a multiple of 4 bytes. The length must be a multiple
 * of 4 bytes except for the last transfer in the frame when it should be
 * the exact number of bytes.
 */
#define AP_ETH_TX_ADDR_OFF      0x6             // Tx DMEM transfer addr
#define AP_ETH_TX_LENGTH_OFF    0x7             // Tx transfer count (bytes)

#endif /* _APGIGE_H_ */
