/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

// ===========================================================================
//
// File name    : $RCSfile: apinternal.h,v $
// Description  : Description of basic I/O ports common to all APs
// ===========================================================================

#ifndef _APINTERNAL_H_
#define _APINTERNAL_H_

/* Note: Only port 0x0 - 0x7 and 0xc - 0xf should be assumed to be accessable
 * from the host processor
 */

// Port 0 is the AP control port.
#define AP_CTRL                 0x0

#define AP_CTRL_RESET_BIT       15              // Reset AP (overrides RUN bit)
#define AP_CTRL_RUN_BIT         14              // Allow AP to run
#define AP_CTRL_AHBDMA2_BIT     8              // Enable AHB DMA 2 on this AP
#define AP_CTRL_BRK_MASK        (0x00ff)        // BRK data bits

/* Port 1 is the current value of the program counter. This is not
 * really useful within an AP program, but this register is also
 * available from the external (CSR) interface to determine the
 * PC of a BRK instruction.
 */
#define AP_PC                   0x1

/* Port 2 represents the "interrupt" state of the AP. These bits
 * are set by the AP and cleared by the host.
 *
 * Port 3 is the AP_INTERRUPT_MASK. These bits can only be set or cleared
 * by the host.
 *
 * If a bit is set in this register and in the same bit is set in the
 * AP_INTERRUPT_MASK register then an interrupt will be seen by the ICU
 * for this AP
 */
#define AP_INTERRUPT_STATUS     0x2
#define AP_INTERRUPT_MASK       0x3

/* Port 4 and 5 are registers that act as 2 * 16 bit memory locations. By
 * convention these are used to pass a configuration block to the AP from
 * the host CPU during initialization. After initialization they are
 * used in an application specific manner
 */

#define AP_GP_REGISTER_1        0x4
#define AP_GP_REGISTER_2        0x5

/* Although not strictly part of the GP registers the following were added to
 * Vx175/Vx185 to enhance split classifier operation. 
 * 
 * Needless to say they are only present on the Vx175/Vx185 and later
 * FUSIV-based devices, and not the Vx200, Vx150, Vx160, Vx170/Vx180 etc.
 */
#if defined(IS_VX185) && (IS_VX185)

#define AP_GP_REGISTER_3        0x48
#define AP_GP_REGISTER_4        0x49
#define AP_GP_REGISTER_5        0x4a
#define AP_GP_REGISTER_6        0x4b
#define AP_GP_REGISTER_7        0x4c
#define AP_GP_REGISTER_8        0x4d
#define AP_GP_REGISTER_9        0x4e
#define AP_GP_REGISTER_10       0x4f

#elif IS_VX180

#define AP_GP_REGISTER_3        0x80
#define AP_GP_REGISTER_4        0x81
#define AP_GP_REGISTER_5        0x82
#define AP_GP_REGISTER_6        0x83
#define AP_GP_REGISTER_7        0x84
#define AP_GP_REGISTER_8        0x85
#define AP_GP_REGISTER_9        0x86
#define AP_GP_REGISTER_10       0x87

#endif
/* Port 6 is a register whose bits can be only by set by the host and
 * cleared by the AP. This allows the host to indicate events to the AP
 */
#define AP_EVENT_IN             0x6

/* Port 7 is similar to port 6, except the bits can only be cleared by
 * the host and set by the AP. This allows the AP to indicate events to
 * the host
 */
#define AP_EVENT_OUT            0x7

/* The following registers are related to the AP internal timer. This
 * timer is driven by the AP clock, with a selectable prescaler
 */

#define AP_TMR_VALUE            0x8             // Holds the timer value. This
                                                // is in the range
                                                // 0 .. limit - 1
#define AP_TMR_LIMIT            0x9             // Specifies limit for the timer

#define AP_TMR_CTRL             0xa
#define AP_TMR_ENABLE_BIT       0               // Enable timer
#define AP_TMR_RESET_BIT        1               // Reset timer
#define AP_TMR_SCALE_BIT        2               // Enable /16 prescaler

#define AP_TMR_STATUS           0xb             // Status bits. Write 1 to
                                                // clear each bit
#define AP_TMR_LIMIT_BIT        0               // Set when limit reached.
                                                // Also connected to AP input
                                                // flag #0
#define AP_TMR_MISS_BIT         1               // Set of the limit is reached
                                                // when AP_TMR_LIMIT_BIT is set

#endif /* _APINTERNAL_H_ */
