/* Copyright (C) 2006 Ikanos Communications 
 * See IKANOS_PROP_LICENSE.txt for license information. */


//======================================================================
// File Name	:	$RCSfile: apipccfg.h,v $
// Description	:	Define queue based interface for AP -> host packets
//====================================================================== 

#ifndef _APIPCCFG_H_
#define _APIPCCFG_H_

#include <netpro/apipc.h>

/* AP ingress and egress ports may need to forward packets (chains of
 * clusters) to the system host CPU.
 *
 * This interface performs those tasks. It has the following features
 *      - supports multiple queues (data prioritization)
 *      - queue indexes seperate from queues (allows indexes to be in
 *        fast memory for lower overheads)
 *
 * When the AP writes to the queue it will update the apNextRead index for the
 * queue, and set the interrupt bit that matches the queue.
 *
 * The host CPU will then clear the interrupt bit and read the entry at
 * hostNextRead, and do a circular increment. This process will continue until 
 * apNextWrite == hostNextRead
 *
 * Note that the maximum capacity of the queue is maxEntries - 1.
 */


/* The following structure defines the information that is passed the the
 * AP tp specify the IPC queues.
 *
 * It would be very difficult to dynamically change this information without
 * losing several frames of data.
 */
#if defined(AP_IPC_Q_TO_HOST) && (AP_IPC_Q_TO_HOST)
typedef struct
{
    /* Array of queue state specifiers */
    apQueueSpec_t       cpuQueueSpec[AP_IPC_Q_TO_HOST];

    /* Pointer to array of queue states. These are in external memory as they
     * are referred to by both the AP and the host
    */
    apQueueState_t      (*pCpuQueueState)[AP_IPC_Q_TO_HOST];
} apIpcConfig_t;
#endif

/* Define the event register bits used to indicate packets in the IPQ host
 * queue to the host.
 */
#define AP_EVT_QUEUE0_UPD_BIT   0       // Bit for Queue 0 updates
#define _AP_EVT_QUEUE_UPD_BIT(n)        (AP_EVT_QUEUE0_UPD_BIT + (n))

/* AP interrupt events generated during IPC operation */
#define AP_EVT_QUEUE_FULL_BIT   11      // Queue to CPU full

#endif /* _APIPCCFG_H_ */
