// ===========================================================================
//
// Copyright (C) 2002 Analog Devices Inc. All rights reserved
//
// The information and source code contained herein is the exclusive property
// of Analog Devices and may not be disclosed, examined or reproduced in whole
// or in part without the explicit written authorization from Analog Devices.
//
// ===========================================================================

#ifndef _APIPCQOS_H_
#define _APIPCQOS_H_

#define INVALID_PRIORITY			0xff
#define INVALID_ETHERTYPE			0xffff

#define DSCP_TABLE_SIZE				64
#define ETHERTYPE_TABLE_SIZE		        8
#define COS_TABLE_SIZE				8

#define DSCP_MAX_VALUE				63
#define DSCP_MASK				0x3F
#define COS_MAX_VALUE				7

typedef struct
{
    unsigned char       dscpPriorityTable[DSCP_TABLE_SIZE];
    /* Next 2 tables are for Ethertype priority settings.
	 * They are defined separately to save 8 bytes of structure alignment "holes".
     */
    unsigned short      ethertypeValuesTable[ETHERTYPE_TABLE_SIZE];
    unsigned char       ethertypePriorityTable[ETHERTYPE_TABLE_SIZE];
    unsigned char       cosPriorityTable[COS_TABLE_SIZE];
    unsigned char       defaultPriority;

} apIpcQPriorityInfo_t;

#if 1		//CACHE::
// Important for CACHE WRITE BACK MODE
// Below definitions are created to have all entries on different cache lines
// In this case a situation when Host and AP are updating elements of different structures
// on the SAME cache line and Host is doing cache flush will be resolved properly
// WITHOUT AP portion data loss.

#ifdef AP_ASSEMBLER
    #define RT_PADDING_TILL_CLINE       (CACHE_LINE_SIZE - (sizeof typedef apIpcQPriorityInfo_t & (CACHE_LINE_SIZE - 1)))
#else
    #define RT_PADDING_TILL_CLINE       (CACHE_LINE_SIZE - (sizeof(apIpcQPriorityInfo_t) & (CACHE_LINE_SIZE - 1)))
#endif

typedef struct apIpcQPriorityInfo_s_chaligned
{
        apIpcQPriorityInfo_t   apIpcQPriorityInfoMain;
        unsigned char   reserved[RT_PADDING_TILL_CLINE];
} apIpcQPriorityInfo_t_chaligned __attribute__((aligned(CACHE_LINE_SIZE)));
#undef RT_PADDING_TILL_CLINE

#ifdef AP_ASSEMBLER
assert (sizeof typedef apIpcQPriorityInfo_t_chaligned & (CACHE_LINE_SIZE - 1)), 0
#else
// will do it later
//assert (0 == (sizeof(apIpcQPriorityInfo_t_chaligned) & (CACHE_LINE_SIZE - 1)));
#endif
#endif		//CACHE::

#endif /* _APIPCQOS_H_ */
