/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in  part without  the explicit written authorization from Ikanos Communications   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : ap2ap_pppoerelay.h                                                    *
 * Author       : Jagannath Pattar      [ jpattar@ikanos.com ]                          *
 * Version      :                                                                       *
 * Date         : 24th July 2008                                                        *
 * Description  : This header file contains required data structure for fastpath 	*
 *		  support for pppoe relay.						*
 *                                                                                      *
 ****************************************************************************************/


#ifndef _AP_PPPOERELAY_FATPATH_H_
#define _AP_PPPOERELAY_FATPATH_H_
/*--------------------------------------INCLUDES----------------------------------------*/
#include <defines.h>
#include <netpro/apprehdr.h>		//apEgressPort_t is defined here
#include <netpro/apflow.h>		//AP_MAX_FLOW_EGRESS is defined here
#ifdef PPPOER_PASSTHROUGH_IPV6
#include <netpro/apflow_ipv6.h>		//AP_MAX_FLOW_EGRESS is defined here
#endif
/*---------------------------------------MACROS-----------------------------------------*/
#define AP_PPPOE_RELAY_VALID_BIT        0
#define AP_PPPOE_RELAY_VLAN_BIT     	1	//0: Remove vlanId, 1: Add vlanId
#ifdef CONFIG_FUSIV_VX185
#define AP_PPPOE_RELAY_HASHMASK		127
#else
#define AP_PPPOE_RELAY_HASHMASK		255
#endif


/*-----------------------------------DATA STRUCTURE-------------------------------------*/
#ifndef CONFIG_FUSIV_VX185
/* 
 * This is the main data structure for fastpath pppoerelay support. This data structure
 * will  be  accessed  by  APs  modify  according to  this and  forwards  the  packets.
 */ 
typedef struct t_ap2ap_pppoerelay
{
        struct 	t_ap2ap_pppoerelay       *ps_next;       //Next node
	apEgressPort_t  egressList[AP_MAX_FLOW_EGRESS];  //egress list
	UINT16		ui16_in_mac[3];		//incoming packets MAC address
	UINT16		ui16_subs_mac[3];	//destination MAC address going to be replaced with
	UINT16		ui16_src_mac[3];	//mac addess of sending iface, used while sending packets
	UINT16		ui16_in_sessid;		//pppoeRelay assigned session id to client
	UINT16		ui16_subs_sessid;	//pppoe server assigned real session id
	UINT16		ui16_vlanid;		//to store vlan id
	UINT16		ui16_flags;		//tells what to do with vlan id field. Add/delete/replace
	VOID		*pv_user_hdlr;		//this stores the corresponding host entry. 
	UINT32		ui32_pktcnt;		//incoming packet counter
} s_ap2ap_pppoerelay;
#endif

typedef struct t_ap2ap_pppoerelay_entry_info
{
#ifdef CONFIG_FUSIV_VX185
	apFlowEntry_t    	*ps_entry;	
#ifdef PPPOER_PASSTHROUGH_IPV6
	apIpv6FlowEntry_t    	*ps_entry_ipv6;	
#endif
#else
	s_ap2ap_pppoerelay	*ps_entry;	
#endif
	UINT16			ui16_hash;
	CHAR			c_apid;
}s_ap2ap_pppoerelay_entry_info;

	
#ifdef AP_ASSEMBLER
        #define PRELAY_RT_PADDING_TILL_CLINE    (CACHE_LINE_SIZE - (sizeof typedef s_ap2ap_pppoerelay) & (CACHE_LINE_SIZE - 1))
#else
        #define PRELAY_RT_PADDING_TILL_CLINE    (CACHE_LINE_SIZE - (sizeof(s_ap2ap_pppoerelay) & (CACHE_LINE_SIZE - 1)))
#endif

#ifndef CONFIG_FUSIV_VX185
typedef struct t_ap2ap_pppoerelay_session_aligned
{
	s_ap2ap_pppoerelay	relay_flow_entry;
	UCHAR			uc_reserved[PRELAY_RT_PADDING_TILL_CLINE];
}s_ap2ap_pppoerelay_session_aligned;
#endif //CONFIG_FUSIV_VX185

#endif //_AP_PPPOERELAY_FATPATH_H_
