/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

// ===========================================================================
//
// File name    : $RCSfile: apsecurity.h,v $
// Description  : The I/O layout and bit definitions for the AP security
//                hardware
// ===========================================================================

#ifndef _APSECURITY_H_
#define _APSECURITY_H_

/* Note that TX and RX in this file refer to the DMA controllers that
 * write data to the CEN and read output from the CEN respectively
 */

/* These registers control the DMA that takes encrypted output data from the
 * CEN as it is produced.
 *
 * Note that the data produced during encryption (HASH) is read from I/O
 * registers, not via DMA
 */
#define AP_SPA_RX_CTRL_OFF      0x0
#define AP_SPA_START_RX_BIT     0               // Start DMA from CEN
#define AP_SPA_RX_ALG_BIT       1               // Encryption algorithm
                                                // 0 = DES/3DES 1 = AES
#define AP_SPA_RX_ABORT         8               // Abort DMA

#define AP_SPA_RX_ADDR_OFF      0x1             // DMEM addr for DMA. 32bit
                                                // aligned
#define AP_SPA_RX_LENGTH_OFF    0x2             // DMA length in bytes. Must
                                                // by 32bit/4byte multiple

/* These registers control the DMA that injects data into the CEN. This is
 * used to inject data for hashing and encryption
 */
#define AP_SPA_TX_CTRL_OFF      0x4
#define AP_SPA_START_TX_BIT     0               // Start DMA to CEN
#define AP_SPA_LOAD_BIT         1               // Load double-buffered keys
                                                // into CEN engines
#define AP_SPA_FIRST_BIT        2               // This is the first block
                                                // of an encryption/hash
#define AP_SPA_LAST_BIT         3               // This is the last block of
                                                // an encryption/hash
#define AP_SPA_TX_ALG_MASK      0x0030          // Defines flow into security
                                                // engines
#define AP_SPA_TX_DES           0x0000          // DES/3DES engine
#define AP_SPA_TX_AES           0x0010          // AES engine
#define AP_SPA_TX_HASH          0x0020          // Hashing operation only
#define AP_SPA_TX_PASS_MASK     0x00c0          // Defines how engines are
                                                // connected
#define AP_SPA_NOPASS           0x0000          // No data passed b/w engines
#define AP_SPA_PARALLEL         0x0040          // Engines in parallel
#define AP_SPA_PIPE_BIT         0x0080          // Enc engine to hash engine
#define AP_SPA_TX_ABORT_BIT     8               // Abort DMA

#define AP_SPA_TX_ADDR_OFF      0x5             // TX DMEM addr, 32 bit align

#define AP_SPA_TX_LENGTH_OFF    0x6             // TX length, 4byte/32bit mult

/* These registers control the operation of the encryption engine. They
 * are set at the start of an packet. Items that change as the encryption
 *
 * Values that change during the processing should be supplied by TX control
 * register to allow the future addition of a second TX controller
 */

#define AP_SPA_ENCR_CTRL_LSW_OFF        0x10
#define AP_SPA_KEY_SIZE_MASK    0x000c          // Key size for encrypt
#define AP_SPA_KEY_128          0x0000          // 128 bit key
#define AP_SPA_KEY_192          0x0004          // 192 bit key
#define AP_SPA_KEY_256          0x0008          // 256 bit key
#define AP_SPA_ENCR_RESET_BIT   6               // Reset encryption block.
                                                // Cleared by H/W
#define AP_SPA_ENCRYPT_BIT      7               // 1 = Encrypt 0 = decrypt
#define AP_SPA_ENCR_ALG_MASK    0x0f00          // Encryption mode
#define AP_SPA_ALG_ECB          0x0000
#define AP_SPA_ALG_CBC          0x0100
#define AP_SPA_ALG_CFB          0x0200
#define AP_SPA_ALG_OFB          0x0300
#define AP_SPA_ALG_TECB         0x0400
#define AP_SPA_ALG_TCBC         0x0500
#define AP_SPA_ALG_TFCB         0x0600
#define AP_SPA_ALG_TOFB         0x0700
#define AP_SPA_ALG_RECB         0x0800
#define AP_SPA_ALG_RCBC         0x0900
#define AP_SPA_ALG_RFCB         0x0a00
#define AP_SPA_ALG_ROFB         0x0b00

#define AP_SPA_ENCR_CTRL_MSW_OFF        0x11
#define AP_SPA_LENGTH_MASK      0x007f          // Feedback length for CFB
#define AP_SPA_BLOCKS_MASK      0xfe00          // #blocks for CFB data packing

#define AP_SPA_AUTH_CTRL_OFF    0x12
#define AP_SPA_IV_FLAG_BIT      3               // Intermediate IV flag
#define AP_SPA_HMAC_BIT         4               // 1 = HMAC, 0 = single pass
#define AP_SPA_AUTH_RESET_BIT   6               // Reset authentication engine
#define AP_SPA_IR_FLAG_BIT      7               // Intermediate result for
                                                // hash algorithm
#define AP_SPA_AUTH_ALG_MASK    0x0700          // Authentication algorithm
#define AP_SPA_ALG_MD5          0x0600          // MD5
#define AP_SPA_ALG_SHA1         0x0700          // SHA-1
#define AP_SPA_ALG_SHA256       0x0400          // SHA-256

/* The following are base addresses for blocks of registers that hold
 * keys, IV, and hash values.
 *
 * Only the base register is specified as these will typically be 
 * accessed using indirect I/O inside loops
 */
#define AP_SPA_HASH_BASE_OFF    0x15            // CEN_HASH
#define AP_SPA_KEY_BASE_OFF     0x2f            // CEN_KEY_CRYPTO
#define AP_SPA_IV_BASE_OFF      0x3f            // CEN_IV_CRYPTO
#define AP_SPA_NEXT_IV_BASE_OFF 0x47            // CEN_NEXT_IV_CRYPTO
#define AP_SPA_MSGLEN_BASE_OFF  0x4f            // CEN_MSGLEN_HASH
#define AP_SPA_KEY_HASH_BASE_OFF        0x53    // CEN_KEY_HASH
#define AP_SPA_IV_HASH_BASE_OFF 0x5d            // CEN_IV_HASH

#endif /* _APSECURITY_H_ */
