/* Copyright (C) 2006 Ikanos Communications 
 * See IKANOS_PROP_LICENSE.txt for license information. */


//======================================================================
// File Name	:	$RCSfile: apsyscfg.h,v $
// Description	:	AP system information for AP configuration
//======================================================================

#ifndef _APSYSCFG_H_
#define _APSYSCFG_H_

#include <netpro/apconf.h>

/* This information is "system" information used by all AP firmware.
 *
 * This information will not change during the execution of the AP firmware
 */
typedef struct
{
    /* In general the AP needs to know where the Buffer Manager AP
     * buffer allocate and buffer free queues are located.
     */
    void                *pAllocate;     /* read to allocate cluster */
    void                *pFree;         /* write to free cluster */

    /* The following address gives the location of the "exchange" information
     * used to implement AP debugging operations. It is expected to by the
     * location of a block of 256 bytes.
     */
    void                *pExchange;

	/* This element if bit field and presents available features of AP code */
	 unsigned short features_bits;
#if (defined(IS_VX185) && IS_VX185) || (defined(IS_VX175) && IS_VX175)
#if defined(POLICY_BASED_VLAN) && (POLICY_BASED_VLAN)
	 unsigned short policy_features_bits;
#endif
#endif
} apSysConfig_t;
#if defined(POLICY_BASED_VLAN) && (POLICY_BASED_VLAN)
/*feature enabling/disabling bit map for respective AP*/
#define	POLICY_VLAN_MAC1_ID				0	//ETH0 AP
#define	POLICY_VLAN_PERI_ID				1	//PERI AP
#define	POLICY_VLAN_MAC2_ID				2	//ETH1 AP
#define	POLICY_VLAN_MAC3_ID				3	//ETH2/VDSL/ATM AP

#endif	

/* The following bits of the AP EVENT IN register are globally defined for all
 * AP instances
 */
#define AP_EVT_CONFIG_BIT       0       // Load/reload configuration
#define AP_EVT_DEBUG_BIT        1       // Debugging support

/* General AP EVENT OUT bits */
#define AP_EVT_STATS_OFLOW_BIT  14      // Statistics overflow
#define AP_EVT_PKT_DROP_BIT     15      // Packet dropped due to deadlock

// An information below describes all possible features User can configure at Run-time or before AP load
// It is specific variable inside of configuration structure named "features_bits" which would be changed
// everytime User will call a function "apChangeFeature"
// This variable is 16 bit wide :
// bits 0-3   - used for specific functions when some additional settings needed to be changed
//             for example, for AP IP_QOS weights should be initialized
// bits 4-11  - used for toggling (ONLY) particular feature
// bits 12-14 - reserved for internal usage
// bit  15    - will provide an information if AP has been loaded ALREADY
//              If Yes, funuction "apDymamicReconfig" will be called
#define	DISABLE_MODE                        0
#define	ENABLE_MODE                         1

/* Features bits definitions */
#define MIN_FEATURE_BIT                     0
#define MAX_FEATURE_BIT                     14

#define AP_IPQOS_BIT          	     0   // related to IPQoS inside of AP
#define AP_CLAP_DROP_PAUSE_FRAMES    0   // Drop Pause frames at CLAP   (applicable for only CLAP)

#define AP_IPQOS_BORROW_BIT          1   // unused BW of the queues is distributed
#define AP_IPQOS_PQ_BIT              2   // run PQ scheduler

#define AP_COS2Q_MAP_BIT             7   // MAP vlan COS values to scheduler Qs
#define AP_TOS_DSCP2Q_MAP_BIT        8   // MAP TOS/DSCP values to scheduler Qs
#if defined(PORT_MONITORING) && (PORT_MONITORING)
#define AP_PORT_MONITORING_BIT               9   // enable/disable port monitoring
#endif

#define AP_FRAGMENT_PROCESS_BIT      4   // related to Fast Path for fragmented packets
#define AP_ROUTE_THRU_AP_BIT         5   // Route thru AP or thru Host
#define AP_BRIDGE_THRU_AP_BIT        6   // Bridge thru AP or thru Host

#define AP_BRIDGE_ACTION_BIT         10  // To enable bridge actions like
                                        // marking and setting priority
#if defined(POLICY_BASED_VLAN) && (POLICY_BASED_VLAN)
#define AP_BRIDGE_VLAN_CLASS_BIT         11  // To enable vlan policy tbl
#endif

#if defined(VENDOR_TAG) && (VENDOR_TAG)
#define AP_VENDOR_TAG_PROCESS_BIT           3  // Vendor specific Tag header processing
#endif

// Added the bit to send packet to routing even mac doesn't match
#define AP_ROUTE_PROMISC_BIT                  12 // For KT
#define AP_BRIDGE_VLAN_ENABLE_BIT             13 /* If any bridge configured with vlan interfaces, then it will be set to 1.*/
#define AP_PPPOE_PASSTHROUGH_FEATURE_BIT      14 /*If PPPoE-Passthrough adds a entry in BFT, then AP will 
						   consider session Id in calculating hash for BFT lookup. */

#define AP_SYS_CONFIGURED_BIT       15  // This bit means if AP has been loaded already

#define QOS_ENABLE		ENABLE_MODE
#define QOS_DISABLE		DISABLE_MODE

#define BORROW_ENABLE                       ENABLE_MODE
#define BORROW_DISABLE                      DISABLE_MODE

//#define FRAGMENT_FAST_ENABLE                0x01
//#define FRAGMENT_FAST_DISABLE               0x02

#endif /* _APSYSCFG_H_ */
