/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

// ===========================================================================
//
// File name    : $RCSfile: aputopia.h,v $
// Description  : The I/O layout and bit definitions for the AP security
//                hardware
// ===========================================================================

#ifndef _APUTOPIA_H_
#define _APUTOPIA_H_

/* These registers represent the Utopia RX DMA. This hardware supports 64
 * channels with automatic DMA checking.
 *
 * Other channels will need to be checked using the IDMA controller on the
 * AHB bus
 */
#define AP_UTP_RX_CTRL_OFF      0x0             // Control / status reg for Rx
#define AP_UTP_START_RX_BIT     0               // Start Rx DMA. Cleared
                                                // by H/W at end of DMA.
                                                // Attached to input flag

#define AP_UTP_ABORT_RX_BIT     1               // Abort Rx DMA
#define AP_UTP_RX_FULL          2              // RX FIFO almost full status
#define AP_UTP_RX_EMPTY         3              // RX FIFO almost empty status


#define AP_UTP_RX_ADDR_OFF      0x2             // DMEM address to transfer

#define AP_UTP_RX_CHAN_STATUS_OFF    0x4        // Remaining Rx status bits
#define AP_UTP_RX_MATCH             0           // PHY/VCC match table entry
#define AP_UTP_RX_MATCH_MASK        0x0001      // VCC match bit field
#define AP_UTP_RX_PTI_LAST_CELL     1           // Bit 0 of PTI field: last cell
                                                // indication
#define AP_UTP_RX_PTI_RM_CELL       3           // Bit 2 of PTI field: resource
                                                // management cell indicator
#define AP_UTP_RX_PRTY_ERR_BIT      4
#define AP_UTP_RX_HEC_ERR_BIT       5           // HEC or parity error in hdr
#define AP_UTP_PLD_TYPE_MASK    0x000e          // Payload type from hdr
#define AP_UTP_RX_PHY_MSK       0x07c0          // PHY number. Used if cell


#define AP_UTP_RX_STATUS_OFF       0x6             // Remaining Rx status bits

#define AP_UTP_RX_VCC_MASK      0x003f          // VCC table entry for cell
#define AP_UTP_RX_CRC_ERR_BIT       6           // CRC32 /AAL5 CRC error. Valid
                                                // only for last cell in frame
#define AP_UTP_RX_CLP_BIT           7           // Cell loss priority field
#define AP_UTP_RX_CLP_MASK          0x80        // CLP bit field
#define AP_UTP_RX_GFC_MSK           0x0f00      // GFC filed of ATM cell header

/* These registers represent the Utopia TX DMA. The hardware supports
 * ATM header insertion, AAL5 CRC generation and insertion for 64 channels
 *
 * If the cell is one of the 64 hardware channels only the ATM payload is
 * given (48 bytes) and the hardware inserts the ATM header and (optionally)
 * the accumulated AAL5/CRC32 value
 *
 * Otherwise the first four bytes of the ATM header (HEC calculated by H/W)
 * and the ATM payload (48 bytes) are given. For channels other than the
 * 64 supported by hardware the AP firmware will need to perform
 * AAL5 CRC32 calculation via external IDMA controller
 *
 */
#define AP_UTP_TX_CTRL_OFF      0x8             // Control reg for Tx
#define AP_UTP_START_TX_BIT     0               // Start transmit. Cleared
                                                // by H/W at end of DMA.
                                                // Attached to input flag

#define AP_UTP_TX_ABORT_BIT               1      // Abort cell TX
#define AP_UTP_TX_FULL_BIT                2      // TX FIFO almost full status
#define AP_UTP_TX_EMPTY_BIT               3      // TX FIFO almost empty status
#define AP_UTP_TX_PTI_LAST_CELL_IND_BIT   8      // PTI bit 0: last cell
                                                 //            indication


#define AP_UTP_TX_CLP_BIT   11      // CLP Bit field in Tx Control Register 
#define AP_UTP_TX_CLP_MASK          0x0800        // CLP bit field

#define AP_UTP_TX_ADDR_OFF      0xa             // Tx DMEM address

#define AP_UTP_TX_CHAN_CTRL_OFF        0xc             // Data specifying processing
                                                // when AP_UTP_VCC_VALID_BIT
                                                // is set.


#define AP_UTP_TX_VCC_MASK      0x003f          // VCC number for this cell
#define AP_UTP_TX_HDR_BIT         6             // ATM header insertion
#define AP_UTP_TX_CRC_BIT         12            // Accumulate CRC32 for cell

#define AP_UTP_TX_LAST_BIT        13            // Insert AAL5/CRC32 into cell


#define AP_UTP_UNMAPPED_CHAN_BIT    14          // Indicates the hardware
                                                // that the header to the Cell
                                                // does not come from the HEADER RAM

#define AP_UTP_UNMAPPED_HEADER_BIT  15          //  It indicates whether the current data
                                                //  under transmission is header or payload

#define AP_UTP_TX_PHY_MASK      0x0f80          // PHY for cell

#endif /* _APUTOPIA_H_ */
