/* =============================================================================
 *
 * Copyright (C) 2003 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ===========================================================================*/

//======================================================================
// File Name	:	$RCSfile: efmBonding.h,v $
// Description	:   Definitions for EFM bonding code
// CVS ID	:	$Id: efmBonding.h,v 1.1.10.1 2012/03/17 12:16:19 csrivats Exp $
//======================================================================

#ifndef _EFMBONDING_H_
#define _EFMBONDING_H_

//#include "apprehdr.h"

/* General definitions */
#define TX_FRAGMENT_DATA_SIZE 256


//#define TX_FRAGMENT_DATA_SIZE 320

#define TX_FRAGMENT_HDR_SIZE 2
#define TX_FRAGMENT_SIZE 258

//#define TX_FRAGMENT_SIZE 322

#define TX_MIN_FRAGMENT_DATA_SIZE 64
#define AP_TX_FRAG_BUF_LEN  (TX_FRAGMENT_DATA_SIZE + TX_MIN_FRAGMENT_DATA_SIZE)

#define XDSL_AP_BIT 0
#define ETH_AP_BIT 1

/* For RX Code */

#define AP_FRAG_HDR_SIZE 2
#define MAX_FRAG_BUF_DATA_LEN 512
#define RX_MIN_FRAG_SIZE 64
#define FRAG_BD_CTRL_WORD_SIZE 8
#define RX_SLB_QUEUE_NUM  0 //rx slb queue# used for rxing frags
// #define RX_DMA 1 //AHB DMA channel# used by frag rx task
#define NUM_FRAG_RING_ELEM 128
#define NUM_FRAG_RING0_ELEM (NUM_FRAG_RING_ELEM)
#define NUM_FRAG_RING1_ELEM (NUM_FRAG_RING_ELEM)


/* For Reassembly code */

/*AHB DMA channel# used by reassembly task*/
#define REASM_DMA 0x2

/*AP cluster field offsets*/
#define AP_CLUSTER_PREHEADER_SIZE 64
#define AP_CLUSTER_PREPAYLOAD_GAP_SIZE 128
#define AP_CLUSTER_PKT_PAYLOAD_OFFSET (AP_CLUSTER_PREHEADER_SIZE + AP_CLUSTER_PREPAYLOAD_GAP_SIZE)

/*Status bits in fragment header --part of fragment context*/

/*Only EOP bit in fragment header is set --last fragment of packet*/
#define FRAG_HDR_EOP_SET 0x1

/*Only SOP bit in fragment header is set --first fragment of packet*/
#define FRAG_HDR_SOP_SET 0x2

/* Both SOP and EOP bits in fragment header are set --fragment is self contained pkt*/
#define FRAG_HDR_EOP_SOP_SET 0x3


/*Status bits in reassembly context --part of packet context*/

/* Reassembly state machine received an EOP fragment the last time */
#define REASSM_CTXT_EOP_SET 0x8
#define REASSM_CTXT_EOP_CLEAR 0xf7
#define REASSM_CTXT_EOP_BIT 0x3

/* Reassembly state machine received an SOP fragment the last time */
#define REASSM_CTXT_SOP_SET 0x10
#define REASSM_CTXT_SOP_CLEAR 0xef

/* The SSID of fragment received on ring0 did not match expected sequence id */
#define REASSM_CTXT_UNMATCHED0_SET 0x20
#define REASSM_CTXT_UNMATCHED0_CLEAR 0xdf
#define REASSM_CTXT_UNMATCHED0_BIT 0x5

/* The SSID of fragment received on ring1 did not match expected sequence id */
#define REASSM_CTXT_UNMATCHED1_SET 0x40
#define REASSM_CTXT_UNMATCHED1_CLEAR 0xbf
#define REASSM_CTXT_UNMATCHED1_BIT 0x6

/*Status of diff. timer - spawned or not */
#define REASSM_CTXT_DIFF_TIMER_SET 0x4
#define REASSM_CTXT_DIFF_TIMER_CLEAR 0x1fb
#define REASSM_CTXT_DIFF_TIMER_BIT 0x2


/*Status bits of lines - up or not*/
#define REASM_LINE0_STATUS_BIT 0x0
#define REASM_LINE1_STATUS_BIT 0x1

/* Maximum permissible value of an ethernet frame
 * used to check for frame length overflow errors
 */
#define MAX_ETHERNET_FRAME_LENGTH 1536


/* Maximum legal value of expected sequence id
 * a.k.a. 2*pow(14) - 1 = 0x3fff
 */
#define MAX_EXPECTED_SEQUENCE_ID 0x3fff

/*Defined as: (((MAX_EXPECTED_SEQUENCE_ID + 1)/2) -1)*/
#define WINDOW_SIZE_FOR_SSID_DEVIATION 0x1fff

/* Offset of total length field in the cluster preheader*/
#define CLUSTER_PREHDR_TOTAL_LENGTH_OFFSET 0x10 //16 bytes from cluster base addr


/*External DMA macro definitions --used by reassembly*/
#define  DMA_BASE_ADDRESS 0x19120000

/* eDMA channel# in use from 1 to 16*/
#define CHAN_IN_USE 0x5

/* _a_ is the DMA channel# from 1 to 16 */
#define GET_DMA_CHANNEL_ADDRESS(_a_)  (DMA_BASE_ADDRESS + 0x0010 * (_a_))

/*eDMA Channel Status CSR bit masks --as per documentation in Section 3 of Vx180 programmer guide*/
/* If set APU got an interrupt from eDMA block*/
#define CHAN_STATUS_APU_INT 0x8

/* All BDs in the channel have been processed*/
#define CHAN_STATUS_DONE 0x40

/* Processing of current BD is completed*/
#define CHAN_STATUS_CURR_DONE 0x80

/* For single BD either CURRENT DONE / DONE / BOTH should be set */
#define CHAN_STATUS_DONE_SINGLE_BD 0xc0

/* eDMA channel is currently active */
#define CHAN_STATUS_DMA_ACTIVE 0x100

/* eDMA channel control info. bit map setup*/
#define CHAN_CTRL_INFO_VAL_LSW 0x4400
//#define CHAN_CTRL_INFO_VAL_LSW 0xC4C0
#define CHAN_CTRL_INFO_VAL_MSW 0x0101

/* eDMA BD control word info. MSW bit map setup*/
#define BD_CTRL_INFO_VAL_MSW 0x0066



#if 0	//SN
typedef struct apRxFragBuf_s
{
    unsigned char   reserved;    	// added for 8 byte alignment and future use
    unsigned char   status;     	// Buffer state

#define BONDING_AP_RX_IDLE 	0       // Buffer is idle
#define BONDING_AP_RX_READY 	1       // Buffer is ready to receive

    unsigned short  length;     		// Size of fragment in bytes
    unsigned short  fragmentHdr ;	// 2 byte fragment header
   unsigned char   buf[MAX_FRAG_BUF_DATA_LEN + RX_MIN_FRAG_SIZE]; // Fragment data
} apRxFragBuf_t;


typedef struct apRxFragBD_s
{
// Note: This structure matches with that of receive frag buffer
    unsigned int    reserved;    	// kept for 8 byte alignment and future use
    unsigned short  length;     	// Size of fragment in bytes (only 10 bits required)
    unsigned short  fragmentHdr;	// 2 byte fragment header
    unsigned char   buf[MAX_FRAG_BUF_DATA_LEN + RX_MIN_FRAG_SIZE]; // Fragment data
} apRxFragBD_t;
#endif
//SN+
typedef struct apRxFragBD_s
{
    unsigned char   state;
#define  BUFFER_AVAILABLE           0
#define  BUFFER_FULL                1
    unsigned char   channel;        // Slot Buffer Channel Number for Ring buffer store purpose
    unsigned char   flags;          // Flags for indicating EOP/SOP (Non-bonding case)
#define     EOP_RCVD_BIT    0       // EOP received in current transfer
    unsigned char   rsvd;           // Future use and alignment word 2
    unsigned short  rsvd2;
    unsigned short  length;
    unsigned short  fragmentHdr;
    unsigned char   buf[MAX_FRAG_BUF_DATA_LEN];
} apRxFragBD_t;


typedef struct apRxFragRingIndex_s
{
    unsigned short   wr_index;     	// Write Index of fragmentation ring
    unsigned short   rd_index;     	// Read Index of fragmentation ring
    unsigned short   count;         // Number of entries currently used
    unsigned short   rsvd;          // alignment bytes
    unsigned int	 wr_cluster;    // DDR address of write pointer
    unsigned int	 rd_cluster;    // DDR address of read pointer
} apRxFragRingIndex_t;
//SN+

// The following structure is used for holding the frag info
// in US direction
typedef struct
{
    unsigned char       tx_ssid[2];       // SSID of the fragment to be transmitted
    unsigned char       numOfFragsInPkt;
    unsigned char       fragBitMap[3];
    unsigned char       numOfAPs;   // APs on which cluster needs to be placed
                                    // If Bit 0 is set, AP0 will be sending the frags
                                    // If Bit 1 is set, AP0 will be sending the frags
} ssid_fraginfo_t;


/*Data structures specific to reassembly logic*/

//frag BD control info, one instance (local copy) of this is maintained in AP DMEM per rx frag ring
typedef struct apRxFrag_s
{
    unsigned char   	status;     		// Buffer state

#define BONDING_REASM_AP_RX_IDLE  0       	// Buffer is idle
#define BONDING_REASM_AP_RX_READY 1      	// Buffer is ready to reassemble

    unsigned char   	reserved;     		// reserved for future use
    unsigned short  	length;     		// Size of fragment in Bytes
    unsigned short  	fragmentHdr;		// 2 byte fragment header
    void *		currFragRingRdpointer;	// pointer to current location in fragment ring
} apRxFrag_t;

//represents the reassembly context
typedef struct apRxReassembly_s
{
    unsigned char   status;     	// Buffer state
    unsigned char   reserved;
    unsigned short  length;     	// Size of packet in Bytes (as it is being reassembled), DMAed to cluster pre-header on receving EOP

    void *cluster ; //represents the reassembled pkt buffer
    void *currFragWritePtr ;   //used to DMA current fragment at the right position in packet
    void *ringWrIndexLoc; // Location of the write index for frag rings (fetched on detecting ring empty condition)
    apPreHeader_t apPreHeader; //64 bytes of AP preheader info. to be updated to cluster preheader in external memory
} apRxReassembly_t;

/*Represents DMA BD control info. for external memory transfers --sizeof eDMA BD CSRs = 4 LWs*/
typedef struct eDMA_BD_ctrl_info_s
{
    unsigned long 	ctrl_word; //MSW:LSW -- BD control info:Buffer length
    unsigned long 	src_addr;
    unsigned long 	dst_addr;
    unsigned long 	reserved; // placeholder for CRC computation result, not used
}eDMA_BD_ctrl_info_t;

/*Represents DMA channel control info. for external memory transfers --sizeof eDMA channel CSRs = 4 LWs*/
typedef struct eDMA_channel_ctrl_info_s
{
    unsigned long 	channel_base_addr; //must be loaded with eDMA BD base address
    unsigned long 	channel_ctrl_info; //defines eDMA channel characteristics
    unsigned long 	channel_status_word; //defines status of eDMA progress, read to check eDMA completion on the channel
    unsigned long 	curr_bd_addr; //contains address of eDMA BD currently being processed
}eDMA_channel_ctrl_info_t;


#endif /* _EFMBONDING_H_ */
