/* =====================================================================
 * Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information.
 * =====================================================================
 */

/* ===========================================================================
 * File name    : $RCSfile: apconf.h
 * Description  : Modules configuration to build AP binaries
 * ===========================================================================
 */

#ifndef _MULTIPLICATION_H_
#define _MULTIPLICATION_H_

#define ROUTE_ENTRY_BASIC_SIZE				96
#define BRIDGE_ENTRY_BASIC_SIZE				64

#define ROUTE_IPV6_ENTRY_BASIC_SIZE				120

#if defined(VENDOR_TAG) && (VENDOR_TAG)
#define VENDOR_TAG_ADDITION					8
#else
#define VENDOR_TAG_ADDITION					0
#endif

#if defined(WLAN_ETH_8023) && (WLAN_ETH_8023)
#define WLAN_ETH_8023_ADDITION				16
#else
#define WLAN_ETH_8023_ADDITION				0
#endif

#if defined(IPV4_INTO_IPV6) && (IPV4_INTO_IPV6)
#define IPV4_INTO_IPV6_ADDITION				32
#else
#define IPV4_INTO_IPV6_ADDITION				0
#endif

#if defined(AP_FASTPATH_STATISTICS) && (AP_FASTPATH_STATISTICS)
#define FASTPATH_STATISTICS_ADDITION                    24
#else
#define FASTPATH_STATISTICS_ADDITION                    0
#endif

#define ROUTE_ENTRY_SIZE_NO_PAD			(ROUTE_ENTRY_BASIC_SIZE + VENDOR_TAG_ADDITION + WLAN_ETH_8023_ADDITION + IPV4_INTO_IPV6_ADDITION + FASTPATH_STATISTICS_ADDITION)
#define BRIDGE_ENTRY_SIZE_NO_PAD		(BRIDGE_ENTRY_BASIC_SIZE + VENDOR_TAG_ADDITION + WLAN_ETH_8023_ADDITION + FASTPATH_STATISTICS_ADDITION)

#define ROUTE_IPV6_ENTRY_SIZE_NO_PAD			(ROUTE_IPV6_ENTRY_BASIC_SIZE + WLAN_ETH_8023_ADDITION + FASTPATH_STATISTICS_ADDITION)

#define RT_PADDING_TILL_CLINE_ADD			(CACHE_LINE_SIZE - (ROUTE_ENTRY_SIZE_NO_PAD & (CACHE_LINE_SIZE - 1)))
#define BR_PADDING_TILL_CLINE_ADD			(CACHE_LINE_SIZE - (BRIDGE_ENTRY_SIZE_NO_PAD & (CACHE_LINE_SIZE - 1)))
#define RT_IPV6_PADDING_TILL_CLINE_ADD			(CACHE_LINE_SIZE - (ROUTE_IPV6_ENTRY_SIZE_NO_PAD & (CACHE_LINE_SIZE - 1)))

#define ROUTE_ENTRY_SIZE					( ROUTE_ENTRY_SIZE_NO_PAD  + RT_PADDING_TILL_CLINE_ADD )
#define BRIDGE_ENTRY_SIZE					( BRIDGE_ENTRY_SIZE_NO_PAD  + BR_PADDING_TILL_CLINE_ADD )
#define IPV6_ENTRY_SIZE					( ROUTE_IPV6_ENTRY_SIZE_NO_PAD  + RT_IPV6_PADDING_TILL_CLINE_ADD )	


#define MULTIPLICATION_BY_192( r16msb, r16lsb, rauxmsb, rauxlsb) \
  lbl	r16msb, 0        	\
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
							\
  mov     rauxlsb, r16lsb   \
  mov     rauxmsb, r16msb   \
							\
  lsl	r16lsb              \
  slc	r16msb              \
							\
  add     r16lsb,rauxlsb    \
  addc    r16msb,rauxmsb

#define MULTIPLICATION_BY_160( r16msb, r16lsb, rauxmsb, rauxlsb) \
  lbl	r16msb, 0        	\
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
							\
  mov     rauxlsb, r16lsb   \
  mov     rauxmsb, r16msb   \
							\
  lsl	r16lsb              \
  slc	r16msb              \
  lsl	r16lsb              \
  slc	r16msb              \
							\
  add     r16lsb,rauxlsb    \
  addc    r16msb,rauxmsb

#define MULTIPLICATION_BY_128( r16msb, r16lsb, rauxmsb, rauxlsb) \
  lbl	r16msb, 0			\
  lsl	r16lsb              \
  slc	r16msb              \
  lsl	r16lsb              \
  slc	r16msb              \
  lsl	r16lsb              \
  slc	r16msb              \
  lsl	r16lsb              \
  slc	r16msb              \
  lsl	r16lsb              \
  slc	r16msb              \
  lsl	r16lsb              \
  slc	r16msb              \
  lsl	r16lsb              \
  slc	r16msb

#define MULTIPLICATION_BY_96( r16msb, r16lsb, rauxmsb, rauxlsb) \
  lbl	r16msb, 0     		\
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl	r16lsb		        \
  slc	r16msb              \
  lsl   r16lsb              \
  slc   r16msb              \
  lsl   r16lsb              \
  slc   r16msb              \
							\
  mov   rauxlsb, r16lsb     \
  mov   rauxmsb, r16msb     \
							\
  lsl   r16lsb              \
  slc   r16msb              \
							\
  add   r16lsb,rauxlsb      \
  addc  r16msb,rauxmsb


#if ( ROUTE_ENTRY_SIZE == 192 )
#define ROUTE_MULTIPLICATION	MULTIPLICATION_BY_192
#elif ( ROUTE_ENTRY_SIZE == 160 )
#define ROUTE_MULTIPLICATION	MULTIPLICATION_BY_160
#elif ( ROUTE_ENTRY_SIZE == 128 )
#define ROUTE_MULTIPLICATION	MULTIPLICATION_BY_128
#elif ( ROUTE_ENTRY_SIZE == 96 )
#define ROUTE_MULTIPLICATION	MULTIPLICATION_BY_96
#else
 ERROR in ROUTE ENTRY SIZE
#endif

#if ( BRIDGE_ENTRY_SIZE == 160 )
#define BRIDGE_MULTIPLICATION	MULTIPLICATION_BY_160
#elif ( BRIDGE_ENTRY_SIZE == 128 )
#define BRIDGE_MULTIPLICATION	MULTIPLICATION_BY_128
#elif ( BRIDGE_ENTRY_SIZE == 96 )
#define BRIDGE_MULTIPLICATION	MULTIPLICATION_BY_96
#else
 ERROR in BRIDGE ENTRY SIZE
#endif

#if ( IPV6_ENTRY_SIZE == 192 )
#define IPV6_MULTIPLICATION	MULTIPLICATION_BY_192
#elif ( IPV6_ENTRY_SIZE == 160 )
#define IPV6_MULTIPLICATION	MULTIPLICATION_BY_160
#elif ( IPV6_ENTRY_SIZE == 128 )
#define IPV6_MULTIPLICATION	MULTIPLICATION_BY_128
#elif ( IPV6_ENTRY_SIZE == 96 )
#define IPV6_MULTIPLICATION	MULTIPLICATION_BY_96
#else
 ERROR in ROUTE ENTRY V6 SIZE
#endif

#endif /* _MULTIPLICATION_H_ */
