/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */


//============================================================================
// File Name    :
// Description  :
// CVS ID       :
//============================================================================

#ifndef _APSECDEF_H_
#define _APSECDEF_H_

#include <sysconf.h>

/* This file contains
 *
 */

#define FRAG 1

#define   SA_TO_PKT_INFO_SZ    16        // size (src addr+ dest addr+ spi+ seq#)
#define   IP_ESP_HDR_SIZE      28        // IP header + ESP fixed header

#define   SEC_PORT               0

/*--- #define   BLOCK_SIZE           128 ---*/ /* BLA@AVM: Never used in fusiv_src, but redefined system wide BLOCK_SIZE */
#define   BLOCK_SPACE     128 + 48
#define   TUN_BUF_SIZE         128


#define   L2_HDR_LEN            14
#define   IP_HDR_LEN            20    // outer IP header size = 20 bytes.
#define   ESP_FIX_HDR            8
#define   ICV_LENGTH            12
#define   IPSEC_ICV_LENGTH      12

          // ESP fixed header [ seq # + spi] size = 8
#define   ESP_FIX_HDR_SIZE              8
#define   ESP_FIX_HDR_IPHDR_SIZE       28
#define   ESP_PADLEN_PROTO_FLDS_SIZE    2
#define   CEN_HASH_BLK_SIZE            64

#define   AH_HDR_SIZE                  24
#define   AH_FIXED_HDR                 12

#define   IP_ADDR_OFFSET               12
#define   IPHDR_PROTO_FIELD_OFFSET     9

#define   NEXT_PKT                      1
#define   NO_PROCESS                    0
#define   PRE_PROCESS                   1


           // In the bits[15..0], the 13th bit is set when the ESP with
           // encryption only is selected.
#define   CEN_IR_FLAG                  9
#define   FRAGMENT_BIT                 14


#define   CEN_HW_BOUND_MASK           0x3
#define   CEN_HW_BOUNDARY             0x4
          // the padding requirement in the ESP_AUTH only case to makeup
          // for multiple of 4 bytes.
#define   ESP_AUTH_ONLY_BLK_LEN       0x04
#define   ESP_AUTH_ONLY_BLK_MASK      0x03
#define   SEC_HW_4BYTE_MAKEUP           2

#define   CEN_CRYPTO_KEY_REG0         0x84
#define   CEN_AUTH_KEY_REG0           0xa4


#define   IANA_ESP_PROTO_NUM            50
#define   IANA_AH_PROTO_NUM            51
#define   IPV4HDR_VER_HDRLEN          0x45
#define   IANA_IP_PROTO_NUM              4

#define   IP_VER_V4					  0x40		/* IPv4 */
#define   IP_VER_V6    				  0x60		/* IPv6 */

#define   ZERO                           0


typedef struct crypto_s
{

#define CRYPT_DATA      0  //bit 0
#define CRYPT_PROTO     1  //bit 1
        unsigned char type;
        unsigned char flags;
        unsigned short len;
        unsigned int  dataStart;
        unsigned int  dataEnd;
	unsigned char hash[32];  //hash value output from SEC Engines

}crypto_t;


typedef struct apsa
{
        struct apsa  *pNext;
	crypto_t   crypto;

#define      ESP_OUTGOING_FUNC  1
#define      ESP_INCOMING_FUNC  2
#define      AH_OUTGOING_FUNC   3
#define      AH_INGOING_FUNC    4
#define      CEN_DIRECT         5
     /* sftCtrl_1 contains the software control information.
      * LSB 3 bits contains the information of the functions to be called
      * Security AP
      *  000 - in valid
      *  001 - ESP outgoing
      *  010 - ESP incoming
      *  011 - AH outgoing
      *  100 - AH  incoming
      *  101 - crypto fn
      *
      *  Bits from 12 - 15  and bit 8 & 9 are defined as below
      */

  #define   CEN_CRYPTO_ENCRYPT           8	// If bit is set it's a crypto; otherwise - hash
  #define   IPV6_SABIT			 9      // If bit is set it's IPv6 packet processing
  #define   ESP_IV_T_DES_AES             12     // If set IV is 16 bytes; otherwise 8
  #define   ESP_ENCRYPTION_ONLY          13     // If set encrypt/decrypt ONLY, no Authentication
  #define   ESP_AUTHENTICN_ONLY          14     // If set Authentication ONLY, no encrypt/decrypt
  #define   PROTO_AH                     15     // Is set 1 is an Authentication; otherwise used
												// to add ESP trailer length with padlength
    unsigned short     sftCtrl_1;

#define   ESP_AUTH_CTRL_WORD          0x0023
#define   AUTH_FIRST_BLK              0x0233  // ( AH/ESP )
#define   AUTH_HMAC_FIRST_BLK         0x0223  // ( AH/ESP )
#define   AH_INTERM_BLK_FLAG          0x0225

	unsigned short     sftCtrl_2;

         /* holds the frist block programming information and the
            info is used to program the CSRs for the consecutive
            blocks of the same packet. In case of ESP with Auth &
            decryption or encryption mulittask field and in other
            case IR, FIRST & LOAD_IV. */

#define ESP_EA_FIRST_BLK              0x004f	// 0x004d
#define ESP_E_FIRST_BLK               0x000f
#define ALGO_AES                      0x0010
#define CEN_AD_PARALLEL               0x0080
#define LD_NEWKEYS                    0x0002
#define LD_NEWKEYS_RESET_MASK         0xfffd
#define FIRST_BLOCK_BIT		      2
#define LOAD_IV_BIT		      3
    unsigned short     cenTxCsrF;        // control information for 1st block

#define MASK_LDN_FB_LI                0xfff1
#define ESP_A_FIRST_BLK               0x0223
#define ESP_A_INTERM_BLK              0x0225
#define AH_INTERM_BLK_FLAG            0x0225
#define LAST_BLK_MASK                 0xfdff
    unsigned short     cenTxCsrL;        // control info for sub-sequent blocks

#define CEN_CRYPTO_FUNC				  0x1
#define CEN_AUTH_FUNC				  0x0
#define CEN_MASK_ENCR_BIT             0xff7f
#define CEN_CTRL_ENCR                 0x0080
#define CEN_CTRL_DES                  0x0100
#define CEN_CTRL_TDES                 0x0500
#define CEN_CTRL_AES128               0x0900
#define CEN_CTRL_AES192               0x0908
#define CEN_CTRL_AES256               0x0910
    unsigned short     cenCtrlLo;

#define CEN_SET_TO_ZERO               0x0000
    unsigned short     cenCtrlHi;

#define CEN_ICV_3WORDS                0x1000
#define CEN_AUTH_CTRL_MD5             0x0e00
#define CEN_AUTH_CTRL_SHA1            0x0f00
#define CEN_AUTH_CTRL_SHA2            0x0c00
#define CEN_AUTH_CTRL_HMACMD5         CEN_AUTH_CTRL_MD5
#define CEN_AUTH_CTRL_HMACSHA1        CEN_AUTH_CTRL_SHA1
#define CEN_AUTH_CTRL_HMACSHA2        CEN_AUTH_CTRL_SHA2
    unsigned short     authCtrl;

#define CEN_RX_START                  0x0001
#define CEN_RX_AES_BIT                0x0002
    unsigned short     rxCtrl;

    unsigned short     cryptoKey[16]; // store encr/decr key max 32bytes
    unsigned short     iv[8];         // store IV key  max of 16bytes
    unsigned short     authKey[16];   // store auth key max of 32 bytes

    union
    {
      struct
      {
                      // most significant 16bits of  IPSEC tunnel source addr
        unsigned short     srcAddrMs;
                       // least significant 16bits of  IPSEC tunnel source addr
        unsigned short     srcAddrLs;
                       // most significant 16bits of  IPSEC tunnel Dest onaddr
        unsigned short     dstAddrMs;
                       // least significant 16bits of  IPSEC tunnel Dest addr
        unsigned short     dstAddrLs;
                       // most significant 16bits of  Security Parameter Index
        unsigned short     spiMs;
                       // least significant 16bits of  Security Parameter Index
        unsigned short     spiLs;
                       // Most significant 16bits of  Sequence number
        unsigned short     seqNumMs;
                       // Least significant 16bits of  Sequence number
        unsigned short     seqNumLs;
      }ipv4;
      struct
      {
        unsigned short     srcAddr[8];
        unsigned short     dstAddr[8];
        unsigned short     spiMs;
        unsigned short     spiLs;
        unsigned short     seqNumMs;
        unsigned short     seqNumLs;
      } ipv6;
    }ipv46_addr;

}apSa_t;

//RV-WB
// Important for CACHE WRITE BACK MODE
// Below definitions are created to have all entries on different cache lines
// In this case a situation when Host and AP are updating elements of different structures
// on the SAME cache line and Host is doing cache flush will be resolved properly
// WITHOUT AP portion data loss.

#ifdef AP_ASSEMBLER
    #define SA_PADDING_TILL_CLINE	(CACHE_LINE_SIZE - (sizeof typedef apSa_t & (CACHE_LINE_SIZE - 1)))
#else
    #define SA_PADDING_TILL_CLINE	(CACHE_LINE_SIZE - (sizeof(apSa_t) & (CACHE_LINE_SIZE - 1)))
#endif

typedef struct apSa_s_chaligned
{
	apSa_t	apSaMain;
	unsigned char   reserved[SA_PADDING_TILL_CLINE];
} apSa_t_chaligned;

#ifdef AP_ASSEMBLER
assert (sizeof typedef apSa_t_chaligned & (CACHE_LINE_SIZE - 1)), 0
#else
// will do it later
//assert (0 == (sizeof(apSa_t_chaligned) & (CACHE_LINE_SIZE - 1)));
#endif

typedef struct pktinfot
{
                         // Most significant 16 bits of IP packet buffer ptr
                         // incase of theh fragmented packets this will be
                         // first buffer of the cchained buffers.
    unsigned short       srcAddrMs;

                         // least significant 16 bits of IP packet buffer ptr
    unsigned short       srcAddrLs;

    unsigned short       crypto;   // holds the address of apSA_t.crypto

                         // pointer to the SA in the DMEM. Using the portID
                         // (portid = SA pointer in the SDRAM) SA is read from
                         // AP DMEM.
    unsigned short       saPtr;

                         // pointer to Pre Header in the DMEM. Incase of the
                         // frgmented packets this is the pre-header part of
                         // the first buffer.
    unsigned short       preHdrPtr;            // ptr to the pre-header buffer.

                         // Packet is read into DMEM  in chunks of 128 byte
                         // blocks. src_offset holds the offset to the next
                         // block with in the packet.
                         // the data i
    unsigned short       src_offset;

                         // Data iis processed in blocks of 128 bytes.
                         // dst_offset used to hold the offset where the pro-
                         // cessed block will be written.
    unsigned short       dst_offset;

                         // total no of btes remained to process in the packet.
    unsigned short       remLen;

                         // This holds the no of bytes left to process with int
                         // the buffer. Incase if the packet is not frgmented
                         //  bufremLen = remLen
    unsigned short       bufRemLen;

                         // tunBufPtr is used to store the encapsulted data.
    unsigned short       tunBufPtr;

    unsigned short       tunbuf_len;

                         // When the packets are coming back to back, the engine
                         // while processing the Nth packet, does pre-processing
                         // for N+1 packet. This pointer holds the pointer from
                         // where the process will start after the Nth packet
                         // processing.
    unsigned short       jmpStartPtr;

                         // MSB 16bit of the next port address
    unsigned short       exitMs;

                         // MSB 16bit of the next port address
    unsigned short       exitLs;

                         // MSB 16bits of the source buffer base address
    unsigned short       srcBaseMs;
                         // MSB 16bits of the source buffer base address
    unsigned short       srcBaseLs;

                         // MSB 16bits of the dest buffer base address
    unsigned short       dstBaseMs;
                         // MSB 16bits of the dest buffer base address
    unsigned short       dstBaseLs;

                         // Initialization Vector length
    unsigned char        ivlen;

                         // IP header length
    unsigned char        iphdrlen;

    unsigned char	 busyflag;
    unsigned char	 res[1];

}pktInfo_t;

typedef struct blockt
{
    unsigned char         dummy[48];
    unsigned char         data[BLOCK_SPACE];
    unsigned char         offset;
    unsigned char         dataLen;
    unsigned short        dummy1;
}block_t;

typedef struct blklist
{
//    unsigned short        current;
    unsigned short        currentBlk;
    unsigned short        nextBlk;
    unsigned short        free;
    unsigned short        temp;
}blkList_t;


typedef struct apsecstatst
{
    unsigned int    totRcvd;
    unsigned int    totTxd;
    unsigned int    totDropped;

    unsigned int    espOutRcvd;
    unsigned int    espOutTxd;
    unsigned int    espOutDropped;

    unsigned int    espInRcvd;
    unsigned int    espInTxd;
    unsigned int    espInDropped;

    unsigned int    ahOutRcvd;
    unsigned int    ahOutTxd;
    unsigned int    ahOutDropped;

    unsigned int    ahInRcvd;
    unsigned int    ahInTxd;
    unsigned int    ahInDropped;

    unsigned int    cryptoRcvd;
    unsigned int    cryptoTxd;
    unsigned int    cryptoInDropped;
}apSecStats_t;

/*****************************************************************************/
/* Function Prototypes                                                       */
/*****************************************************************************/

#ifndef AP_ASSEMBLER

void display(apSa_t *);
int getCenHash(apSa_t *, char *, unsigned short );
int checkPktSecAp(apSa_t *);
int setCryptoCtlBlk( apSa_t    * ,
                     unsigned char ,
                     unsigned char ,
                     unsigned char ,
                     unsigned char ,
                     unsigned char *,
                     unsigned char *
                   );
//void  freeAPSA(apSa_t *pSA);
void sendPktSecAp(apSa_t *pSa, char *data, int len);
void secApLibDriverInit(void);
void printPkt(char *pBuf, unsigned short len); /* in crypto_slow_path.c */
int addSecSAsToLocalDataBase(apSa_t *pSA);
int censet_eAlgo( unsigned char encrAlgo,
                  unsigned char encr,
                  unsigned short *outEncr);
int censet_aAlgo( unsigned char algo, unsigned short * temp, unsigned short * temp1);
int setCENKey(unsigned char algo, unsigned char*pInKey, unsigned char*pCenKey);

/*int apsaLibInit(unsigned int numSAs);
void  freeAllAPSAs(void);
void* newAPSA(void);
int setAPSA(
            unsigned char   ipversion,
            unsigned int    spi,
            unsigned char*  srcIp,
            unsigned char*  dstIp,
            unsigned int    seqNum,
            unsigned int    direction,
            unsigned char   proto,
            unsigned char   espOpt,      // auth & encry/auth only/encr only
            unsigned char   encrAlgo,
            unsigned char   keySize,     // only for AES
            unsigned char * pEncrkey,
            unsigned char   authAlgo,
            unsigned char * pAuthKey,
            apSa_t        * pSa         // pointer to SA
           );
apSa_t * getAPSA(
            unsigned char   ipversion,
            unsigned int    spi,
            unsigned char*  srcIp,
            unsigned char*  dstIp,
            unsigned int    seqNum,
            unsigned int    direction,
            unsigned char   proto,
            unsigned char   espOpt,      // auth & encry/auth only/encr only
            unsigned char   encrAlgo,
            unsigned char   keySize,     // only for AES
            unsigned char * pEncrkey,
            unsigned char   authAlgo,
            unsigned char * pAuthKey
           );*/


#endif // !(AP_ASSEMBLER)

#endif
