/* =====================================================================
 * Copyright (C) 2006 Ikanos Communications 
 * See IKANOS_PROP_LICENSE.txt for license information.
 * =====================================================================
 */

/* =====================================================================
 * File name    : $RCSfile: apbm_vx180.h,v $
 * Description  : AHB and port information for the Vox200 Ethernet AP
 * =====================================================================
 */

#ifndef _APBM_VX180_H_
#define _APBM_VX180_H_


#define AD6834_DEVICE_ID        0x6834          // Vox200
#define AD6833_DEVICE_ID        0x6833          // Vox150
#define AD6836_DEVICE_ID        0x6836          // Vx160
#define AD6850_DEVICE_ID        0x6850          // Vx180
#define AD6502_DEVICE_ID        0x6502          // AT200
#define AD6503_DEVICE_ID        0x6503          // AT300
#define AD6501_DEVICE_ID        0x6501          // AT100
#define AD65XX_DEVICE_ID        0x6500          // AT200/AT300/AT100

// Define the "fast" memory. This is intended to simulate output queues and
// internal memories
#define FAST_RAM_ADDR           0x191f0000      // Base address
#define FAST_RAM_SIZE_AD6834    1 * (1 << 15)   // Size for Vox200
#define FAST_RAM_SIZE_AD6833    1 * (1 << 12)   // Size for Vox150
#define FAST_RAM_SIZE_AD6836    1 * (1 << 11)   // Size for Vx160
#define FAST_RAM_SIZE_AD6850    1 * (1 << 15)   // Size for Vx180   32K
#define FAST_RAM_SIZE_AD65xx    1 * (1 << 12)   // Size for AT200
#define FAST_RAM_FIRST          1               // AHB CLK to first access
#define FAST_RAM_SUCC           1               // AHB clk to successive access

// Define the "slow" memory. This is intended to simulate system SDRAM
#define SLOW_RAM_ADDR           0x20000000      // Base address
#define SLOW_RAM_SIZE           1 * (1 << 20)   // Size
#define SLOW_RAM_RD_FIRST       10              // AHB CLK to first access
#define SLOW_RAM_RD_SUCC        1               // AHB clk to successive access
#define SLOW_RAM_WR_FIRST       10              // AHB CLK to first access
#define SLOW_RAM_WR_SUCC        2               // AHB clk to successive access

// Define BM O/Q & I/Q
#define BM_INQ1_ADDR            0x19210510     // INQ1 Fifo Addr
#define BM_INQ2_ADDR            0x19210610     // INQ2 Fifo Addr
#define BM_INQ3_ADDR            0x19210710     // INQ2 Fifo Addr
#define BM_OPQ1_ADDR            0x19210810     // OPQ1 Fifo Addr
#define BM_OPQ2_ADDR            0x19210910     // OPQ2 Fifo Addr

// Define SPA input queue
#define SPA_INQ1_ADDR           0x19100510       // INQ for SPA

// Define the location (AHB and I/O port) for the test input queue
#define INPQ_ADDR               0x30000000      // AHB address of queue

// Define the location (AHB and I/O port) for the test output queue
#define OUTQ_ADDR               0x30010000      // AHB address of queue

// Globally allocated places.

#define AHB_BUF_ALLOC_BASE      (BM_OPQ1_ADDR)  // New buffer list BM OPQ
#define AHB_BUF_FREE_BASE       (BM_INQ1_ADDR)  // Used buffer list
#define AHB_SECURITY_BASE       (SPA_INQ1_ADDR) // SPA input queue
#define AHB_INPUT_QUEUE_ADDR    (INPQ_ADDR)     // Location in input queue

#define AHB_TXQUEUE_BASE        (SLOW_RAM_ADDR + 0x4000) // Tx Queue location
#define AHB_TX_QUEUE_ENTRIES    256

#define AHB_TABLE_BASE          (SLOW_RAM_ADDR + 0x1000) // Flow descriptor entries

#define AHB_BUFFER              (SLOW_RAM_ADDR + 0x2000) // Buffer space
#define AHB_DUMMY               (SLOW_RAM_ADDR + 0x8000) // Dummy load accesses

#define BM_BUFFER_OFFSET        0x10000
#define BM_BUFFER_BASE          (SLOW_RAM_ADDR + BM_BUFFER_OFFSET)

#define RAMQ_BASE               				(BM_BUFFER_BASE + 0)
#define RAMQ_SIZE               				512
#define RAMQ_LOW_SPARE_THRESHOLD                160
#define RAMQ_HIGH_SPARE_THRESHOLD				416

#define APUQ_BASE               				(FAST_RAM_ADDR + 0x100)
#define NUM_RECORDS_IN_BLOCK					16
#define NUM_APUQ_RECORDS        				0x80
#define APUQ_SIZE               				(0x80 * 4)
#define APUQ_LOW_SPARE_THRESHOLD				0x20

#define NO_FREE_SPACE_ATRAMQ					1 << 10
#define LOT_POINTERS_ATRAMQ                     1<< 9   // malati 1 << 11
#define SHORTAGE_POINTERS_ATRAMQ                1 << 12
#define SHORTAGE_POINTERS_ATBM                  1 << 13
#define AP_EVT_UART_BIT                         4
#define UART2_INTRPT_VAL                        1 << AP_EVT_UART_BIT
// DMA Channels definitions
/* Definitions for the utility task */
#define UTIL_DMA        0       // Utility process DMA channel

/* Definitions for the Buffer Management task */
#define BM_DMA          1       // Buffer Management DMA channel

/* Definitions for the classification task */
#define CLASS_DMA       2       // DMA channel for classifier
#if defined(BM_SPECIAL_STATS) && (BM_SPECIAL_STATS)
#define NUM_SUPPORTED_ROUTED_STATS				2
#if (NUM_SUPPORTED_ROUTED_STATS > 4)
Number of routed supported STATS CAN NOT BE MORE THAN 4
#endif
#define AP_PPPOE_SESSIONID_UPDATE_EVENT_BIT		12
#define AP_PPPOE_SESSIONID_UPDATE_EVENT_VAL		(1 << 12)
#endif

	// Define the statistics structure
	typedef struct
	{
		unsigned long	numTakenFromInputQ;
		unsigned long	numPutToOutputQ;
		unsigned long   numToClassify;
		unsigned long 	numEthClassified;
                unsigned short	numDMAsramQ2DMem;
		unsigned short	numDMAsDMem2ramQ;

                unsigned long   numFreed2BM;
                unsigned long   numput2IPCQ;
                unsigned long   numput2APs;

                unsigned long   numPutToOutputQ2;
		unsigned long	numTakenFromInputQ3;

#if defined(BM_SPECIAL_STATS) && (BM_SPECIAL_STATS)
		unsigned int    specialRxPackets[NUM_SUPPORTED_ROUTED_STATS];  /* Counts rx packets for differrent MACs */
		unsigned int    specialRxBytes[NUM_SUPPORTED_ROUTED_STATS];    /* Counts rx bytes for differrent MACs */
#endif
	} apBMStatsInfo_s;


#endif /* _APBM_VX180_H_ */
