/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : classifierhw.h                                          		*
 * Author       : Ikanos Communications  [ jpattar@ikanos.com ]                         *
 * Version      :                                                                       *
 * Date         : 30th Nov 2009                                                         *
 * Description  : This file contains the circular buffer register offsets.              *
 *                                                                                      *
 ****************************************************************************************/

/*_____________________________________Header Gaurd_____________________________________*/

#ifndef _CLASSIFIERHW_H_
#define _CLASSIFIERHW_H_

/*_______________________________________Includes_______________________________________*/



/*________________________________________MACROS________________________________________*/

/* 
 * Global Registers:
 * These register are accessible by both APs. Assumed that both AP will no try to write 
 * at same time. If that happens AP1 write will be discarded.
 */

#define	CL_BASE_ADDR			0x00

#define	CL_HDR_SIZE_REG			0x80	//Header size register
#define	CL_AHB_MASTER_CONTROL_REG	0x81	//AHB Master control register
#define	CL_APU0_APU1_MAILBOX_REG	0x82	//Written by AP0, readable by both APs.
#define	CL_APU1_APU0_MAILBOX_REG	0x83	//Written by AP1, readable by both APs.
#define	CL_APU0_WAIT_COUNTER_REG	0x84	//Debug register
#define CL_APU1_WAIT_COUNTER_REG        0x85    //Debug register



#define CL_AHB_RETRY_WAIT_TIME_BIT0	0
#define CL_AHB_RETRY_WAIT_TIME_BIT1	1
#define CL_AHB_RETRY_WAIT_TIME_BIT2	2
#define CL_AHB_RETRY_WAIT_TIME_BIT3	3
//BIT 4,5,6, and 7 are reserved
#define	CL_AHB_BURST_16_BIT		8
#define	CL_AHB_INCR_BURST_BIT		9
#define	CL_AHB_INCR_BURST_WLEN_BIT	10
#define CL_AHB_INCR_BURST_WLEN_EN_BIT	11

#define	SET_BURST_16_MODE		(1 << CL_AHB_BURST_16_BIT)


/*
 * Multiplier registers.
 */
#define	CL_MULT_OPRND			0x53
#define	CL_MULT_RESULT			0x54
#define CL_CBUS_MULT_OPRND              0x03AC
#define CL_CBUS_MULT_RESULT             0x03B0


/* 
 * Smart DMA registers 
 * There is one set of register for each AP. AP has access only to its set of registers.
 */

#define	CL_SDMA_CONTROL_REG		0x88	//Control register
#define	CL_SDMA_DSC_ADDR_REG		0x89	//SDMA descriptor address register


#define	CL_SDMA_START_BIT		0
#define	CL_SDMA_BUSY_BIT		1
#define	CL_SDMA_CMPERR_BIT		2

#define	SDMA_START			(1 << CL_SDMA_START_BIT )
/* READ ONLY REGS 
#define	SDMA_BUSY			(1 << CL_SDMA_BUSY_BIT )
#define	SDMA_CMPERR			(1 << CL_SDMA_CMPERR_BIT)
*/

/*
 * CRC registers
 * Following registers are used to configure CRC engine.
 */
#define CL_CRC_BYTE_EN_REG 		0x8A
#define CL_CRC_DATA_LO_REG     		0x8B
#define CL_CRC_DATA_HI_REG  		0x8C
#define CL_CRC_VALUE_REG                0x8D

#define CL_PRG_CRC_P0                   CL_CRC_VALUE_REG
#define CL_PRG_CRCBE_P0                 CL_CRC_BYTE_EN_REG
#define CL_PRG_CRCDATA_P0               CL_CRC_DATA_LO_REG
#define CL_PRG_CRCDATA_P1               CL_CRC_DATA_HI_REG


#define	_CL_ADDR(reg) 			( CL_BASE_ADDR + ( CL_ ## reg ) )


/*
 * Circular buffer registers
 * There is one set of register for each logical Rx Port; There are 6 logical Rx Ports.
 * Port{n} Base: 0x90 + (n * 0x8), n varies from 0 to 5.
 */

#define CL_CB_BASE_ADR                  0x90

#define	CL_CB_PRT_START_ADR_REG		0x0
#define	CL_CB_PRT_END_ADR_REG		0x1
#define	CL_CB_HDR_DONE_REG		0x2
#define	CL_CB_WR_PTR_REG		0x3
#define CL_CB_RD_PTR_REG		0x4

/* Use following macro to configure Circular buffer.
 * reg1 should be PORT0 to PORT5, and 
 * reg2 should be of of circular buffer */
#define	_CL_CB_ADDR(reg1, reg2)		( CL_CB_BASE_ADR + \
					( (CL_CB_ ## reg1) * 0x8) + (CL_CB_ ## reg2))

/* 
 * The following function get you respective port's CB base address.
 * reg1 should contain port number, reg2 will contains CB base address and
 * aux is tempory register.
 */

#define MUL8(reg)			lsl	reg	\
					lsl	reg	\
					lsl	reg
/*TODO: replace MUL8 with mul	reg2, reg1 */
#define GET_CB_BASE_ADDR(reg1, aux)		MUL8(reg1)             		\
						lbl	aux, CL_CB_BASE_ADR	\
						add	reg1, aux

/* 
 * Use following macro to get the apClassInp_t structure in selected CB.
 * reg1 should be base address of respective CB. Result will be in reg1 it self.
 * The + 8 is hole1 size
 */
#define GET_CB_RD_PTR(reg1, aux)		add	reg1, CL_CB_RD_PTR_REG	\
						in	reg1, reg1		\
						li	aux, 0x1000		\
						add	reg1, aux		
					

#define GET_CB_WR_PTR(reg1, aux)                add     reg1, CL_CB_WR_PTR_REG  \
                                                in      reg1, reg1              \
						li      aux, 0x1000             \
						add     reg1, aux
        

//accepts port base register address (reg1) and one auxillary reg.
#define CB_HDR_DONE_INDICATION(reg1, aux)	lbl	aux, 1				\
						add     reg1, CL_CB_HDR_DONE_REG  	\
						out     aux, reg1            		\
                                                nop
						

#define	DMEM_PIO_START_ADDR		0x0
#define	CB_PIO_START_ADDR		0x1000
#define	FT_PIO_START_ADDR		0x4000


/* Temporary, following register act as input FLAG register. */
//#define	CL_INFLAG_REG			0xA4

#define	CL_INFLAG_APU1_APU0_MAIL_BIT	15
#define CL_INFLAG_APU0_APU1_MAIL_BIT    14
#define	CL_INFLAG_PORT5_HDRAVIAL_BIT	13
#define CL_INFLAG_PORT4_HDRAVIAL_BIT    12
#define CL_INFLAG_PORT3_HDRAVIAL_BIT    11
#define CL_INFLAG_PORT2_HDRAVIAL_BIT    10
#define CL_INFLAG_PORT1_HDRAVIAL_BIT    9
#define CL_INFLAG_PORT0_HDRAVIAL_BIT    8

/*
 * Initialize 0x368 register, so that proper arbitration between CLAP1
 * CLAP2 toward LMEM happens properly. This is used only from MIPS.
 */
#define	CLAP_LMEM_ARBITRATION_REG368	0x368


/*_____________________________________Header Gaurd_____________________________________*/
#endif //_CLASSIFIERHW_H_
