/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : descriptorslist.h                                            		*
 * Author       : Ikanos  [ vrevzin@ikanos.com ]                                   	*
 * Version      :                                                                       *
 * Date         : 09th Nov 2010                                                         *
 * Description  : ??????This file contains the structure of parser output           	*
 *                to store packet information. It also contain information to carry     *
 *                to transmiting interface.                                             *
 *                                                                                      *
 ****************************************************************************************/

/*_____________________________________Header Gaurd_____________________________________*/
#ifndef _DESCRIPTORS_FIELDS_DEFINITIONS_H_
#define _DESCRIPTORS_FIELDS_DEFINITIONS_H_

#define DESC_DEF_SDRAM_PKT_START_OFF                        (PARSER_HOLE1 + APPREHEADER_SIZE)
#define DESC_DEF_SDMA_STATUS                                0x00000000 //fill by SDMA after executing the descriptor
#define DESC_DEF_TEMP_CB_BASE_ADDR                          0x1000 //replace by SDMA task with actual CB port address before executing the descriptor
#define DESC_DEF_TEMP_SDRAM_BASE_ADDR                       0x00001000 //replace by SDMA task with actual cluster address before executing the descriptor
#define DESC_DEF_TEMP_FT_BASE_ADDR                          0x4100 //replace by SDMA task with actual rft/bft address before executing the descriptor
#define DESC_DEF_TEMP_APDMEM_BASE_ADDR                      0x0000 //replace by SDMA task with actual dmem apPreHdr address before executing the descriptor

#define DESC_DEF_APPREHDR_SIZE                              APPREHEADER_SIZE  
#define DESC_DEF_APPREHDR_START_OFF                         0x00
#define DESC_DEF_CB_START_OFF                               0x00
#define DESC_DEF_SDRAM_CLUSTER_START_OFF                    0x00


#define DESC_DEF_CB_PKT_HDR_SIZE                            0x80//PACKET_HEADER_SIZE
#define DESC_DEF_ETH_HDR_MACADDRS_SIZE                      0x0c //Contains DEST and SRC MAC address size 6+6; or offset to ETH_TYPE in ETH header             
#define DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET                    DESC_DEF_ETH_HDR_MACADDRS_SIZE        
#define DESC_DEF_ETH_HDR_VLAN_HDR_OFF                       0x0e //in vlan pkt        
#define DESC_DEF_PLAIN_PKT_IPHDR_OFF                        0x0e // IP header offset in plain ipv4 pkt ; no extra header other than eth; i.e no vlan/pppoe etc.        
#define DESC_DEF_PLAIN_PKT_l3SRCIP_OFF                      0x1a // src IP offset in plain ipv4 pkt ; no extra header other than eth; i.e no vlan/pppoe etc.        
#define DESC_DEF_PLAIN_PKT_l3DESTIP_OFF                     0x1e // Dest IP offset in plain ipv4 pkt ; no extra header other than eth; i.e no vlan/pppoe etc.        
#define DESC_DEF_PLAIN_PKT_l4HDR_OFF                        0x22 // l4 header offset in plain ipv4 pkt ; no extra header other than eth; i.e no vlan/pppoe etc.        
#define DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF                  0x22 // l4 src port offset in plain ipv4 pkt ; no extra header other than eth; i.e no vlan/pppoe etc.        
#define DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF                 0x24 // l4 src port offset in plain ipv4 pkt ; no extra header other than eth; i.e no vlan/pppoe etc.        

#define DESC_DEF_IPADDR_SIZE                                4
#define DESC_DEF_IPPORT_SIZE                                2
#define DESC_DEF_ESP_SPI_SIZE								4		//dekumar: IPSec passthru
#define DESC_DEF_VLAN_ID_SIZE                               0x2
#define DESC_DEF_VLAN_HDR_SIZE                              0x4
#define DESC_DEF_PPPOE_HDR_SIZE                             8
#define DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE                (DESC_DEF_PPPOE_HDR_SIZE - DESC_DEF_VLAN_HDR_SIZE) //4
#define DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE                (DESC_DEF_PPPOE_HDR_SIZE + DESC_DEF_VLAN_HDR_SIZE) //0x0c
#define	DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS	    20
#define	DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	    40

#define DESC_DEF_ETH_TYPE_IP                                 0x0800
#define DESC_DEF_ETH_TYPE_IPV6                               0x86DD
#define DESC_DEF_ETH_TYPE_SIZE                               2
#define DESC_DEF_ETH_TYPE_8021Q                              0x8100
#define DESC_DEF_DESC_DELIMITER                              0xffffffff
#define DESC_DEF_DESC_DUMMY                                  0x00000000

// IPv6/IPv4 Tunneling/Translation related macros

// IPV6 (plain) to IPV4 (plain) Tunnel - encapsulation macros
#define	PLAIN_2_PLAIN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PLAIN_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PLAIN_2_PLAIN_V4_TUNNEL_SDRAM_ADJ)
#define	PLAIN_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(PLAIN_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_2_PLAIN_V4_TUNNEL_IP_SDRAM_OFF		(PLAIN_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_2_PLAIN_V4_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_ETH_TYPE_SIZE)

// IPV6 (pppoe) to IPV4 (plain) Tunnel - encapsulation macros
#define	PPPOE_2_PLAIN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS - DESC_DEF_PPPOE_HDR_SIZE)
#define	PPPOE_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PPPOE_2_PLAIN_V4_TUNNEL_SDRAM_ADJ)
#define	PPPOE_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(PPPOE_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_2_PLAIN_V4_TUNNEL_IP_SDRAM_OFF		(PPPOE_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_2_PLAIN_V4_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (plain) Tunnel to  IPV6 (plain) - decapsulation macros
#define	PLAIN_V4_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PLAIN_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PLAIN_2_PLAIN_V4_TUNNEL_SDRAM_ADJ)
#define	PLAIN_V4_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(PLAIN_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_V4_TUNNEL_2_PLAIN_IP_SDRAM_OFF		(PLAIN_V4_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_V4_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)		

// IPV4 (pppoe) Tunnel to  IPV6 (plain) - decapsulation macros
#define	PPPOE_V4_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS	\
							+ DESC_DEF_PPPOE_HDR_SIZE)
#define	PPPOE_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PPPOE_V4_TUNNEL_2_PLAIN_SDRAM_ADJ)
#define	PPPOE_V4_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(PPPOE_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_V4_TUNNEL_2_PLAIN_IP_SDRAM_OFF		(PPPOE_V4_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_V4_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE               \
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)		

// IPV6 (plain) to IPV4 (plain) Translation macros
#define	PLAIN_2_PLAIN_V4_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							- DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PLAIN_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PLAIN_2_PLAIN_V4_TRANS_SDRAM_ADJ)
#define PLAIN_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF	(PLAIN_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_2_PLAIN_V4_TRANS_IP_SDRAM_OFF		(PLAIN_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_2_PLAIN_V4_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)		

// IPV6 (pppoe) to IPV4 (plain) Translation macros
#define	PPPOE_2_PLAIN_V4_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							- DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS	\
							+ DESC_DEF_PPPOE_HDR_SIZE)
#define	PPPOE_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PPPOE_2_PLAIN_V4_TRANS_SDRAM_ADJ)
#define PPPOE_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF	(PPPOE_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_2_PLAIN_V4_TRANS_IP_SDRAM_OFF		(PPPOE_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_2_PLAIN_V4_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)		

// IPV4 (plain) to IPV6 (plain) Tunneling - encapsulation macros
#define	PLAIN_2_PLAIN_V6_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)
#define	PLAIN_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PLAIN_2_PLAIN_V6_TUNNEL_SDRAM_ADJ)
#define	PLAIN_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF	(PLAIN_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_2_PLAIN_V6_TUNNEL_IP_SDRAM_OFF		(PLAIN_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_2_PLAIN_V6_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (pppoe) to IPV6 (plain) Tunneling - encapsulation macros
#define	PPPOE_2_PLAIN_V6_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_PPPOE_HDR_SIZE)
#define	PPPOE_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PPPOE_2_PLAIN_V6_TUNNEL_SDRAM_ADJ)
#define	PPPOE_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF	(PPPOE_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_2_PLAIN_V6_TUNNEL_IP_SDRAM_OFF		(PPPOE_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_2_PLAIN_V6_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+DESC_DEF_ETH_TYPE_SIZE)

// IPV6 (plain) Tunnel to IPV4 (plain) - decapsulation macros
#define	PLAIN_V6_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)
#define	PLAIN_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PLAIN_V6_TUNNEL_2_PLAIN_SDRAM_ADJ)
#define	PLAIN_V6_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(PLAIN_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_V6_TUNNEL_2_PLAIN_IP_SDRAM_OFF		(PLAIN_V6_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_V6_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)		

// IPV6 (pppoe) Tunnel to IPV4 (plain) - decapsulation macros
#define	PPPOE_V6_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS		\
							+ DESC_DEF_PPPOE_HDR_SIZE)
#define	PPPOE_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PPPOE_V6_TUNNEL_2_PLAIN_SDRAM_ADJ)
#define	PPPOE_V6_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(PPPOE_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_V6_TUNNEL_2_PLAIN_IP_SDRAM_OFF		(PPPOE_V6_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_V6_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)		
// IPV4 (plain) to IPV6 (plain) Translation
#define	PLAIN_2_PLAIN_V6_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							- DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PLAIN_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PLAIN_2_PLAIN_V6_TRANS_SDRAM_ADJ)
#define PLAIN_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF	(PLAIN_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_2_PLAIN_V6_TRANS_IP_SDRAM_OFF		(PLAIN_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_2_PLAIN_V6_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)		

// IPV4 (pppoe) to IPV6 (plain) Translation
#define	PPPOE_2_PLAIN_V6_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							- DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS	\
							- DESC_DEF_PPPOE_HDR_SIZE)
#define	PPPOE_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PPPOE_2_PLAIN_V6_TRANS_SDRAM_ADJ)
#define PPPOE_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF	(PPPOE_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_2_PLAIN_V6_TRANS_IP_SDRAM_OFF		(PPPOE_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_2_PLAIN_V6_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)		

// IPV4 (plain) to IPV6 (vlan) Tunneling - encapsulation macros
#define	PLAIN_2_VLAN_V6_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS 	\
							+ DESC_DEF_VLAN_HDR_SIZE)

#define PLAIN_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PLAIN_2_VLAN_V6_TUNNEL_SDRAM_ADJ)

#define PLAIN_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF		(PLAIN_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)	

#define PLAIN_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF	(PLAIN_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)

#define PLAIN_2_VLAN_V6_TUNNEL_IP_SDRAM_OFF		(PLAIN_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)

#define	PLAIN_2_VLAN_V6_TUNNEL_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (pppoe) to IPV6 (vlan) Tunneling - encapsulation macros
#define	PPPOE_2_VLAN_V6_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS 	\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							- DESC_DEF_PPPOE_HDR_SIZE)

#define PPPOE_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PPPOE_2_VLAN_V6_TUNNEL_SDRAM_ADJ)

#define PPPOE_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF		(PPPOE_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)	

#define PPPOE_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF	(PPPOE_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)

#define PPPOE_2_VLAN_V6_TUNNEL_IP_SDRAM_OFF		(PPPOE_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)

#define	PPPOE_2_VLAN_V6_TUNNEL_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)

// IPV6 (vlan) Tunnel to IPV4 (plain) - decapsulation macros
#define	VLAN_V6_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS+DESC_DEF_VLAN_HDR_SIZE)
#define VLAN_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VLAN_V6_TUNNEL_2_PLAIN_SDRAM_ADJ)

#define VLAN_V6_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(VLAN_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)

#define	VLAN_V6_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (vlan+pppoe) Tunnel to IPV4 (plain) - decapsulation macros
#define	VPPPOE_V6_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE)
#define VPPPOE_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VPPPOE_V6_TUNNEL_2_PLAIN_SDRAM_ADJ)

#define VPPPOE_V6_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(VPPPOE_V6_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)

#define	VPPPOE_V6_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV4 (vlan) to IPV6 (plain) Tunneling - encapsulation macros
#define	VLAN_2_PLAIN_V6_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS 	\
							- DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VLAN_2_PLAIN_V6_TUNNEL_SDRAM_ADJ)

#define	VLAN_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF	(VLAN_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_PLAIN_V6_TUNNEL_IP_SDRAM_OFF		(VLAN_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_2_PLAIN_V6_TUNNEL_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (vlan+pppoe) to IPV6 (plain) Tunneling - encapsulation macros
#define	VPPPOE_2_PLAIN_V6_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS 	\
							- DESC_DEF_VLAN_HDR_SIZE			\
							- DESC_DEF_PPPOE_HDR_SIZE)
#define	VPPPOE_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VPPPOE_2_PLAIN_V6_TUNNEL_SDRAM_ADJ)

#define	VPPPOE_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF	(VPPPOE_2_PLAIN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VPPPOE_2_PLAIN_V6_TUNNEL_IP_SDRAM_OFF		(VPPPOE_2_PLAIN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VPPPOE_2_PLAIN_V6_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_VLAN_HDR_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)

// IPV6 (plain) Tunnel to IPV4 (vlan) - decapsulation macros
#define	PLAIN_V6_TUNNEL_2_VLAN_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS-DESC_DEF_VLAN_HDR_SIZE)
#define	PLAIN_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PLAIN_V6_TUNNEL_2_VLAN_SDRAM_ADJ)
#define	PLAIN_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF		(PLAIN_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define PLAIN_V6_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF	(PLAIN_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PLAIN_V6_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							 + DESC_DEF_ETH_TYPE_SIZE			\
							 + DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (pppoe) Tunnel to IPV4 (vlan) - decapsulation macros
#define	PPPOE_V6_TUNNEL_2_VLAN_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							-DESC_DEF_VLAN_HDR_SIZE)
#define	PPPOE_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PPPOE_V6_TUNNEL_2_VLAN_SDRAM_ADJ)
#define	PPPOE_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF		(PPPOE_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define PPPOE_V6_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF	(PPPOE_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PPPOE_V6_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							 + DESC_DEF_PPPOE_HDR_SIZE			\
							 + DESC_DEF_ETH_TYPE_SIZE			\
							 + DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV4 (vlan) to IPV6 (vlan) Tunneling - encapsulation macros
#define VLAN_2_VLAN_V6_TUNNEL_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)
#define	VLAN_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF - VLAN_2_VLAN_V6_TUNNEL_SDRAM_ADJ)

#define	VLAN_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF		(VLAN_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF		(VLAN_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_2_VLAN_V6_TUNNEL_IP_SDRAM_OFF		(VLAN_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_2_VLAN_V6_TUNNEL_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (vlan+pppoe) to IPV6 (vlan) Tunneling - encapsulation macros
#define VPPPOE_2_VLAN_V6_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							- DESC_DEF_PPPOE_HDR_SIZE)
#define	VPPPOE_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VPPPOE_2_VLAN_V6_TUNNEL_SDRAM_ADJ)

#define	VPPPOE_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF		(VPPPOE_2_VLAN_V6_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VPPPOE_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF	(VPPPOE_2_VLAN_V6_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VPPPOE_2_VLAN_V6_TUNNEL_IP_SDRAM_OFF		(VPPPOE_2_VLAN_V6_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VPPPOE_2_VLAN_V6_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
							+ DESC_DEF_VLAN_HDR_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)

// IPV6 (vlan) Tunnel to IPV4 (vlan) - decapsulation macros
#define	VLAN_V6_TUNNEL_2_VLAN_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)
#define	VLAN_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF + VLAN_V6_TUNNEL_2_VLAN_SDRAM_ADJ)
#define VLAN_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF		(VLAN_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define VLAN_V6_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF		(VLAN_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_V6_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (vlan+pppoe) Tunnel to IPV4 (vlan) - decapsulation macros
#define	VPPPOE_V6_TUNNEL_2_VLAN_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							+ DESC_DEF_PPPOE_HDR_SIZE)
#define	VPPPOE_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VPPPOE_V6_TUNNEL_2_VLAN_SDRAM_ADJ)
#define VPPPOE_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF		(VPPPOE_V6_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define VPPPOE_V6_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF	(VPPPOE_V6_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VPPPOE_V6_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV4 (plain) to IPV6 (vlan) Translation macros
#define	PLAIN_2_VLAN_V6_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 + DESC_DEF_VLAN_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PLAIN_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF - PLAIN_2_VLAN_V6_TRANS_SDRAM_ADJ)
#define	PLAIN_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF		(PLAIN_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF		(PLAIN_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PLAIN_2_VLAN_V6_TRANS_IP_SDRAM_OFF		(PLAIN_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_2_VLAN_V6_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV4 (pppoe) to IPV6 (vlan) Translation macros
#define	PPPOE_2_VLAN_V6_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 + DESC_DEF_VLAN_HDR_SIZE			\
							 - DESC_DEF_PPPOE_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PPPOE_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF - PPPOE_2_VLAN_V6_TRANS_SDRAM_ADJ)
#define	PPPOE_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF		(PPPOE_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF		(PPPOE_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PPPOE_2_VLAN_V6_TRANS_IP_SDRAM_OFF		(PPPOE_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_2_VLAN_V6_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV4 (vlan) to IPV6 (plain) Translation macros
#define	VLAN_2_PLAIN_V6_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_VLAN_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VLAN_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF - VLAN_2_PLAIN_V6_TRANS_SDRAM_ADJ)
#define	VLAN_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF		(VLAN_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_PLAIN_V6_TRANS_IP_SDRAM_OFF		(VLAN_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_2_PLAIN_V6_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
// IPV4 (vlan+pppoe) to IPV6 (plain) Translation macros
#define	VPPPOE_2_PLAIN_V6_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_VLAN_HDR_SIZE			\
							 - DESC_DEF_PPPOE_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VPPPOE_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VPPPOE_2_PLAIN_V6_TRANS_SDRAM_ADJ)
#define	VPPPOE_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF	(VPPPOE_2_PLAIN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VPPPOE_2_PLAIN_V6_TRANS_IP_SDRAM_OFF		(VPPPOE_2_PLAIN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VPPPOE_2_PLAIN_V6_TRANS_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV4 (vlan) to IPV6 (vlan) Translation macros
#define	VLAN_2_VLAN_V6_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VLAN_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF - VLAN_2_VLAN_V6_TRANS_SDRAM_ADJ)
#define	VLAN_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF		(VLAN_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF		(VLAN_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_2_VLAN_V6_TRANS_IP_SDRAM_OFF		(VLAN_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_2_VLAN_V6_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV4 (vlan+pppoe) to IPV6 (vlan) Translation macros
#define	VPPPOE_2_VLAN_V6_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_PPPOE_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VPPPOE_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VPPPOE_2_VLAN_V6_TRANS_SDRAM_ADJ)
#define	VPPPOE_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF		(VPPPOE_2_VLAN_V6_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VPPPOE_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF	(VPPPOE_2_VLAN_V6_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VPPPOE_2_VLAN_V6_TRANS_IP_SDRAM_OFF		(VPPPOE_2_VLAN_V6_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VPPPOE_2_VLAN_V6_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV6 (plain) to IPV4 (vlan) Tunneling - encapsulation macros
#define	PLAIN_2_VLAN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS + DESC_DEF_VLAN_HDR_SIZE)
#define	PLAIN_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PLAIN_2_VLAN_V4_TUNNEL_SDRAM_ADJ)
#define PLAIN_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF		(PLAIN_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define PLAIN_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(PLAIN_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PLAIN_2_VLAN_V4_TUNNEL_IP_SDRAM_OFF		(PLAIN_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define PLAIN_2_VLAN_V4_TUNNEL_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE + DESC_DEF_ETH_TYPE_SIZE)

// IPV6 (pppoe) to IPV4 (vlan) Tunneling - encapsulation macros
#define	PPPOE_2_VLAN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS 	\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							- DESC_DEF_PPPOE_HDR_SIZE)
#define	PPPOE_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - PPPOE_2_VLAN_V4_TUNNEL_SDRAM_ADJ)
#define PPPOE_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF		(PPPOE_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define PPPOE_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(PPPOE_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PPPOE_2_VLAN_V4_TUNNEL_IP_SDRAM_OFF		(PPPOE_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define PPPOE_2_VLAN_V4_TUNNEL_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE 	\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (vlan) Tunnel IPV6 (plain) - decapsulation macros
#define VLAN_V4_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS + DESC_DEF_VLAN_HDR_SIZE)
#define VLAN_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VLAN_V4_TUNNEL_2_PLAIN_SDRAM_ADJ)
#define	VLAN_V4_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(VLAN_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define VLAN_V4_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV4 (vlan+pppoe) Tunnel IPV6 (plain) - decapsulation macros
#define VPPPOE_V4_TUNNEL_2_PLAIN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS 	\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE)
#define VPPPOE_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VPPPOE_V4_TUNNEL_2_PLAIN_SDRAM_ADJ)
#define	VPPPOE_V4_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF	(VPPPOE_V4_TUNNEL_2_PLAIN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define VPPPOE_V4_TUNNEL_2_PLAIN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV6 (vlan) to IPV4 (plain) Tunneling - encapsulation macros
#define	VLAN_2_PLAIN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS - DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VLAN_2_PLAIN_V4_TUNNEL_SDRAM_ADJ)
#define VLAN_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(VLAN_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_PLAIN_V4_TUNNEL_IP_SDRAM_OFF		(VLAN_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define VLAN_2_PLAIN_V4_TUNNEL_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE 	\
							+ DESC_DEF_VLAN_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)

// IPV6 (vlan+pppoe) to IPV4 (plain) Tunneling - encapsulation macros
#define	VPPPOE_2_PLAIN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS 	\
							- DESC_DEF_PPPOE_HDR_SIZE			\
							- DESC_DEF_VLAN_HDR_SIZE)
#define	VPPPOE_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VPPPOE_2_PLAIN_V4_TUNNEL_SDRAM_ADJ)
#define VPPPOE_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(VPPPOE_2_PLAIN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VPPPOE_2_PLAIN_V4_TUNNEL_IP_SDRAM_OFF		(VPPPOE_2_PLAIN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define VPPPOE_2_PLAIN_V4_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE 	\
							+ DESC_DEF_VLAN_HDR_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (plain) Tunnel to IPV6 (vlan) - decapsulation macros
#define PLAIN_V4_TUNNEL_2_VLAN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS - DESC_DEF_VLAN_HDR_SIZE)
#define PLAIN_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PLAIN_V4_TUNNEL_2_VLAN_SDRAM_ADJ)
#define PLAIN_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF		(PLAIN_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_V4_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF	(PLAIN_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define PLAIN_V4_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV4 (pppoe) Tunnel to IPV6 (vlan) - decapsulation macros
#define PPPOE_V4_TUNNEL_2_VLAN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS 	\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							- DESC_DEF_VLAN_HDR_SIZE)
#define PPPOE_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + PPPOE_V4_TUNNEL_2_VLAN_SDRAM_ADJ)
#define PPPOE_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF		(PPPOE_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_V4_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF	(PPPOE_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define PPPOE_V4_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)

// IPV6 (plain) to IPV4 (vlan) Translation macros
#define	PLAIN_2_VLAN_V4_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_VLAN_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PLAIN_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF + PLAIN_2_VLAN_V4_TRANS_SDRAM_ADJ)
#define	PLAIN_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF		(PLAIN_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PLAIN_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF		(PLAIN_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PLAIN_2_VLAN_V4_TRANS_IP_SDRAM_OFF		(PLAIN_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PLAIN_2_VLAN_V4_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (pppoe) to IPV4 (vlan) Translation macros
#define	PPPOE_2_VLAN_V4_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_VLAN_HDR_SIZE			\
							 + DESC_DEF_PPPOE_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	PPPOE_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF + PPPOE_2_VLAN_V4_TRANS_SDRAM_ADJ)
#define	PPPOE_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF		(PPPOE_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	PPPOE_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF		(PPPOE_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	PPPOE_2_VLAN_V4_TRANS_IP_SDRAM_OFF		(PPPOE_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	PPPOE_2_VLAN_V4_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (vlan) to IPV4 (plain) Translation macros
#define	VLAN_2_PLAIN_V4_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 + DESC_DEF_VLAN_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VLAN_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF + VLAN_2_PLAIN_V4_TRANS_SDRAM_ADJ)
#define	VLAN_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF		(VLAN_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_PLAIN_V4_TRANS_IP_SDRAM_OFF		(VLAN_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_2_PLAIN_V4_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (vlan+pppoe) to IPV4 (plain) Translation macros
#define	VPPPOE_2_PLAIN_V4_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 + DESC_DEF_VLAN_HDR_SIZE			\
							 + DESC_DEF_PPPOE_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VPPPOE_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VPPPOE_2_PLAIN_V4_TRANS_SDRAM_ADJ)
#define	VPPPOE_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF	(VPPPOE_2_PLAIN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VPPPOE_2_PLAIN_V4_TRANS_IP_SDRAM_OFF		(VPPPOE_2_PLAIN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VPPPOE_2_PLAIN_V4_TRANS_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (vlan) to IPV4 (vlan) Translation macros
#define	VLAN_2_VLAN_V4_TRANS_SDRAM_ADJ			(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VLAN_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF		(DESC_DEF_SDRAM_PKT_START_OFF + VLAN_2_VLAN_V4_TRANS_SDRAM_ADJ)
#define	VLAN_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF		(VLAN_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF		(VLAN_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_2_VLAN_V4_TRANS_IP_SDRAM_OFF		(VLAN_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_2_VLAN_V4_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)
// IPV6 (vlan+pppoe) to IPV4 (vlan) Translation macros
#define	VPPPOE_2_VLAN_V4_TRANS_SDRAM_ADJ		(DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS	\
							 + DESC_DEF_PPPOE_HDR_SIZE			\
							 - DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VPPPOE_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VPPPOE_2_VLAN_V4_TRANS_SDRAM_ADJ)
#define	VPPPOE_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF		(VPPPOE_2_VLAN_V4_TRANS_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VPPPOE_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF	(VPPPOE_2_VLAN_V4_TRANS_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VPPPOE_2_VLAN_V4_TRANS_IP_SDRAM_OFF		(VPPPOE_2_VLAN_V4_TRANS_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VPPPOE_2_VLAN_V4_TRANS_ORIG_HDR_SKIP_SZ		(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
							+ DESC_DEF_VLAN_HDR_SIZE			\
							+ DESC_DEF_PPPOE_HDR_SIZE			\
							+ DESC_DEF_ETH_TYPE_SIZE			\
							+ DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS)

// IPV6 (vlan) to IPV4 (vlan) Tunnel - encapsulation macros
#define	VLAN_2_VLAN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VLAN_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VLAN_2_VLAN_V4_TUNNEL_SDRAM_ADJ)
#define	VLAN_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF	(VLAN_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(VLAN_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_2_VLAN_V4_TUNNEL_IP_SDRAM_OFF	(VLAN_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_2_VLAN_V4_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE		\
						+ DESC_DEF_VLAN_HDR_SIZE		\
						+ DESC_DEF_ETH_TYPE_SIZE)

// IPV4 (vlan) Tunnel to  IPV6 (vlan) - decapsulation macros
#define	VLAN_V4_TUNNEL_2_VLAN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)
#define	VLAN_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VLAN_V4_TUNNEL_2_VLAN_SDRAM_ADJ)
#define	VLAN_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF	(VLAN_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define	VLAN_V4_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF	(VLAN_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VLAN_V4_TUNNEL_2_VLAN_IP_SDRAM_OFF	(PLAIN_V4_TUNNEL_2_PLAIN_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define	VLAN_V4_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE			\
						+ DESC_DEF_VLAN_HDR_SIZE			\
						+ DESC_DEF_ETH_TYPE_SIZE			\
						+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)		

// IPV6 (vlan+pppoe) to IPV4 (vlan) Tunneling - encapsulation macros
#define	VPPPOE_2_VLAN_V4_TUNNEL_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS 	\
							- DESC_DEF_PPPOE_HDR_SIZE)
#define	VPPPOE_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF - VPPPOE_2_VLAN_V4_TUNNEL_SDRAM_ADJ)
#define VPPPOE_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF		(VPPPOE_2_VLAN_V4_TUNNEL_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define VPPPOE_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF	(VPPPOE_2_VLAN_V4_TUNNEL_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VPPPOE_2_VLAN_V4_TUNNEL_IP_SDRAM_OFF		(VPPPOE_2_VLAN_V4_TUNNEL_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define VPPPOE_2_VLAN_V4_TUNNEL_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE 	\
							+ DESC_DEF_VLAN_HDR_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE)
// IPV4 (vlan+pppoe) Tunnel to IPV6 (vlan) - decapsulation macros
#define	VPPPOE_V4_TUNNEL_2_VLAN_SDRAM_ADJ		(DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS 	\
							+ DESC_DEF_PPPOE_HDR_SIZE)
#define	VPPPOE_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF	(DESC_DEF_SDRAM_PKT_START_OFF + VPPPOE_V4_TUNNEL_2_VLAN_SDRAM_ADJ)
#define VPPPOE_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF		(VPPPOE_V4_TUNNEL_2_VLAN_MACADDR_SDRAM_OFF + DESC_DEF_ETH_HDR_MACADDRS_SIZE)
#define VPPPOE_V4_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF	(VPPPOE_V4_TUNNEL_2_VLAN_VLAN_SDRAM_OFF + DESC_DEF_VLAN_HDR_SIZE)
#define	VPPPOE_V4_TUNNEL_2_VLAN_IP_SDRAM_OFF		(VPPPOE_V4_TUNNEL_2_VLAN_ETHTYPE_SDRAM_OFF + DESC_DEF_ETH_TYPE_SIZE)
#define VPPPOE_V4_TUNNEL_2_VLAN_ORIG_HDR_SKIP_SZ	(DESC_DEF_ETH_HDR_MACADDRS_SIZE 	\
							+ DESC_DEF_VLAN_HDR_SIZE		\
							+ DESC_DEF_PPPOE_HDR_SIZE		\
							+ DESC_DEF_ETH_TYPE_SIZE		\
							+ DESC_DEF_IPV4_HEADER_SIZE_WITHOUT_OPTIONS)


// Generic Tunneling/Translation Descriptor definitions
	 
 //IPV6 to IPV4 Tunnel
#define	IPV6_TO_IPV4_TUNNEL_DESC(x)											\
        PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, x##_SDRAM_ADJ, 10),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),	\
        COPY_IMED(DESC_DEF_ETH_TYPE_IP, SDRAM, x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        IP_CHECKSUM( TTL_NO_CHANGE, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),	\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

#define	IPV6_TO_IPV4_TUNNEL_WITH_VLAN_DESC(x)										\
        PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, x##_SDRAM_ADJ, 11),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),	\
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, SDRAM, x##_VLAN_SDRAM_OFF, CHAIN_CONT),		\
        COPY_IMED(DESC_DEF_ETH_TYPE_IP, SDRAM, x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        IP_CHECKSUM( TTL_NO_CHANGE, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),	\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

// IPV4 Tunnel to IPV6
#define	IPV4_TUNNEL_TO_IPV6_DESC(x)											\
        PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, x##_SDRAM_ADJ, 9),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),	\
        COPY_IMED(DESC_DEF_ETH_TYPE_IPV6, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

#define	IPV4_TUNNEL_TO_IPV6_WITH_VLAN_DESC(x)										\
        PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, x##_SDRAM_ADJ, 10),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),	\
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, SDRAM, x##_VLAN_SDRAM_OFF, CHAIN_CONT),		\
        COPY_IMED(DESC_DEF_ETH_TYPE_IPV6, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

// IPV6 to IPV4 Translation
#define	IPV6_TO_IPV4_TRANSLATION_DESC(x)										\
        PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, x##_SDRAM_ADJ, 10),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),	\
        COPY_IMED(DESC_DEF_ETH_TYPE_IP, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        IP_CHECKSUM( TTL_NO_CHANGE, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),	\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

#define	IPV6_TO_IPV4_TRANSLATION_WITH_VLAN_DESC(x)									\
        PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, x##_SDRAM_ADJ, 11),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),	\
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, SDRAM, x##_VLAN_SDRAM_OFF, CHAIN_CONT),		\
        COPY_IMED(DESC_DEF_ETH_TYPE_IP, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        IP_CHECKSUM( TTL_NO_CHANGE, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),	\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

 // IPV4 to IPV6 Tunnel
#define	IPV4_TO_IPV6_TUNNEL_DESC(x)											\
        PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, x##_SDRAM_ADJ, 11),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),		\
        COPY_IMED(DESC_DEF_ETH_TYPE_IPV6, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        COPY( DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),	\
        IP_CHECKSUM( TTL_DECREMENT, CB, x##_ORIG_HDR_SKIP_SZ, CB, x##_ORIG_HDR_SKIP_SZ, CHAIN_CONT),			\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

#define	IPV4_TO_IPV6_TUNNEL_WITH_VLAN_DESC(x)										\
        PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, x##_SDRAM_ADJ, 12),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),		\
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, x##_VLAN_SDRAM_OFF, CHAIN_CONT),			\
        COPY_IMED(DESC_DEF_ETH_TYPE_IPV6, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        COPY( DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),	\
        IP_CHECKSUM( TTL_DECREMENT, CB, x##_ORIG_HDR_SKIP_SZ, CB, x##_ORIG_HDR_SKIP_SZ, CHAIN_CONT),			\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

// IPV6 Tunnel to IPV4
#define IPV6_TUNNEL_TO_IPV4_DESC(x)											\
        PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, x##_SDRAM_ADJ, 10),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),		\
        COPY_IMED(DESC_DEF_ETH_TYPE_IP, SDRAM, x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        IP_CHECKSUM( TTL_DECREMENT, CB, x##_ORIG_HDR_SKIP_SZ, CB, x##_ORIG_HDR_SKIP_SZ, CHAIN_CONT),			\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

#define IPV6_TUNNEL_TO_IPV4_WITH_VLAN_DESC(x)										\
        PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, x##_SDRAM_ADJ, 11),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),		\
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, x##_VLAN_SDRAM_OFF, CHAIN_CONT),			\
        COPY_IMED(DESC_DEF_ETH_TYPE_IP, SDRAM, x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        IP_CHECKSUM( TTL_DECREMENT, CB, x##_ORIG_HDR_SKIP_SZ, CB, x##_ORIG_HDR_SKIP_SZ, CHAIN_CONT),			\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

// IPV4 to IPV6 Translation
#define	IPV4_TO_IPV6_TRANSLATION_DESC(x)										\
        PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, x##_SDRAM_ADJ, 10),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),		\
        COPY_IMED(DESC_DEF_ETH_TYPE_IPV6, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        COPY( DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),	\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

#define	IPV4_TO_IPV6_TRANSLATION_WITH_VLAN_DESC(x)									\
        PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, x##_SDRAM_ADJ, 11),							\
        DESC_DEF_SDMA_STATUS,												\
        SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),							\
        SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),						\
        SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),							\
        SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),						\
        COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, SDRAM, x##_MACADDR_SDRAM_OFF, CHAIN_CONT),		\
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, x##_VLAN_SDRAM_OFF, CHAIN_CONT),			\
        COPY_IMED(DESC_DEF_ETH_TYPE_IPV6, SDRAM,  x##_ETHTYPE_SDRAM_OFF, CHAIN_CONT),					\
        COPY( DESC_DEF_IPV6_HEADER_SIZE_WITHOUT_EXTN_HDRS, APDMEM, DESC_DEF_TUNNELIPHDR_START_OFF, SDRAM, x##_IP_SDRAM_OFF, CHAIN_CONT),													\
        COPY( (DESC_DEF_CB_PKT_HDR_SIZE-x##_ORIG_HDR_SKIP_SZ), CB, (DESC_DEF_CB_START_OFF+x##_ORIG_HDR_SKIP_SZ), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+x##_ORIG_HDR_SKIP_SZ), CHAIN_END),

// This symbolic constants describe descritop number in descArray
#define DESC_SELECT_SEND_TO_HOST                                0
#define RESERVED2                                               1
#define DESC_SELECT_ROUTE                                       2
#define DESC_SELECT_ROUTE_SNAT                                  3
#define DESC_SELECT_ROUTE_DNAT                                  4
#if 0
#define DESC_SELECT_ROUTE_PLAIN_VLAN                            5
#else
#define DESC_SELECT_ROUTE_PLAIN_VLAN                            5
#define DESC_SELECT_ROUTE_PLAIN_VLAN_SNAT                       6
#define DESC_SELECT_ROUTE_PLAIN_VLAN_DNAT                       7
#endif
#if 0
#define DESC_SELECT_ROUTE_VLAN_PLAIN                            6
#else
#define DESC_SELECT_ROUTE_VLAN_PLAIN                            8
#define DESC_SELECT_ROUTE_VLAN_PLAIN_SNAT                       9
#define DESC_SELECT_ROUTE_VLAN_PLAIN_DNAT                       10
#endif
#if 0
#define DESC_SELECT_ROUTE_VLAN_VLAN                             7
#else
#define DESC_SELECT_ROUTE_VLAN_VLAN                             11
#define DESC_SELECT_ROUTE_VLAN_VLAN_SNAT                        12
#define DESC_SELECT_ROUTE_VLAN_VLAN_DNAT                        13 
#endif
#if 0
#define DESC_SELECT_ROUTE_WITH_APPREHDR_WRITE                   8
#define DESC_SELECT_BRIDGE                                      9
#else
#define DESC_SELECT_ROUTE_WITH_APPREHDR_WRITE                   14
#define DESC_SELECT_BRIDGE                                      15      // NOT USED
#endif

#define DESC_SELECT_PPPOE_PLAIN                                          16
#define DESC_SELECT_ROUTE_SNAT_PPPOE_PLAIN                               17
#define DESC_SELECT_ROUTE_DNAT_PPPOE_PLAIN                               18
#define DESC_SELECT_ROUTE_PLAIN_VLAN_PPPOE_PLAIN                         19
#define DESC_SELECT_ROUTE_PLAIN_VLAN_SNAT_PPPOE_PLAIN                    20
#define DESC_SELECT_ROUTE_PLAIN_VLAN_DNAT_PPPOE_PLAIN                    21
#define DESC_SELECT_ROUTE_VLAN_PLAIN_PPPOE_PLAIN                         22
#define DESC_SELECT_ROUTE_VLAN_PLAIN_SNAT_PPPOE_PLAIN                    23
#define DESC_SELECT_ROUTE_VLAN_PLAIN_DNAT_PPPOE_PLAIN                    24
#define DESC_SELECT_ROUTE_VLAN_VLAN_PPPOE_PLAIN                          25
#define DESC_SELECT_ROUTE_VLAN_VLAN_SNAT_PPPOE_PLAIN                     26
#define DESC_SELECT_ROUTE_VLAN_VLAN_DNAT_PPPOE_PLAIN                     27 
//RVRV+
// Attention - Maintain below 4 bridge descriptors in same order
#define DESC_SELECT_BRIDGE_PLAIN_PLAIN                                   28
#define DESC_SELECT_BRIDGE_VLAN_PLAIN                                    29
#define DESC_SELECT_BRIDGE_PLAIN_VLAN                                    30
#define DESC_SELECT_BRIDGE_VLAN_VLAN                                     31

//MULTICAST related descriptors
// MULTICAST ROUTING
#define DESC_SELECT_MCAST_ROUTING_PLAIN                                  32
#define DESC_SELECT_MCAST_ROUTING_PPPOE                                  33
#define DESC_SELECT_MCAST_ROUTING_PPPOE_VLAN                             34
#define DESC_SELECT_MCAST_ROUTING_VLAN                                   35

// MULTICAST BRIDGING
#define DESC_SELECT_MCAST_BRIDGING_PLAIN                                 36
#define DESC_SELECT_MCAST_BRIDGING_VLAN                                  37

//PPPOE RELAY/PASSTHROUGH 
#define DESC_SELECT_PPPOE_TO_PPPOE	                                  38
#define DESC_SELECT_VLANPPPOE_TO_VLANPPPOE	                          39
#define DESC_SELECT_PLAINPPPOE_TO_VLANPPPOE                               40
#define DESC_SELECT_VLANPPPOE_TO_PLAINPPPOE                               41
#define DESC_SELECT_ESP_ROUTE_SNAT                                        42	//dekumar:IPSec passthru										
#define DESC_SELECT_ESP_ROUTE_DNAT                                        43	//dekumar:IPSec passthru										

/* Moving These descriptors to LMEM
//PPPOE RELAY/PASSTHROUGH_IPv6 
#define DESC_SELECT_PPPOE_TO_PPPOE_IPV6	                                  44
#define DESC_SELECT_VLANPPPOE_TO_VLANPPPOE_IPV6	                          45
#define DESC_SELECT_PLAINPPPOE_TO_VLANPPPOE_IPV6                          46
#define DESC_SELECT_VLANPPPOE_TO_PLAINPPPOE_IPV6                          47
*/




// First command of every set has an information which will be used
// by CL AP to adjust packet length and payload startOffset
// after packet modification. As well it has number of commands in
// the descriptor which is being used by Host to calculate relative offset
// of specific descriptor from start address of decriptors table.
// Format of first command in every set is:
// bit 31 - Is packet adjustment needed? 1 - Yes, 0 - No
// bit 30 - sign of size change: 0 - positive, 1 - negative.
//          Attention! For startOffset this sign will have opposite meaning:
//          0 - negative, 1 - positive ( It's obvious )
//          This bit will be used by CL AP ONLY
// bits 29 - 24 - value of packet size and startOffset change
// bits 23 - 8  - RESERVED
// bits  7 - 0  - offset to last command in descriptor (used by Host)
// ATTENTION! When no adjustment is needed, all 3 fields: adjustment, sign_size_change and
//            size_start_offset can ( recommended ) be set with symbolic constant "NO_ADJUST"

#define PACKET_ADJUSTMENT_BIT           31
#define SIGN_SIZE_CHANGE_BIT            30
#define SIZE_START_OFFSET_CHANGE_BITS   24              // bits 29 - 24
#define NUMBER_COMMANDS_IN_DESC_BITS    0               // bits 7 - 0
#define SIZE_OF_BRIDGE_DESCR            11

#define NO_ADJUST                       0
#define YES_ADJUST                      1

#define POS_CHANGE                      0
#define NEG_CHANGE                      1

#define PREDESCRIPTOR( ADJUST, SIGN_OF_CHANGE, SIZE, NUM_COMMANDS)                                  \
                     ((ADJUST << PACKET_ADJUSTMENT_BIT)        |                                    \
                      (SIGN_OF_CHANGE << SIGN_SIZE_CHANGE_BIT) |                                    \
                      (SIZE << SIZE_START_OFFSET_CHANGE_BITS)  |                                    \
                      ( NUM_COMMANDS<< NUMBER_COMMANDS_IN_DESC_BITS))

// SDMA commands types
#define SET_BASE_ADDR_CMD               0
#define COMPARE_CMD                     1
#define COPY_CMD                        2
#define COPY_IMMEDIATE_CMD              3
#define IP_CHECKSUM_CMD                 4
#define LOAD_L4_CHECKSUM_CMD            5
#define SHORT_COPY_CMD                  6

// SDMA commands Mem Ids
#define APDMEM                          0
#define CB                              1
#define FT                              2
#define SDRAM                           3

// SDMA commands Chain field
#define CHAIN_CONT                      1
#define CHAIN_END                       0

// SDMA IPvX types
#define IPV4_TYPE                       0
#define IPV6_TYPE                       1

// SDMA TTL Operations type
#define TTL_DECREMENT                   0
#define TTL_NO_CHANGE                   1


// SDMA commands macros
#define SET_BASE_ADDR( ADDRESS, MEMID, CHAIN)                                                       \
                      (( ADDRESS << 16 ) | (MEMID << 4) | (CHAIN << 3) | SET_BASE_ADDR_CMD)

#define SET_SDRAM_BASE_ADDR( ADDRESS,  CHAIN)                                                       \
                      ((SDRAM << 4) | (CHAIN << 3) | SET_BASE_ADDR_CMD), ADDRESS


#define COMPARE( LENGTH, SRC_MEMID_1, SRC_ADDR_1, SRC_MEMID_2, SRC_ADDR_2, CHAIN)                   \
                      (( LENGTH << 24 ) | (SRC_MEMID_1 << 22) | (SRC_ADDR_1 << 14) |                \
                       (SRC_MEMID_2 << 12) | (SRC_ADDR_2 << 4) | (CHAIN << 3) | COMPARE_CMD )

#define COPY( LENGTH, SRC_MEMID, SRC_ADDR, DST_MEMID, DST_ADDR, CHAIN)                              \
                      (( LENGTH << 24 ) | (SRC_MEMID << 22) | (SRC_ADDR << 14) |                    \
                       (DST_MEMID << 12) | (DST_ADDR << 4) | (CHAIN << 3) | COPY_CMD )

#define COPY_IMED( VALUE, DST_MEMID, DST_ADDR, CHAIN)                                               \
                      (( VALUE << 14 ) | (DST_MEMID << 12) | (DST_ADDR << 4) |                      \
                       (CHAIN << 3) | COPY_IMMEDIATE_CMD )

#define IP_CHECKSUM( TTL_VAL, SRC_MEMID, SRC_ADDR, DST_MEMID, DST_ADDR, CHAIN)                      \
                      (( TTL_VAL << 24 ) | (SRC_MEMID << 22) | (SRC_ADDR << 14) |                   \
                       (DST_MEMID << 12) | (DST_ADDR << 4) | (CHAIN << 3) | IP_CHECKSUM_CMD )

#define LOAD_L4_CHECKSUM( IP_TYPE, MEMID_1, L3_HDR_ADDR, MEMID_2, L4_HDR_ADDR, CHAIN)               \
                      (( IP_TYPE << 24 ) | (MEMID_1 << 22) | (L3_HDR_ADDR << 14) |                  \
                       (MEMID_2 << 12) | (L4_HDR_ADDR << 4) | (CHAIN << 3) | LOAD_L4_CHECKSUM_CMD )


#define SHORT_COPY( LENGTH, SRC_MEMID, SRC_ADDR, DST_MEMID, DST_ADDR, CHAIN)                    \
                      (( LENGTH << 24 ) | (SRC_MEMID << 22) | (SRC_ADDR << 14) |                    \
                       (DST_MEMID << 12) | (DST_ADDR << 4) | (CHAIN << 3) | SHORT_COPY_CMD )

#ifdef AP_ASSEMBLER
#define FT_DST_MAC_OFFSET                       typedef   apFlowEntry_t.apModify.dstMAC
#define FT6_DST_MAC_OFFSET                      typedef   apIpv6FlowEntry_t.apModify.dstMacAddr
#else
#define FT_DST_MAC_OFFSET                       offsetof( apFlowEntry_t,apModify.dstMAC )
#define FT6_DST_MAC_OFFSET                      offsetof( apIpv6FlowEntry_t,apModify.dstMacAddr )
#endif

#ifdef AP_ASSEMBLER
#define FT_ETHER_TYPE_OFFSET                    typedef   apFlowEntry_t.apModify.etherType
#define FT6_ETHER_TYPE_OFFSET                   typedef   apIpv6FlowEntry_t.apModify.etherType
#else
#define FT_ETHER_TYPE_OFFSET                    offsetof( apFlowEntry_t,apModify.etherType )
#define FT6_ETHER_TYPE_OFFSET                   offsetof( apIpv6FlowEntry_t,apModify.etherType )
#endif

#ifdef AP_ASSEMBLER
#define FT_VLANID_OFFSET                        typedef   apFlowEntry_t.apModify.vlanID
#define FT6_VLANID_OFFSET                       typedef   apIpv6FlowEntry_t.apModify.vlanID
#else
#define FT_VLANID_OFFSET                        offsetof( apFlowEntry_t,apModify.vlanID )
#define FT6_VLANID_OFFSET                       offsetof( apIpv6FlowEntry_t,apModify.vlanID )
#endif

#ifdef AP_ASSEMBLER
#define FT_NATIP_OFFSET                        typedef   apFlowEntry_t.apModify.natIP
#else
#define FT_NATIP_OFFSET                        offsetof( apFlowEntry_t,apModify.natIP )
#endif

#ifdef AP_ASSEMBLER
#define FT_NATPORT_OFFSET                        typedef   apFlowEntry_t.apModify.natPort
#else
#define FT_NATPORT_OFFSET                        offsetof( apFlowEntry_t,apModify.natPort )
#endif

#ifdef AP_ASSEMBLER
#define FT_SPI_OFFSET                        typedef   apFlowEntry_t.apModify.SPI
#else
#define FT_SPI_OFFSET                        offsetof( apFlowEntry_t,apModify.SPI )
#endif


#ifdef AP_ASSEMBLER
#define FT_SESSIONID_OFFSET                    typedef   apFlowEntry_t.apModify.sessionID
#define FT6_SESSIONID_OFFSET                   typedef   apIpv6FlowEntry_t.apModify.sessionID
#else
#define FT_SESSIONID_OFFSET                    offsetof( apFlowEntry_t,apModify.sessionID )
#define FT6_SESSIONID_OFFSET                   offsetof( apIpv6FlowEntry_t,apModify.sessionID )
#endif


#define CB_PPPOE_SESSIONID_OFFSET         0x10 
#define DESC_DEF_PPPOE_SESSIONID_SIZE	  0x02

#if 0
// IPv4 Bridging definitions
#ifdef AP_ASSEMBLER
#define BR_VLANID_OFFSET                        typedef   apNewBridgeEntry_t.vlanAddr
#else
#define BR_VLANID_OFFSET                        offsetof( apNewBridgeEntry_t,vlanAddr )
#endif
#endif

// IPv4 Bridging definitions
// The vlan is copied from DMEM (clTaskQueue_t), where it contains the updated COS value if bridge 
// marking is enabled. Since sdma uses the start of the apPreHeader as the base address of DMEM, 
// here the sizeof is used to get the offset of the vlanAddr which is stored after apPreHeader in 
// clTaskQueue_t strucuture
#ifdef AP_ASSEMBLER
#define BR_VLANID_OFFSET                        sizeof typedef apPreHeader_t
#else
#define BR_VLANID_OFFSET                        sizeof (apPreHeader_t)
#endif

#ifdef AP_ASSEMBLER
#define DESC_DEF_TUNNELIPHDR_START_OFF                    (sizeof typedef apPreHeader_t + 4)
#else
#define DESC_DEF_TUNNELIPHDR_START_OFF                    (sizeof (apPreHeader_t) + 4)
#endif

#define		DMEM_DESC_TYPE			0
#define		LMEM_DESC_TYPE			1
#define		MAX_NUM_DESC_TYPE		2

typedef struct descInfo_s {
	unsigned int	desclen;
	unsigned int 	*desclist;
	unsigned int 	*descmap;
} descInfo_t;

#define	COPY_DESC(descindex, dest)									\
	do {												\
		if ((descindex) < (MAX_NUM_DESCRIPTORS))						\
		{											\
			(dest) = descinfo[DMEM_DESC_TYPE].descmap[descindex]; 				\
		}											\
		else											\
		{											\
			(dest) = descinfo[LMEM_DESC_TYPE].descmap[(descindex) - (MAX_NUM_DESCRIPTORS)];	\
			(dest) |= 0x8000;								\
		}											\
	} while (0);

// define the descriptor selectors for LMEM descriptors here
#if defined(IPV6) && (IPV6)

// All LMEM descriptors should start after DMEM descroptors. That is why the base value used is MAX_NUM_DESCRIPTORS
#define LMEM_DESC_SELECT_ROUTE_IPV6                            		  MAX_NUM_DESCRIPTORS + 0
#define LMEM_DESC_SELECT_PPPOE_PLAIN_IPV6                                 MAX_NUM_DESCRIPTORS + 1
#define LMEM_DESC_SELECT_ROUTE_PLAIN_VLAN_PPPOE_PLAIN_IPV6                MAX_NUM_DESCRIPTORS + 2
#define LMEM_DESC_SELECT_ROUTE_VLAN_PLAIN_PPPOE_PLAIN_IPV6                MAX_NUM_DESCRIPTORS + 3
#define LMEM_DESC_SELECT_ROUTE_VLAN_VLAN_PPPOE_PLAIN_IPV6                 MAX_NUM_DESCRIPTORS + 4
#define LMEM_DESC_SELECT_ROUTE_PLAIN_VLAN_IPV6                            MAX_NUM_DESCRIPTORS + 5
#define LMEM_DESC_SELECT_ROUTE_VLAN_PLAIN_IPV6                            MAX_NUM_DESCRIPTORS + 6
#define LMEM_DESC_SELECT_ROUTE_VLAN_VLAN_IPV6                             MAX_NUM_DESCRIPTORS + 7

#if defined(IPV6_TO_IPV4_TUNNELING) && (IPV6_TO_IPV4_TUNNELING)

#define IPV6_TO_IPV4_TUNNELING_DESC_START	(LMEM_DESC_SELECT_ROUTE_VLAN_VLAN_IPV6 + 1)
#define IPV6_TO_IPV4_TUNNELING_DESC_END		(LMEM_DESC_SELECT_ROUTE_VLAN_VLAN_IPV6 + 16)

#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_PLAIN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_V4_TUNNEL_2_PLAIN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 1)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_VLAN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START + 2)
#define LMEM_DESC_SELECT_ROUTE_VLAN_V4_TUNNEL_2_PLAIN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 3)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_PLAIN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START + 4)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_V4_TUNNEL_2_VLAN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 5)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_VLAN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START + 6)
#define LMEM_DESC_SELECT_ROUTE_VLAN_V4_TUNNEL_2_VLAN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 7)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_PLAIN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START + 8)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_V4_TUNNEL_2_PLAIN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 9)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_VLAN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START + 10)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_V4_TUNNEL_2_PLAIN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 11)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_PLAIN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START + 12)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_V4_TUNNEL_2_VLAN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 13)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_VLAN_V4_TUNNEL	(IPV6_TO_IPV4_TUNNELING_DESC_START + 14)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_V4_TUNNEL_2_VLAN	(IPV6_TO_IPV4_TUNNELING_DESC_START + 15)

#else	// #if IPV6_TO_IPV4_TUNNELING

#define IPV6_TO_IPV4_TUNNELING_DESC_START	(LMEM_DESC_SELECT_ROUTE_VLAN_VLAN_IPV6 + 0)
#define IPV6_TO_IPV4_TUNNELING_DESC_END		(LMEM_DESC_SELECT_ROUTE_VLAN_VLAN_IPV6 + 0)

#endif	// #if IPV6_TO_IPV4_TUNNELING

#if defined(IPV6_TO_IPV4_TRANSLATION) && (IPV6_TO_IPV4_TRANSLATION)

#define IPV6_TO_IPV4_TRANSLATION_DESC_START	(IPV6_TO_IPV4_TUNNELING_DESC_END + 1)
#define IPV6_TO_IPV4_TRANSLATION_DESC_END	(IPV6_TO_IPV4_TUNNELING_DESC_END + 8)

#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_PLAIN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_VLAN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START + 1)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_PLAIN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START + 2)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_VLAN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START + 3)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_PLAIN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START + 4)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_VLAN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START + 5)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_PLAIN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START + 6)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_VLAN_V4_TRANS	(IPV6_TO_IPV4_TRANSLATION_DESC_START + 7)

#else	// #if IPV6_TO_IPV4_TRANSLATION

#define IPV6_TO_IPV4_TRANSLATION_DESC_START	(IPV6_TO_IPV4_TUNNELING_DESC_END + 0)
#define IPV6_TO_IPV4_TRANSLATION_DESC_END	(IPV6_TO_IPV4_TUNNELING_DESC_END + 0)

#endif	// #if IPV6_TO_IPV4_TRANSLATION

#if defined(IPV4_TO_IPV6_TUNNELING) && (IPV4_TO_IPV6_TUNNELING)

#define IPV4_TO_IPV6_TUNNELING_DESC_START	(IPV6_TO_IPV4_TRANSLATION_DESC_END + 1)
#define IPV4_TO_IPV6_TUNNELING_DESC_END		(IPV6_TO_IPV4_TRANSLATION_DESC_END + 16)

#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_PLAIN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_V6_TUNNEL_2_PLAIN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 1)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_VLAN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START + 2)
#define LMEM_DESC_SELECT_ROUTE_VLAN_V6_TUNNEL_2_PLAIN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 3)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_PLAIN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START + 4)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_V6_TUNNEL_2_VLAN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 5)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_VLAN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START + 6)
#define LMEM_DESC_SELECT_ROUTE_VLAN_V6_TUNNEL_2_VLAN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 7)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_PLAIN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START + 8)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_V6_TUNNEL_2_PLAIN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 9)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_VLAN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START + 10)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_V6_TUNNEL_2_PLAIN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 11)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_PLAIN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START + 12)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_V6_TUNNEL_2_VLAN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 13)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_VLAN_V6_TUNNEL	(IPV4_TO_IPV6_TUNNELING_DESC_START + 14)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_V6_TUNNEL_2_VLAN	(IPV4_TO_IPV6_TUNNELING_DESC_START + 15)

#else	// #if IPV4_TO_IPV6_TUNNELING

#define IPV4_TO_IPV6_TUNNELING_DESC_START	(IPV6_TO_IPV4_TRANSLATION_DESC_END + 0)
#define IPV4_TO_IPV6_TUNNELING_DESC_END		(IPV6_TO_IPV4_TRANSLATION_DESC_END + 0)

#endif	// #if IPV4_TO_IPV6_TUNNELING

#if defined(IPV4_TO_IPV6_TRANSLATION) && (IPV4_TO_IPV6_TRANSLATION)

#define IPV4_TO_IPV6_TRANSLATION_DESC_START	(IPV4_TO_IPV6_TUNNELING_DESC_END + 1)
#define IPV4_TO_IPV6_TRANSLATION_DESC_END	(IPV4_TO_IPV6_TUNNELING_DESC_END + 8)

#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_PLAIN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START)
#define LMEM_DESC_SELECT_ROUTE_PLAIN_2_VLAN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START + 1)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_PLAIN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START + 2)
#define LMEM_DESC_SELECT_ROUTE_VLAN_2_VLAN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START + 3)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_PLAIN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START + 4)
#define LMEM_DESC_SELECT_ROUTE_PPPOE_2_VLAN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START + 5)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_PLAIN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START + 6)
#define LMEM_DESC_SELECT_ROUTE_VPPPOE_2_VLAN_V6_TRANS	(IPV4_TO_IPV6_TRANSLATION_DESC_START + 7)

#else	// #if IPV4_TO_IPV6_TRANSLATION

#define IPV4_TO_IPV6_TRANSLATION_DESC_START	(IPV4_TO_IPV6_TUNNELING_DESC_END + 0)
#define IPV4_TO_IPV6_TRANSLATION_DESC_END	(IPV4_TO_IPV6_TUNNELING_DESC_END + 0)

#endif	// #if IPV4_TO_IPV6_TRANSLATION

#endif

#define VLAN_NAT_ESP_DESC_START 							(IPV4_TO_IPV6_TRANSLATION_DESC_END + 1)
#define VLAN_NAT_ESP_DESC_END								(IPV4_TO_IPV6_TRANSLATION_DESC_END + 14)
#define DESC_SELECT_ESP_ROUTE_PLAIN_VLAN_SNAT				(VLAN_NAT_ESP_DESC_START)
#define DESC_SELECT_ESP_ROUTE_PLAIN_VLAN_DNAT				(VLAN_NAT_ESP_DESC_START + 1)
#define DESC_SELECT_ESP_ROUTE_VLAN_PLAIN_SNAT				(VLAN_NAT_ESP_DESC_START + 2)
#define DESC_SELECT_ESP_ROUTE_VLAN_PLAIN_DNAT				(VLAN_NAT_ESP_DESC_START + 3)
#define DESC_SELECT_ESP_ROUTE_VLAN_VLAN_SNAT				(VLAN_NAT_ESP_DESC_START + 4)
#define DESC_SELECT_ESP_ROUTE_VLAN_VLAN_DNAT				(VLAN_NAT_ESP_DESC_START + 5)
#define DESC_SELECT_ESP_ROUTE_SNAT_PPPOE_PLAIN				(VLAN_NAT_ESP_DESC_START + 6)
#define DESC_SELECT_ESP_ROUTE_DNAT_PPPOE_PLAIN				(VLAN_NAT_ESP_DESC_START + 7)
#define DESC_SELECT_ESP_ROUTE_PLAIN_VLAN_SNAT_PPPOE_PLAIN	(VLAN_NAT_ESP_DESC_START + 8)
#define DESC_SELECT_ESP_ROUTE_PLAIN_VLAN_DNAT_PPPOE_PLAIN	(VLAN_NAT_ESP_DESC_START + 9)
#define DESC_SELECT_ESP_ROUTE_VLAN_PLAIN_SNAT_PPPOE_PLAIN	(VLAN_NAT_ESP_DESC_START + 10)
#define DESC_SELECT_ESP_ROUTE_VLAN_PLAIN_DNAT_PPPOE_PLAIN	(VLAN_NAT_ESP_DESC_START + 11)
#define DESC_SELECT_ESP_ROUTE_VLAN_VLAN_SNAT_PPPOE_PLAIN	(VLAN_NAT_ESP_DESC_START + 12)
#define DESC_SELECT_ESP_ROUTE_VLAN_VLAN_DNAT_PPPOE_PLAIN	(VLAN_NAT_ESP_DESC_START + 13)


//PPPOE RELAY/PASSTHROUGH_IPv6 
#define PPPOE_PASSTHRU_RELAY_IPV6_DESC_START					(VLAN_NAT_ESP_DESC_END + 1)
#define PPPOE_PASSTHRU_RELAY_IPV6_DESC_END					(VLAN_NAT_ESP_DESC_END + 4)

#define DESC_SELECT_PPPOE_TO_PPPOE_IPV6	                                  	(PPPOE_PASSTHRU_RELAY_IPV6_DESC_START)
#define DESC_SELECT_VLANPPPOE_TO_VLANPPPOE_IPV6	                         	(PPPOE_PASSTHRU_RELAY_IPV6_DESC_START + 1) 
#define DESC_SELECT_PLAINPPPOE_TO_VLANPPPOE_IPV6                          	(PPPOE_PASSTHRU_RELAY_IPV6_DESC_START + 2) 
#define DESC_SELECT_VLANPPPOE_TO_PLAINPPPOE_IPV6                         	(PPPOE_PASSTHRU_RELAY_IPV6_DESC_START + 3) 



/*_____________________________________HEADER GAURD_____________________________________*/
#endif /* _DESCRIPTORS_FIELDS_DEFINITIONS_H_ */
