/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : descriptorslist.h                                                   	*
 * Author       : Ikanos  [ vrevzin@ikanos.com ]                                   	*
 * Version      :                                                                       *
 * Date         : 09th Nov 2010                                                         *
 * Description  : ??????This file contains the structure of parser output            	*
 *                to store packet information. It also contain information to carry     *
 *                to transmiting interface.                                             *
 *                                                                                      *
 ****************************************************************************************/

/*_____________________________________Header Gaurd_____________________________________*/
#ifndef _DESCRIPTORS_LIST_H_
#define _DESCRIPTORS_LIST_H_

    // First command of every set has an information which will be used
    // by CL AP to adjust packet length and payload startOffset
    // after packet modification. As well it has number of commands in
    // the descriptor which is being used by Host to calculate relative offset
    // of specific descriptor from start address of decriptors table.
    // Format of first command in every set is:
    // bit 31 - Is packet adjustment needed? 1 - Yes, 0 - No
    // bit 30 - sign of size change: 0 - positive, 1 - negative.
    //          Attention! For startOffset this sign will have opposite meaning:
    //          0 - negative, 1 - positive ( It's obvious )
    //          This bit will be used by CL AP ONLY
    // bits 29 - 24 - value of packet size and startOffset change
    // bits 23 - 8  - RESERVED
    // bits  7 - 0  - offset to last command in descriptor (used by Host)

    //0. Send to Host -- IT SHOULD BE FIRST ONE ON THE LIST
//    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 8),
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 7),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
//    COPY( DESC_DEF_APPREHDR_SIZE, APDMEM, DESC_DEF_APPREHDR_START_OFF, SDRAM, DESC_DEF_SDRAM_CLUSTER_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //1.RESERVED
    (0x0 << 31) | (0x0 << 30) | (0x00 << 24) | 1,  // 0 - len change and offset, 4 commands in desc
    0xAAAAAAAA,     //commands

    //2.Simple routing
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //3.Simple routing + SNAT
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 12),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_l4HDR_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3SRCIP_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //4.Simple routing + DNAT
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 12),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_l4HDR_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3DESTIP_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //5. Routing Plain to VLAN ( Add VLAN )
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF - DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF - DESC_DEF_VLAN_HDR_SIZE) + DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE - DESC_DEF_ETH_HDR_MACADDRS_SIZE) , CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+ DESC_DEF_ETH_HDR_MACADDRS_SIZE + DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    //6. Routing Plain to VLAN ( Add VLAN ) + SNAT
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 14),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_l4HDR_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3SRCIP_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+ DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    //7. Routing Plain to VLAN ( Add VLAN ) + DNAT
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 14),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_l4HDR_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3DESTIP_OFF, CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM,(DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    //8. Routing VLAN to Plain ( Remove VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    //9. Routing VLAN to Plain ( Remove VLAN ) + SNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 12),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),


    //10. Routing VLAN to Plain ( Remove VLAN ) + DNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 12),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    //11. Routing VLAN to VLAN ( Substitute VLAN )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_ID_SIZE, FT, FT_VLANID_OFFSET, CB, DESC_DEF_ETH_HDR_VLAN_HDR_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //12. Routing VLAN to VLAN ( Substitute VLAN ) + SNAT
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 13),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_ID_SIZE, FT, FT_VLANID_OFFSET, CB, DESC_DEF_ETH_HDR_VLAN_HDR_OFF, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB,(DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM,DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //13. Routing VLAN to VLAN ( Substitute VLAN ) + DNAT
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 13),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_ID_SIZE, FT, FT_VLANID_OFFSET, CB, DESC_DEF_ETH_HDR_VLAN_HDR_OFF, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB,(DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //14. Simple routing descriptor with apPrehdr written to sdram
//    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 10),
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),
//    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_CONT),
//    COPY( DESC_DEF_APPREHDR_SIZE, APDMEM, DESC_DEF_APPREHDR_START_OFF, SDRAM, DESC_DEF_SDRAM_CLUSTER_START_OFF, CHAIN_END),

    //15. Simple bridging descriptor
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 7),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),


    //16. Routing PPPOE to Plain ( Remove PPPOE )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE-DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

    

   //17.Pppoe to plain +SNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 13),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE-DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

    //18.Pppoe to plain + DNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 13),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB,(DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE-DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

    //19. Pppoe to plain + Routing Plain to VLAN ( Add VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE, 12),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE + DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB,(DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),
    
    //20. Pppoe to plain +Routing Plain to VLAN ( Add VLAN ) + SNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE, 15),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB,(DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB,(DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    //21. Pppoe to plain + Routing Plain to VLAN ( Add VLAN ) + DNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE, 15),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB,(DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),
    
//22. Pppoe to plain + Routing VLAN to Plain ( Remove VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE+ DESC_DEF_PPPOE_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CHAIN_END),

    //23. Pppoe to plain + Routing VLAN to Plain ( Remove VLAN ) + SNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, 13),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB,(DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE) , CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB,(DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE) , CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CHAIN_END),


    //24. Pppoe to plain + Routing VLAN to Plain ( Remove VLAN ) + DNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, 13),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CHAIN_END),

    //25. Pppoe to plain + Routing VLAN to VLAN ( Substitute VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

    //26. Pppoe to plain + Routing VLAN to VLAN ( Substitute VLAN ) + SNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 14),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

    //27. Pppoe to plain + Routing VLAN to VLAN ( Substitute VLAN ) + DNAT
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 14),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    LOAD_L4_CHECKSUM( IPV4_TYPE, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_l4HDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    SHORT_COPY( DESC_DEF_IPPORT_SIZE, FT, FT_NATPORT_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_DEST_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

/* 
 * ATTENTION: Keep below 4 descriptors in same order Maintain the size of all descriptors are same
 * If size changed then update bridging.inc to compute new offsets
 */
 
    //28. Simple bridging descriptor
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),
    DESC_DEF_DESC_DUMMY,  // DUMMY to make all Bridge descr as same size (10)
    DESC_DEF_DESC_DUMMY,  // DUMMY
    DESC_DEF_DESC_DUMMY,  // DUMMY
    DESC_DEF_DESC_DUMMY,  // DUMMY

    // 29. Bridging VLAN to Plain ( Remove VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, SIZE_OF_BRIDGE_DESCR),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),  // copy mac addrs
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),
    DESC_DEF_DESC_DUMMY,  // DUMMY to make all Bridge descr as same size (10)
    DESC_DEF_DESC_DUMMY,  // DUMMY
    DESC_DEF_DESC_DUMMY,  // DUMMY
    
    // 30. Bridging Plain to VLAN ( Add VLAN )
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, SIZE_OF_BRIDGE_DESCR),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),  // copy mac addrs
    COPY_IMED( DESC_DEF_ETH_TYPE_8021Q, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),     // write 0x8100
    COPY( DESC_DEF_VLAN_ID_SIZE, APDMEM, BR_VLANID_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_ETH_TYPE_SIZE), CHAIN_CONT),// write new VLAN value
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),   // copy leftover data
    DESC_DEF_DESC_DUMMY,  // DUMMY


    // 31. Bridging VLAN to VLAN ( Substitute VLAN )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, SIZE_OF_BRIDGE_DESCR),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_ID_SIZE, APDMEM, BR_VLANID_OFFSET, CB, DESC_DEF_ETH_HDR_VLAN_HDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),
    DESC_DEF_DESC_DUMMY,  // DUMMY to make all Bridge descr as same size (10)
    DESC_DEF_DESC_DUMMY,  // DUMMY
    DESC_DEF_DESC_DUMMY,  // DUMMY

    // 32. MCAST Routing PLAIN to ANY
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    // 33. MCAST Routing PPPOE to ANY
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

    // 34. MCAST Routing PPPOE + VLAN  to ANY
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE+ DESC_DEF_PPPOE_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CHAIN_END),

    // 35. MCAST Routing VLAN  to ANY
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    // 36. MCAST Bridging PLAIN to ANY
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 7),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    // 37. MCAST Bridging VLAN  to ANY
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 8),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),  // copy mac addrs
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),


    //38. Routing PPPOE Relay ( relay/passthrough )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    //IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT_SESSIONID_OFFSET, CB, CB_PPPOE_SESSIONID_OFFSET, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //39. Routing PPPOE Relay (vlan to vlan ) ( relay/passthrough )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    //IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT_SESSIONID_OFFSET, CB, (CB_PPPOE_SESSIONID_OFFSET+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //40. Routing PPPOE Relay (plain to vlan) ( relay/passthrough )
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    //IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF - DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM,((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT_SESSIONID_OFFSET, CB, CB_PPPOE_SESSIONID_OFFSET, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_VLAN_HDR_SIZE) , CHAIN_END),

    //41. Routing PPPOE Relay ( vlan to plain )( relay/passthrough )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, (DESC_DEF_CB_START_OFF+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    //IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT_SESSIONID_OFFSET, CB, (CB_PPPOE_SESSIONID_OFFSET+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE, CB, (DESC_DEF_CB_START_OFF+DESC_DEF_VLAN_HDR_SIZE), SDRAM, DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE, CHAIN_END),

    //42.Simple routing + SNAT + ESP			dekumar:IPSec passthru.Modify SPI instead src port
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3SRCIP_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //43.Simple routing + DNAT + ESP		dekumar:IPSec passthru. 
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3DESTIP_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

/* Moving the following Descriptors to LMEM
    //44. Routing PPPOE Relay ( relay/passthrough )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, CB_PPPOE_SESSIONID_OFFSET, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //45. Routing PPPOE Relay (vlan to vlan ) ( relay/passthrough )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, (CB_PPPOE_SESSIONID_OFFSET+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //46. Routing PPPOE Relay (plain to vlan) ( relay/passthrough )
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF - DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, SDRAM,((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, CB_PPPOE_SESSIONID_OFFSET, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_VLAN_HDR_SIZE) , CHAIN_END),

    //47. Routing PPPOE Relay ( vlan to plain )( relay/passthrough )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, (DESC_DEF_CB_START_OFF+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, (CB_PPPOE_SESSIONID_OFFSET+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE, CB, (DESC_DEF_CB_START_OFF+DESC_DEF_VLAN_HDR_SIZE), SDRAM, DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE, CHAIN_END),
*/


    // Delimiter
    DESC_DEF_DESC_DELIMITER

/*_____________________________________HEADER GAURD_____________________________________*/
#endif /* _DESCRIPTORS_LIST_H_ */
