/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : descriptorslmemlist.h                                                   	*
 * Author       : Ikanos  [ bthakur@ikanos.com ]                                   	*
 * Version      :                                                                       *
 * Date         : 08th Apr 2011                                                         *
 * Description  : This file contains descriptor definitions which will go in LMEM   	*
 *                                                                                      *
 ****************************************************************************************/

/*_____________________________________Header Gaurd_____________________________________*/
#ifndef _DESCRIPTORS_LMEM_LIST_H_
#define _DESCRIPTORS_LMEM_LIST_H_

    // First command of every set has an information which will be used
    // by CL AP to adjust packet length and payload startOffset
    // after packet modification. As well it has number of commands in
    // the descriptor which is being used by Host to calculate relative offset
    // of specific descriptor from start address of decriptors table.
    // Format of first command in every set is:
    // bit 31 - Is packet adjustment needed? 1 - Yes, 0 - No
    // bit 30 - sign of size change: 0 - positive, 1 - negative.
    //          Attention! For startOffset this sign will have opposite meaning:
    //          0 - negative, 1 - positive ( It's obvious )
    //          This bit will be used by CL AP ONLY
    // bits 29 - 24 - value of packet size and startOffset change
    // bits 23 - 8  - RESERVED
    // bits  7 - 0  - offset to last command in descriptor (used by Host)

#if defined(IPV6) && (IPV6)
	//0. IPv6 routing
	PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 8),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
	COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //1. IPV6: Routing PPPOE to Plain ( Remove PPPOE )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IPV6, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE-DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

    //2.IPV6: Pppoe to plain + Routing Plain to VLAN ( Add VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE + DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IPV6, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB,(DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

    //3. IPV6: Pppoe to plain + Routing VLAN to Plain ( Remove VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IPV6, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CHAIN_END),


    //4. Pppoe to plain + Routing VLAN to VLAN ( Substitute VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY_IMED( DESC_DEF_ETH_TYPE_IPV6, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

//IPV6 VLAN Desc

    //5. Routing Plain to VLAN ( Add VLAN )
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF - DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF - DESC_DEF_VLAN_HDR_SIZE) + DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE - DESC_DEF_ETH_HDR_MACADDRS_SIZE) , CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+ DESC_DEF_ETH_HDR_MACADDRS_SIZE + DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),


    //6. Routing VLAN to Plain ( Remove VLAN )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 8),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),


    //7. Routing VLAN to VLAN ( Substitute VLAN )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_ID_SIZE, FT, FT6_VLANID_OFFSET, CB, DESC_DEF_ETH_HDR_VLAN_HDR_OFF, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

#if defined(IPV6_TO_IPV4_TUNNELING) && (IPV6_TO_IPV4_TUNNELING)

        //8. IPV6 (plain) to IPV4 (plain) Tunnel - encapsulation
	IPV6_TO_IPV4_TUNNEL_DESC (PLAIN_2_PLAIN_V4_TUNNEL)

        //9. IPV4 (plain) Tunnel to  IPV6 (plain) - decapsulation
	IPV4_TUNNEL_TO_IPV6_DESC (PLAIN_V4_TUNNEL_2_PLAIN)

        //10. IPV6 (plain) to IPV4 (vlan) Tunneling - encapsulation
	IPV6_TO_IPV4_TUNNEL_WITH_VLAN_DESC (PLAIN_2_VLAN_V4_TUNNEL)

        //11. IPV4 (vlan) Tunnel to IPV6 (plain) - decapsulation
	IPV4_TUNNEL_TO_IPV6_DESC (VLAN_V4_TUNNEL_2_PLAIN)

        //12. IPV6 (vlan) to IPV4 (plain) Tunneling - encapsulation
	IPV6_TO_IPV4_TUNNEL_DESC (VLAN_2_PLAIN_V4_TUNNEL)

        //13. IPV4 (plain) Tunnel to IPV6 (vlan) - decapsulation
	IPV4_TUNNEL_TO_IPV6_WITH_VLAN_DESC (PLAIN_V4_TUNNEL_2_VLAN)

        //14. IPV6 (vlan) to IPV4 (vlan) Tunneling - encapsulation
	IPV6_TO_IPV4_TUNNEL_WITH_VLAN_DESC (VLAN_2_VLAN_V4_TUNNEL)

        //15. IPV4 (vlan) Tunnel to IPV6 (vlan) - decapsulation
	IPV4_TUNNEL_TO_IPV6_WITH_VLAN_DESC (VLAN_V4_TUNNEL_2_VLAN)

        //16. IPV6 (pppoe) to IPV4 (plain) Tunnel - encapsulation
	IPV6_TO_IPV4_TUNNEL_DESC (PPPOE_2_PLAIN_V4_TUNNEL)

        //17. IPV4 (pppoe) Tunnel to  IPV6 (plain) - decapsulation
	IPV4_TUNNEL_TO_IPV6_DESC (PPPOE_V4_TUNNEL_2_PLAIN)

        //18. IPV6 (pppoe) to IPV4 (vlan) Tunneling - encapsulation
	IPV6_TO_IPV4_TUNNEL_WITH_VLAN_DESC (PPPOE_2_VLAN_V4_TUNNEL)

        //19. IPV4 (vlan+pppoe) Tunnel to IPV6 (plain) - decapsulation
	IPV4_TUNNEL_TO_IPV6_DESC (VPPPOE_V4_TUNNEL_2_PLAIN)

        //20. IPV6 (vlan+pppoe) to IPV4 (plain) Tunneling - encapsulation
	IPV6_TO_IPV4_TUNNEL_DESC (VPPPOE_2_PLAIN_V4_TUNNEL)

        //21. IPV4 (pppoe) Tunnel to IPV6 (vlan) - decapsulation
	IPV4_TUNNEL_TO_IPV6_WITH_VLAN_DESC (PPPOE_V4_TUNNEL_2_VLAN)

        //22. IPV6 (vlan+pppoe) to IPV4 (vlan) Tunneling - encapsulation
	IPV6_TO_IPV4_TUNNEL_WITH_VLAN_DESC (VPPPOE_2_VLAN_V4_TUNNEL)

        //23. IPV4 (vlan+pppoe) Tunnel to IPV6 (vlan) - decapsulation
	IPV4_TUNNEL_TO_IPV6_WITH_VLAN_DESC (VPPPOE_V4_TUNNEL_2_VLAN)
#endif

#if defined(IPV6_TO_IPV4_TRANSLATION) && (IPV6_TO_IPV4_TRANSLATION)

        //24. IPV6 (plain) to IPV4 (plain) Translation
	IPV6_TO_IPV4_TRANSLATION_DESC (PLAIN_2_PLAIN_V4_TRANS)

        //25. IPV6 (plain) to IPV4 (vlan) Translation
	IPV6_TO_IPV4_TRANSLATION_WITH_VLAN_DESC (PLAIN_2_VLAN_V4_TRANS)

        //26. IPV6 (vlan) to IPV4 (plain) Translation
	IPV6_TO_IPV4_TRANSLATION_DESC (VLAN_2_PLAIN_V4_TRANS)

        //27. IPV6 (vlan) to IPV4 (vlan) Translation
	IPV6_TO_IPV4_TRANSLATION_WITH_VLAN_DESC (VLAN_2_VLAN_V4_TRANS)

        //28. IPV6 (pppoe) to IPV4 (plain) Translation
	IPV6_TO_IPV4_TRANSLATION_DESC (PPPOE_2_PLAIN_V4_TRANS)

        //29. IPV6 (pppoe) to IPV4 (vlan) Translation
	IPV6_TO_IPV4_TRANSLATION_WITH_VLAN_DESC (PPPOE_2_VLAN_V4_TRANS)

        //30. IPV6 (vlan+pppoe) to IPV4 (plain) Translation
	IPV6_TO_IPV4_TRANSLATION_DESC (VPPPOE_2_PLAIN_V4_TRANS)

        //31. IPV6 (vlan+pppoe) to IPV4 (vlan) Translation
	IPV6_TO_IPV4_TRANSLATION_WITH_VLAN_DESC (VPPPOE_2_VLAN_V4_TRANS)
#endif

#if defined(IPV4_TO_IPV6_TUNNELING) && (IPV4_TO_IPV6_TUNNELING)

        //32. IPV4 (plain) to IPV6 (plain) Tunnel - encapsulation
	IPV4_TO_IPV6_TUNNEL_DESC (PLAIN_2_PLAIN_V6_TUNNEL)

        //33. IPV6 (plain) Tunnel to IPV4 (plain) - decapsulation
	IPV6_TUNNEL_TO_IPV4_DESC (PLAIN_V6_TUNNEL_2_PLAIN)

        //34. IPV4 (plain) to IPV6 (vlan) Tunneling - encapsulation
	IPV4_TO_IPV6_TUNNEL_WITH_VLAN_DESC (PLAIN_2_VLAN_V6_TUNNEL)

        //35. IPV6 (vlan) Tunnel to IPV4 (plain) - decapsulation
	IPV6_TUNNEL_TO_IPV4_DESC (VLAN_V6_TUNNEL_2_PLAIN)

        //36. IPV4 (vlan) to IPV6 (plain) Tunneling - encapsulation
	IPV4_TO_IPV6_TUNNEL_DESC (VLAN_2_PLAIN_V6_TUNNEL)

        //37. IPV6 (plain) Tunnel to IPV4 (vlan) - decapsulation
	IPV6_TUNNEL_TO_IPV4_WITH_VLAN_DESC (PLAIN_V6_TUNNEL_2_VLAN)

        //38. IPV4 (vlan) to IPV6 (vlan) Tunneling - encapsulation
	IPV4_TO_IPV6_TUNNEL_WITH_VLAN_DESC (VLAN_2_VLAN_V6_TUNNEL)

        //39. IPV6 (vlan) Tunnel to IPV4 (vlan) - decapsulation
	IPV6_TUNNEL_TO_IPV4_WITH_VLAN_DESC (VLAN_V6_TUNNEL_2_VLAN)

        //40. IPV4 (pppoe) to IPV6 (plain) Tunnel - encapsulation
	IPV4_TO_IPV6_TUNNEL_DESC (PPPOE_2_PLAIN_V6_TUNNEL)

        //41. IPV6 (pppoe) Tunnel to IPV4 (plain) - decapsulation
	IPV6_TUNNEL_TO_IPV4_DESC (PPPOE_V6_TUNNEL_2_PLAIN)

        //42. IPV4 (pppoe) to IPV6 (vlan) Tunneling - encapsulation
	IPV4_TO_IPV6_TUNNEL_WITH_VLAN_DESC (PPPOE_2_VLAN_V6_TUNNEL)

        //43. IPV6 (vlan+pppoe) Tunnel to IPV4 (plain) - decapsulation
	IPV6_TUNNEL_TO_IPV4_DESC (VPPPOE_V6_TUNNEL_2_PLAIN)

        //44. IPV4 (vlan+pppoe) to IPV6 (plain) Tunneling - encapsulation
	IPV4_TO_IPV6_TUNNEL_DESC (VPPPOE_2_PLAIN_V6_TUNNEL)

        //45. IPV6 (pppoe) Tunnel to IPV4 (vlan) - decapsulation
	IPV6_TUNNEL_TO_IPV4_WITH_VLAN_DESC (PPPOE_V6_TUNNEL_2_VLAN)

        //46. IPV4 (vlan+pppoe) to IPV6 (vlan) Tunneling - encapsulation
	IPV4_TO_IPV6_TUNNEL_WITH_VLAN_DESC (VPPPOE_2_VLAN_V6_TUNNEL)

        //47. IPV6 (vlan+pppoe) Tunnel to IPV4 (vlan) - decapsulation
	IPV6_TUNNEL_TO_IPV4_WITH_VLAN_DESC (VPPPOE_V6_TUNNEL_2_VLAN)
#endif

#if defined(IPV4_TO_IPV6_TRANSLATION) && (IPV4_TO_IPV6_TRANSLATION)

        //48. IPV4 (plain) to IPV6 (plain) Translation
	IPV4_TO_IPV6_TRANSLATION_DESC (PLAIN_2_PLAIN_V6_TRANS)

        //49. IPV4 (plain) to IPV6 (vlan) Translation
	IPV4_TO_IPV6_TRANSLATION_WITH_VLAN_DESC (PLAIN_2_VLAN_V6_TRANS)

        //50. IPV4 (vlan) to IPV6 (plain) Translation
	IPV4_TO_IPV6_TRANSLATION_DESC (VLAN_2_PLAIN_V6_TRANS)

        //51. IPV4 (vlan) to IPV6 (vlan) Translation
	IPV4_TO_IPV6_TRANSLATION_WITH_VLAN_DESC (VLAN_2_VLAN_V6_TRANS)

        //52. IPV4 (pppoe) to IPV6 (plain) Translation
	IPV4_TO_IPV6_TRANSLATION_DESC (PPPOE_2_PLAIN_V6_TRANS)

        //53. IPV4 (pppoe) to IPV6 (vlan) Translation
	IPV4_TO_IPV6_TRANSLATION_WITH_VLAN_DESC (PPPOE_2_VLAN_V6_TRANS)

        //54. IPV4 (vlan+pppoe) to IPV6 (plain) Translation
	IPV4_TO_IPV6_TRANSLATION_DESC (VPPPOE_2_PLAIN_V6_TRANS)

        //55. IPV4 (vlan+pppoe) to IPV6 (vlan) Translation
	IPV4_TO_IPV6_TRANSLATION_WITH_VLAN_DESC (VPPPOE_2_VLAN_V6_TRANS)
#endif

#endif

	
	//56. Routing Plain to VLAN ( Add VLAN ) + SNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 13),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3SRCIP_OFF, CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+ DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

	
	//57. Routing Plain to VLAN ( Add VLAN ) + DNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 13),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, DESC_DEF_PLAIN_PKT_l3DESTIP_OFF, CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF, CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CB, DESC_DEF_PLAIN_PKT_IPHDR_OFF, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM,(DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

	
	//58. Routing VLAN to Plain ( Remove VLAN ) + SNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 11),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

	
	//59. Routing VLAN to Plain ( Remove VLAN ) + DNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 11),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_VLAN_HDR_SIZE, CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE), CB, DESC_DEF_VLAN_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

	
	//60. Routing VLAN to VLAN ( Substitute VLAN ) + SNAT + ESP
	PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 12),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
	COPY( DESC_DEF_VLAN_ID_SIZE, FT, FT_VLANID_OFFSET, CB, DESC_DEF_ETH_HDR_VLAN_HDR_OFF, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM,DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

	
	//61. Routing VLAN to VLAN ( Substitute VLAN ) + DNAT + ESP
	PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 12),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
	COPY( DESC_DEF_VLAN_ID_SIZE, FT, FT_VLANID_OFFSET, CB, DESC_DEF_ETH_HDR_VLAN_HDR_OFF, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

	
	//62.Pppoe to plain +SNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 12),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE-DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

	
	//63.Pppoe to plain + DNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 12),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE-DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

	
	//64. Pppoe to plain +Routing Plain to VLAN ( Add VLAN ) + SNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE, 14),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB,(DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

	//65. Pppoe to plain + Routing Plain to VLAN ( Add VLAN ) + DNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE, 14),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE) , CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+ DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB,(DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_PPPOE_REM_VLAN_ADD_ADJ_SIZE+ DESC_DEF_ETH_HDR_MACADDRS_SIZE+ DESC_DEF_VLAN_HDR_SIZE), CHAIN_END),

	
	//66. Pppoe to plain + Routing VLAN to Plain ( Remove VLAN ) + SNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, 12),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB,(DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE) , CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CHAIN_END),

	
	//67. Pppoe to plain + Routing VLAN to Plain ( Remove VLAN ) + DNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, 12),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CB, DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_REM_VLAN_REM_ADJ_SIZE), CHAIN_END),

	
	//68. Pppoe to plain + Routing VLAN to VLAN ( Substitute VLAN ) + SNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 13),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3SRCIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),

	
	//69. Pppoe to plain + Routing VLAN to VLAN ( Substitute VLAN ) + DNAT + ESP
	PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_PPPOE_HDR_SIZE, 13),
	DESC_DEF_SDMA_STATUS,
	SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
	SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
	SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
	COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT_DST_MAC_OFFSET, CB, DESC_DEF_PPPOE_HDR_SIZE, CHAIN_CONT),
	COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT_ETHER_TYPE_OFFSET, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_IPADDR_SIZE, FT, FT_NATIP_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l3DESTIP_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY( DESC_DEF_ESP_SPI_SIZE, FT, FT_SPI_OFFSET, CB, (DESC_DEF_PLAIN_PKT_l4_SRC_PORT_OFF + DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	IP_CHECKSUM( TTL_DECREMENT, CB,(DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CB, (DESC_DEF_PLAIN_PKT_IPHDR_OFF+DESC_DEF_PPPOE_HDR_SIZE+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
	COPY_IMED( DESC_DEF_ETH_TYPE_IP, CB, (DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET+DESC_DEF_VLAN_HDR_SIZE+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_CONT),
	COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_PPPOE_HDR_SIZE), CB, DESC_DEF_PPPOE_HDR_SIZE, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF+DESC_DEF_PPPOE_HDR_SIZE), CHAIN_END),


    //70. Routing PPPOE Relay ( relay/passthrough )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, CB_PPPOE_SESSIONID_OFFSET, CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //71. Routing PPPOE Relay (vlan to vlan ) ( relay/passthrough )
    PREDESCRIPTOR( NO_ADJUST, NO_ADJUST, NO_ADJUST, 10),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, (CB_PPPOE_SESSIONID_OFFSET+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, DESC_DEF_SDRAM_PKT_START_OFF, CHAIN_END),

    //72. Routing PPPOE Relay (plain to vlan) ( relay/passthrough )
    PREDESCRIPTOR( YES_ADJUST, POS_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 11),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, DESC_DEF_CB_START_OFF, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, CB, DESC_DEF_CB_START_OFF, SDRAM, (DESC_DEF_SDRAM_PKT_START_OFF - DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_VLAN_HDR_SIZE, FT, FT6_ETHER_TYPE_OFFSET, SDRAM,((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+DESC_DEF_ETH_HDR_MACADDRS_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, CB_PPPOE_SESSIONID_OFFSET, CHAIN_CONT),
    COPY( (DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_ETH_HDR_MACADDRS_SIZE), CB, DESC_DEF_ETH_HDR_ETH_TYPE_OFFSET, SDRAM, ((DESC_DEF_SDRAM_PKT_START_OFF- DESC_DEF_VLAN_HDR_SIZE)+DESC_DEF_ETH_HDR_MACADDRS_SIZE+DESC_DEF_VLAN_HDR_SIZE) , CHAIN_END),

    //73. Routing PPPOE Relay ( vlan to plain )( relay/passthrough )
    PREDESCRIPTOR( YES_ADJUST, NEG_CHANGE, DESC_DEF_VLAN_HDR_SIZE, 9),
    DESC_DEF_SDMA_STATUS,
    SET_BASE_ADDR( DESC_DEF_TEMP_CB_BASE_ADDR, CB, CHAIN_CONT),
    SET_SDRAM_BASE_ADDR( DESC_DEF_TEMP_SDRAM_BASE_ADDR, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_FT_BASE_ADDR, FT, CHAIN_CONT),
    SET_BASE_ADDR( DESC_DEF_TEMP_APDMEM_BASE_ADDR, APDMEM, CHAIN_CONT),
    COPY( DESC_DEF_ETH_HDR_MACADDRS_SIZE, FT, FT6_DST_MAC_OFFSET, CB, (DESC_DEF_CB_START_OFF+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_PPPOE_SESSIONID_SIZE , FT, FT6_SESSIONID_OFFSET, CB, (CB_PPPOE_SESSIONID_OFFSET+DESC_DEF_VLAN_HDR_SIZE), CHAIN_CONT),
    COPY( DESC_DEF_CB_PKT_HDR_SIZE- DESC_DEF_VLAN_HDR_SIZE, CB, (DESC_DEF_CB_START_OFF+DESC_DEF_VLAN_HDR_SIZE), SDRAM, DESC_DEF_SDRAM_PKT_START_OFF+ DESC_DEF_VLAN_HDR_SIZE, CHAIN_END),


    // Delimiter
    DESC_DEF_DESC_DELIMITER

/*_____________________________________HEADER GAURD_____________________________________*/
#endif /* _DESCRIPTORS_LMEM_LIST_H_ */
