/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : ftindex.h                                                         	*
 * Author       : Ikanos Communications  [ jpattar@ikanos.com ]                         *
 * Version      :                                                                       *
 * Date         : 13 July 2010                                                       	*
 * Description  : This file contains indexing configuration of rft array index and    	*
 *                also bft array index. It also tell how lmem is configured.		*
 *                                                                                      *
 ****************************************************************************************/

/*_____________________________________Header Gaurd_____________________________________*/

#ifndef _FTINDEX_H_
#define _FTINDEX_H_

#include "netpro/apclasscfg.h"
/*_________________________________FT MEMORY ALLOCATION_________________________________*/
#if 0
		 -------------------------------
	  0x0000|	RFT_INDEX_BASE		|
		|				| Size: 0x200
		|				|
		 -------------------------------	
	  0x0200|	BFT_INDEX_BASE		|
		|				| Size: 0x100
		|				|
		 -------------------------------
	  0x0300|	   RFT_BASE		|
		|				| 
		|				| Size: Variable
	  	|	   BFT_BASE		|
		|				|
		|				|
		 -------------------------------
	  0x4600|    LMEM_OPERATIONTABLE_ADDR	|
		|          			| Size: Variable
		|				|
		 -------------------------------
	  0x4x00|	     FREE		|
		|				| Size: 
		|				|
		 -------------------------------

#endif


/*________________________________________MACROS________________________________________*/
#define PIO_LMEM_BASE               0x4000
#define CBUS_LMEM_BASE              0x191D0000


#define NUM_RFT_INDEX_ENTRY         128
#define NUM_BFT_INDEX_ENTRY         64
#if defined(IPV6) && (IPV6)
#define NUM_IPV6_RFT_INDEX_ENTRY    128
#else
#define NUM_IPV6_RFT_INDEX_ENTRY    0       // DO NOT CHANGE THIS ZERO VALUE
#endif

#define NUM_TOTAL_FT_INDEXES       (NUM_RFT_INDEX_ENTRY + NUM_BFT_INDEX_ENTRY + NUM_IPV6_RFT_INDEX_ENTRY)
#define	NUM_TOTAL_FT_ENTRIES   	   128

#define FT_INDEX_ENTRY_SIZE         4	    //size of ftIndex_s

#define RFT_HASH_MASK_BITS          NUM_RFT_INDEX_ENTRY - 1
#define BFT_HASH_MASK_BITS          NUM_BFT_INDEX_ENTRY - 1  
#define IPV6_RFT_HASH_MASK_BITS     NUM_IPV6_RFT_INDEX_ENTRY - 1  

// Below asserts will ensure that number of RFT, BFT and FT6 are power of 2
#ifdef AP_ASSEMBLER
assert NUM_RFT_INDEX_ENTRY & RFT_HASH_MASK_BITS,0
assert NUM_BFT_INDEX_ENTRY & BFT_HASH_MASK_BITS,0
assert NUM_IPV6_RFT_INDEX_ENTRY & IPV6_RFT_HASH_MASK_BITS,0
#else
#if (( NUM_RFT_INDEX_ENTRY & RFT_HASH_MASK_BITS != 0 ) |  \
     ( NUM_BFT_INDEX_ENTRY & BFT_HASH_MASK_BITS != 0 ) |  \
     ( NUM_IPV6_RFT_INDEX_ENTRY & IPV6_RFT_HASH_MASK_BITS != 0 ))
 Hash Mask value error
#endif
#endif

#define MAX_NUM_DESCRIPTORS	    64 //Maximum Number of Descriptors Supported

//The upper 16bits of 32bit address; if 15th bit (i.e. 31st bit) is 1 then FT entry is 
//in LMEM else FT entry is in SDRAM. 
#define	FT_ENTRY_IN_LMEM_BIT        15

#define FT_ENTRY_IN_LMEM_CHKFRMHOST_BIT        31

#define	PIO_RFT_INDEX_BASE	        PIO_LMEM_BASE
#define	CBUS_RFT_INDEX_BASE	        CBUS_LMEM_BASE

#define PIO_IPV6_RFT_INDEX_BASE        	( PIO_RFT_INDEX_BASE + \
				                ( NUM_RFT_INDEX_ENTRY * FT_INDEX_ENTRY_SIZE ))
#define CBUS_IPV6_RFT_INDEX_BASE       	( CBUS_RFT_INDEX_BASE + \
                                    		( NUM_RFT_INDEX_ENTRY * FT_INDEX_ENTRY_SIZE ))

#define PIO_BFT_INDEX_BASE         	( PIO_IPV6_RFT_INDEX_BASE + \
					                ( NUM_IPV6_RFT_INDEX_ENTRY * FT_INDEX_ENTRY_SIZE ))
#define CBUS_BFT_INDEX_BASE        	( CBUS_IPV6_RFT_INDEX_BASE + \
                                    		( NUM_IPV6_RFT_INDEX_ENTRY * FT_INDEX_ENTRY_SIZE ))

#define PIO_RFT_BASE              	(PIO_RFT_INDEX_BASE + (NUM_TOTAL_FT_INDEXES*FT_INDEX_ENTRY_SIZE))
#define CBUS_RFT_BASE             	(CBUS_RFT_INDEX_BASE + (NUM_TOTAL_FT_INDEXES*FT_INDEX_ENTRY_SIZE)) 

// BFT base is same as RFT base as all entries are allocated from the same LMEM pool
#define PIO_BFT_BASE              	PIO_RFT_BASE
#define CBUS_BFT_BASE             	CBUS_RFT_BASE
// IPV6 RFT base is same as RFT base as all entries are allocated from the same LMEM pool
#define PIO_IPV6_RFT_BASE          	PIO_RFT_BASE
#define CBUS_IPV6_RFT_BASE         	CBUS_RFT_BASE

#define	MAX_NUM_LMEM_DESCRIPTORS	80		// current desc count is 56. space for 4 more desc
#define	MAX_LMEM_DESC_SPACE		(44*MAX_NUM_LMEM_DESCRIPTORS)	// assuming 11 commands per descriptor

#define	PIO_CLASSIFIER1_LMEM_DESC_BASE	(PIO_RFT_BASE + (NUM_TOTAL_FT_ENTRIES * sizeof(commonEntry_t)))
#define	CBUS_CLASSIFIER1_LMEM_DESC_BASE (CBUS_RFT_BASE + (NUM_TOTAL_FT_ENTRIES * sizeof(commonEntry_t)))

#define	PIO_CLASSIFIER2_LMEM_DESC_BASE	(PIO_CLASSIFIER1_LMEM_DESC_BASE + MAX_LMEM_DESC_SPACE)
#define	CBUS_CLASSIFIER2_LMEM_DESC_BASE (CBUS_CLASSIFIER1_LMEM_DESC_BASE + MAX_LMEM_DESC_SPACE)

//RVRV How CBUS_DESCRIPTOR_BASE value has been calculated?  Do we need it? May be, in the future when DMem will be in shotage?
//#define PIO_DESCRIPTOR_BASE 		0xBC00
#define CBUS_DESCRIPTOR_BASE 		0x191D7C00

//check whether the ft entry address is of LMEM or SDRAM
//The reg1 should contains upper 2bytes (i.e. MSB) of 4byte address.
#define	FT_ENTRY_SDRAM_BIT	        15
#define	IS_FT_ENTRY_IN_SDRAM(reg1)	if bs	reg1, FT_ENTRY_SDRAM_BIT


/* L-Mem address translation macros */
#define LMEM_CBUS2PIO(addr)         (((UINT32)addr & 0x0000FFFF) + PIO_LMEM_BASE)
#define LMEM_PIO2CBUS(addr)         (((UINT32)addr & 0x0000FFFF) - PIO_LMEM_BASE + CBUS_LMEM_BASE)	/*dekumar:Corrected the definition*/
#define LMEM_IDENTIFICATION_MASK    0x80000000


//RVRV How CBUS_DESCRIPTOR_BASE value has been calculated?
//#define	LMEM_OPERATIONTABLE_ADDR	0xB91D4600
#define	LMEM_OPERATIONTABLE_ADDR	(0xB91D0000 + (NUM_TOTAL_FT_INDEXES * FT_INDEX_ENTRY_SIZE) + \
							NUM_TOTAL_FT_ENTRIES * sizeof(commonEntry_t) + \
							2*(MAX_LMEM_DESC_SPACE))

#if defined(POLICY_BASED_VLAN) && (POLICY_BASED_VLAN)
#define MAX_PORT_SUPPORTED					4 //(eth0, eth1, eth2(avdsl), wlan)
#define	CBUS_LMEM_POLICY_VLAN_BASE			(LMEM_OPERATIONTABLE_ADDR + OPTBL_MAX_ENTRIES * sizeof(apOps_t))
#define NUM_OF_TOTAL_POLICY_VLAN_ENTRY		MAX_PORT_SUPPORTED * MAX_VLAN_CLASS_ENTRIES
#define NUM_TOTAL_POLICY_VLAN_LMEM_SPACE	((NUM_OF_TOTAL_POLICY_VLAN_ENTRY * (sizeof(apVlanClass_t)))/4 + 1) /*In U32 */
#define	PIO_LMEM_POLICY_VLAN_BASE			(LMEM_CBUS2PIO(CBUS_LMEM_POLICY_VLAN_BASE))
#define SIZE_POLICY_VLAN_TABLE_OF_AN_INTERFACE	(MAX_VLAN_CLASS_ENTRIES * (sizeof(apVlanClass_t)))
#endif

/*_____________________________________INCLUDES_________________________________________*/

typedef struct ftIndex_s
{
	unsigned long	entryAddr;	
} ftIndex_t;

/*_____________________________________Header Gaurd_____________________________________*/
#endif  //_FTINDEX_H_ end of header guard.

