/* =====================================================================
 * Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information.
 * =====================================================================
 */

#ifndef _LISTMGRHW_H_
#define _LISTMGRHW_H_

/* This file will have the definitions of all the registers of the LIST MANAGER at TX
 * (QUEUEING DMA)
 *
 *
 *
 */

#define NO_OF_QUEUES				8
#define LIST_MGR_Q_LENGTH_IN_Q_ENTRIES		100
#define LIST_MGR_Q_ENTRY_SIZE_IN_WORDS		2    /* 2 * 4 = 8 bytes */
#define LIST_MGR_ATM_Q_ENTRY_SIZE_IN_WORDS	3    /* 3 * 4 = 12 bytes */
#define LIST_MGR_ETH_Q_ENTRY_SIZE_IN_WORDS	3    /* 3 * 4 = 12 bytes */	


#define LIST_MANAGER_BASE_ADDR		0x2400 //0x0C00 //list_mgr_register_array // change with the baseaddress	

/* Offsets of the various registers of List Manager */

/* Registers to set pointers of the QoS Queues located in SDRAM */
#define LIST_MGR_Q0_ADDR_OFFSET  		0x00  // 0x00 --> lower 16 bits , 0x01--> upper 16 bits
#define LIST_MGR_Q1_ADDR_OFFSET        		0x02 
#define LIST_MGR_Q2_ADDR_OFFSET         	0x04 
#define LIST_MGR_Q3_ADDR_OFFSET         	0x06 
#define LIST_MGR_Q4_ADDR_OFFSET         	0x08 
#define LIST_MGR_Q5_ADDR_OFFSET         	0x0A 
#define LIST_MGR_Q6_ADDR_OFFSET         	0x0C 
#define LIST_MGR_Q7_ADDR_OFFSET         	0x0E 

/* Control and Status Registers */

#define LIST_MGR_Q_ENTRY_LEN_OFFSET             0x10  /* Queue entry size in words - 4 bits */
#define LIST_MGR_Q_LEN_OFFSET         		0x11  /* Queue length in number of entries - 8 bits */
#define LIST_MGR_DMEM_Q_ENTRY_ADDR_OFFSET  	0x12  /* Pointer to an entry in DMEM - 16 bits */ 
#define LIST_MGR_DMA_CTRL_OFFSET      		0x13  /* DMA Control Reg - 8 bits  ---> [1,0] - Command Enq/Deq,[4-2]-Q num,[5]-DMA Start */
#define LIST_MGR_DMA_STATUS_OFFSET     		0x14  /* DMA Status - 8bits ----> [0]-DMA complete,[3-1]- Status -> Success, failure due to q full */

/* Packet Length at Queue Head Registers */
#define LIST_MGR_Q0_PKT_LEN_AT_HEAD_OFFSET  	0x15  /* Value 0 means Q is empty */
#define LIST_MGR_Q1_PKT_LEN_AT_HEAD_OFFSET  	0x16  /* Value 0 means Q is empty */
#define LIST_MGR_Q2_PKT_LEN_AT_HEAD_OFFSET  	0x17  /* Value 0 means Q is empty */
#define LIST_MGR_Q3_PKT_LEN_AT_HEAD_OFFSET  	0x18  /* Value 0 means Q is empty */
#define LIST_MGR_Q4_PKT_LEN_AT_HEAD_OFFSET  	0x19  /* Value 0 means Q is empty */
#define LIST_MGR_Q5_PKT_LEN_AT_HEAD_OFFSET  	0x1A  /* Value 0 means Q is empty */
#define LIST_MGR_Q6_PKT_LEN_AT_HEAD_OFFSET  	0x1B  /* Value 0 means Q is empty */
#define LIST_MGR_Q7_PKT_LEN_AT_HEAD_OFFSET  	0x1C  /* Value 0 means Q is empty */


#define  _LISTMGR_ADDR(reg) ( LIST_MGR_ ## reg ## _OFFSET + LIST_MANAGER_BASE_ADDR )


#define LIST_MGR_CBUS_Q0_ADDR_OFFSET                 0x00  
#define LIST_MGR_CBUS_Q1_ADDR_OFFSET                 0x04
#define LIST_MGR_CBUS_Q2_ADDR_OFFSET                 0x08
#define LIST_MGR_CBUS_Q3_ADDR_OFFSET                 0x0C
#define LIST_MGR_CBUS_Q4_ADDR_OFFSET                 0x10
#define LIST_MGR_CBUS_Q5_ADDR_OFFSET                 0x14
#define LIST_MGR_CBUS_Q6_ADDR_OFFSET                 0x18
#define LIST_MGR_CBUS_Q7_ADDR_OFFSET                 0x1C



#define  _LISTMGR_CBUS_ADDR(reg) ( LIST_MGR_CBUS_ ## reg ## _OFFSET + LIST_MANAGER_BASE_ADDR )
#define DEQUE_OPER_BIT  	3
#define ENQ_OPER_BIT  		3
#define ENQ_AT_HEAD_BIT         4
#define START_DMA_BIT  		5

#define DMA_FAIL_BIT  		1
#define DMA_BUSY_BIT  		0


#endif /* _LISTMGRHW_H_ */
