/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : parserhw.h                                                           	*
 * Author       : Ikanos Communications  [ jpattar@ikanos.com ]                         *
 * Version      :                                                                       *
 * Date         : 27th Nov 2009                                                         *
 * Description  : This file contains the parser's register offsets. 			*
 *                                                                                      *
 ****************************************************************************************/

/*_____________________________________Header Gaurd_____________________________________*/

#ifndef _PARSERHW_H_
#define _PARSERHW_H_

/*_______________________________________Includes_______________________________________*/



/*________________________________________MACROS________________________________________*/

/*
 * Following are the PIO offset. These macros are used by Port AP to configure parser.
 * These macros should not be used to refer from HOST (MIPS).
 */


/*	Parser PIO Base Address		  	*/
#define PARSER_PIO_BASE_ADDR                    0x1C00
#define PARSER_CBUS_BASE_ADDR               PARSER_PIO_BASE_ADDR


/*      Configuration Registers           	*/

#define PARSER_PIO_CFG_REG_P0    		0x01 
#define PARSER_PIO_CFG_REG_P1            	0x02 
#define PARSER_PIO_MAX_HDR_SIZE_REG_P0 		0x03 
#define PARSER_PIO_MAX_HDR_SIZE_REG_P1    	0x04 
#define PARSER_PIO_CNTRS_EN_REG_P0		0x05 
#define PARSER_PIO_CNTRS_EN_REG_P1   		0x06 
#define PARSER_PIO_HDR_HOLE1_LEN_REG_P0		0x07 
#define PARSER_PIO_HDR_HOLE1_LEN_REG_P1   	0x08 
#define PARSER_PIO_HDR_HOLE2_LEN_REG_P0		0x09 
#define PARSER_PIO_HDR_HOLE2_LEN_REG_P1		0x0A 
#define PARSER_PIO_HDR_DEST_REG_P0 		0x0B 
#define PARSER_PIO_HDR_DEST_REG_P1 		0x0C 
#define PARSER_PIO_DATA_DEST_REG_P0 		0x0D 
#define PARSER_PIO_DATA_DEST_REG_P1 		0x0E 

/* 	Status Register			  	*/
#define PARSER_PIO_STATUS_REG_P0 		0x0F 
#define PARSER_PIO_STATUS_REG_P1        	0x10 

/*	Counter Registers		  	*/
#define PARSER_PIO_PKT_CNTR_P0      		0x11 
#define PARSER_PIO_PKT_CNTR_P1            	0x12 
#define PARSER_PIO_BRDCAST_CNTR_P0       	0x13 
#define PARSER_PIO_BRDCAST_CNTR_P1         	0x14 
#define PARSER_PIO_RCVDBYTES_CNTR_P0    	0x15 
#define PARSER_PIO_RCVDBYTES_CNTR_P1    	0x16 
#define PARSER_PIO_RCVDBYTES_CNTR_P2 		0x17 
#define PARSER_PIO_RCVDBYTES_CNTR_P3    	0x18 
#define PARSER_PIO_TCP_CNTR_P0          	0x19 
#define PARSER_PIO_TCP_CNTR_P1          	0x1A 
#define PARSER_PIO_UDP_CNTR_P0          	0x1B 
#define PARSER_PIO_UDP_CNTR_P1          	0x1C 
#define PARSER_PIO_AHPKT_CNTR_P0       		0x21 
#define PARSER_PIO_AHPKT_CNTR_P1        	0x22 
#define PARSER_PIO_ESPPKT_CNTR_P0     		0x23 
#define PARSER_PIO_ESPPKT_CNTR_P1    		0x24 
#define PARSER_PIO_IPV6_CNTR_P0        		0x25 
#define PARSER_PIO_IPV6_CNTR_P1         	0x26 
#define PARSER_PIO_IPV4_CNTR_P0         	0x27 
#define PARSER_PIO_IPV4_CNTR_P1         	0x28 

/*	MAC address table Registers	  	*/
#define PARSER_PIO_PRG_ETHADR0_P0    	   	0x37 
#define PARSER_PIO_PRG_ETHADR0_P1     	   	0x38 
#define PARSER_PIO_PRG_ETHADR0_P2        	0x39 
#define PARSER_PIO_PRG_ETHADR1_P0          	0x3A 
#define PARSER_PIO_PRG_ETHADR1_P1        	0x3B 
#define PARSER_PIO_PRG_ETHADR1_P2        	0x3C 
#define PARSER_PIO_PRG_ETHADR2_P0        	0x3D 
#define PARSER_PIO_PRG_ETHADR2_P1        	0x3E 
#define PARSER_PIO_PRG_ETHADR2_P2        	0x3F 
#define PARSER_PIO_PRG_ETHADR3_P0        	0x40 
#define PARSER_PIO_PRG_ETHADR3_P1        	0x41 
#define PARSER_PIO_PRG_ETHADR3_P2        	0x42 
#define PARSER_PIO_PRG_ETHADR4_P0        	0x43 
#define PARSER_PIO_PRG_ETHADR4_P1        	0x44 
#define PARSER_PIO_PRG_ETHADR4_P2        	0x45 
#define PARSER_PIO_PRG_ETHADR5_P0        	0x46 
#define PARSER_PIO_PRG_ETHADR5_P1        	0x48 
#define PARSER_PIO_PRG_ETHADR5_P2        	0x49 
#define PARSER_PIO_PRG_ETHADR6_P0        	0x50 
#define PARSER_PIO_PRG_ETHADR6_P1        	0x51 
#define PARSER_PIO_PRG_ETHADR6_P2        	0x52 
#define PARSER_PIO_PRG_ETHADR7_P0        	0x53 
#define PARSER_PIO_PRG_ETHADR7_P1        	0x54 
#define PARSER_PIO_PRG_ETHADR7_P2        	0x55 

/* 	IP address table Registers	  	*/
#define PARSER_PIO_PRG_IPADR0_P0      	   	0x56 
#define PARSER_PIO_PRG_IPADR0_P1         	0x57 
#define PARSER_PIO_PRG_IPADR0_P2         	0x58 
#define PARSER_PIO_PRG_IPADR0_P3         	0x59 
#define PARSER_PIO_PRG_IPADR0_P4         	0x5A 
#define PARSER_PIO_PRG_IPADR0_P5         	0x5B 
#define PARSER_PIO_PRG_IPADR0_P6         	0x5C 
#define PARSER_PIO_PRG_IPADR0_P7         	0x5D 
#define PARSER_PIO_PRG_IPADR0_P8         	0x5E 
#define PARSER_PIO_PRG_IPADR1_P0         	0x5F 
#define PARSER_PIO_PRG_IPADR1_P1         	0x60 
#define PARSER_PIO_PRG_IPADR1_P2         	0x61 
#define PARSER_PIO_PRG_IPADR1_P3         	0x62 
#define PARSER_PIO_PRG_IPADR1_P4         	0x63 
#define PARSER_PIO_PRG_IPADR1_P5         	0x64 
#define PARSER_PIO_PRG_IPADR1_P6         	0x65 
#define PARSER_PIO_PRG_IPADR1_P7         	0x66 
#define PARSER_PIO_PRG_IPADR1_P8         	0x67 
#define PARSER_PIO_PRG_IPADR2_P0         	0x68 
#define PARSER_PIO_PRG_IPADR2_P1         	0x69 
#define PARSER_PIO_PRG_IPADR2_P2         	0x6A 
#define PARSER_PIO_PRG_IPADR2_P3         	0x6B 
#define PARSER_PIO_PRG_IPADR2_P4         	0x6C 
#define PARSER_PIO_PRG_IPADR2_P5         	0x6D 
#define PARSER_PIO_PRG_IPADR2_P6         	0x6E 
#define PARSER_PIO_PRG_IPADR2_P7         	0x6F 
#define PARSER_PIO_PRG_IPADR2_P8         	0x70 
#define PARSER_PIO_PRG_IPADR3_P0         	0x71 
#define PARSER_PIO_PRG_IPADR3_P1         	0x72 
#define PARSER_PIO_PRG_IPADR3_P2         	0x73 
#define PARSER_PIO_PRG_IPADR3_P3         	0x74 
#define PARSER_PIO_PRG_IPADR3_P4         	0x75 
#define PARSER_PIO_PRG_IPADR3_P5         	0x76 
#define PARSER_PIO_PRG_IPADR3_P6         	0x77 
#define PARSER_PIO_PRG_IPADR3_P7         	0x78 
#define PARSER_PIO_PRG_IPADR3_P8         	0x79 

/*	Vendor ID table registers	  	*/
#define PARSER_PIO_VID_0_P0      	   	0x7A 
#define PARSER_PIO_VID_0_P1              	0x7B 
#define PARSER_PIO_VID_1_P0              	0x7C 
#define PARSER_PIO_VID_1_P1              	0x7D 
#define PARSER_PIO_VID_2_P0              	0x7E 
#define PARSER_PIO_VID_2_P1              	0x7F 
#define PARSER_PIO_VID_3_P0              	0x80 
#define PARSER_PIO_VID_3_P1              	0x81 
#define PARSER_PIO_VID_4_P0              	0x82 
#define PARSER_PIO_VID_4_P1              	0x83 
#define PARSER_PIO_VID_5_P0              	0x84 
#define PARSER_PIO_VID_5_P1              	0x85 
#define PARSER_PIO_VID_6_P0              	0x86 
#define PARSER_PIO_VID_6_P1              	0x87 
#define PARSER_PIO_VID_7_P0              	0x88 
#define PARSER_PIO_VID_7_P1              	0x89 

/*	Parser output	registers
	Only available for port AP.
	Cannot access these from Host.	  	*/
#define PARSER_PIO_OUTPUT_0_P0  	   	0x8E 
#define PARSER_PIO_OUTPUT_0_P1           	0x8F 
#define PARSER_PIO_OUTPUT_1_P0           	0x90 
#define PARSER_PIO_OUTPUT_1_P1           	0x91 
#define PARSER_PIO_OUTPUT_2_P0           	0x92 
#define PARSER_PIO_OUTPUT_2_P1           	0x93 
#define PARSER_PIO_OUTPUT_3_P0           	0x94 
#define PARSER_PIO_OUTPUT_3_P1           	0x95 
#define PARSER_PIO_OUTPUT_4_P0           	0x96 
#define PARSER_PIO_OUTPUT_4_P1           	0x97 

/*	CRC engine Registers		  	*/
#define PARSER_PIO_PRG_CRC_P0     	   	0x98 
#define PARSER_PIO_PRG_CRCBE_P0          	0x99 
#define PARSER_PIO_PRG_CRCDATA_P0        	0x9A 
#define PARSER_PIO_PRG_CRCDATA_P1        	0x9B 

/*	Configuration Register continued  	*/
#define PARSER_PIO_MAX_BUF_LMT_P0		0x9C
#define PARSER_PIO_MAX_BUF_LMT_P1		0x9D
#define PARSER_PIO_DEF_VLANID_P0		0x9E
#define PARSER_PIO_DEF_VLANID_P1		0x9F
#define PARSER_PIO_CLASS_BUF_STATUS_P0		0xA0
#define PARSER_PIO_CLASS_BUF_STATUS_P1		0xA1


/*	
 * Use following macro to configure/access parser registers.
 */
#define _PARSER_PIO_ADDR(reg) ( (PARSER_PIO_ ## reg) + PARSER_PIO_BASE_ADDR )

/*
 * Parser config register bits.
 */
#define	PARSER_CFG_ENABLE_PARSER_BIT		0
#define PARSER_CFG_START_DMA_BIT              	1
#define PARSER_CFG_HDR_DEST_BIT0		2
#define PARSER_CFG_HDR_DEST_BIT1		3
#define PARSER_CFG_DATA_DEST_BIT0 		4
#define PARSER_CFG_DATA_DEST_BIT1		5
#define PARSER_CFG_ABORT_BIT			6
#define	PARSER_CFG_NEW_PACKET_BIT            	7

#define	RESET_PARSER_CFG_REG		( 1 << PARSER_CFG_ENABLE_PARSER_BIT 	|	\
					  1 << PARSER_CFG_HDR_DEST_BIT0 	|	\
					  0 << PARSER_CFG_HDR_DEST_BIT1		|	\
					  1 << PARSER_CFG_DATA_DEST_BIT0	|	\
					  0 << PARSER_CFG_DATA_DEST_BIT1	|	\
					  0 << PARSER_CFG_ABORT_BIT		|	\
					  1 << PARSER_CFG_NEW_PACKET_BIT )		

#define	PARSER_DMA_START		( 1 <<	PARSER_CFG_START_DMA_BIT )
#define	PARSER_DMA_BUSY			( 1 <<	PARSER_CFG_START_DMA_BIT )

#define	PARSER_DMA_BUSY_BIT			PARSER_CFG_START_DMA_BIT


/* 
 * Parser status register bits.
 */
#define	PARSER_STATUS_HDRWRT_BIT		0
#define	PARSER_STATUS_DMABLKDONE_BIT		1
#define	PARSER_STATUS_EOP_BIT			2
#define	PARSER_STATUS_DATADMADONE_BIT		3
#define PARSER_STATUS_PKTEOF_BIT		4
#define PARSER_STATUS_ABORTED_BIT		5
#define PARSER_STATUS_CRCERROR_BIT		9
#define PARSER_STATUS_OVERRUN_BIT		10
#define PARSER_STATUS_OVERRUN_CNT_BIT0		11
#define PARSER_STATUS_OVERRUN_CNT_BIT1		12
#define PARSER_STATUS_OVERRUN_CNT_BIT2		13
#define PARSER_STATUS_OVERRUN_CNT_BIT3		14
#define PARSER_STATUS_FILTER_BIT		15

/* NOTE: The status register will be reset by writing 1 to respective bits */
#define	RESET_PARSER_STATUS_REG			0xFFFF

#define	PARSER_STATUS_ERRORS	( 1 << PARSER_STATUS_ABORTED_BIT 	| \
				  1 << PARSER_STATUS_CRCERROR_BIT	| \
				  1 << PARSER_STATUS_OVERRUN_BIT	| \
				  1 << PARSER_STATUS_FILTER_BIT	)




/*
 * Parser circular buffer status register 
 */
#define	PARSER_CBSTATUS_PORT0_FULL_BIT		0
#define PARSER_CBSTATUS_PORT1_FULL_BIT		1
#define PARSER_CBSTATUS_PORT2_FULL_BIT       	2
#define PARSER_CBSTATUS_PORT3_FULL_BIT   	3
#define PARSER_CBSTATUS_PORT4_FULL_BIT  	4
#define PARSER_CBSTATUS_PORT5_FULL_BIT  	5






/*______________________________CBUS OFFSETS______________________________*/

/*
 * Following are the CBUS offset. These macros are used by HOST (MIPS) to configure parser.
 * These macros should not be used to refer from Port AP.
 */

/*      Configuration Registers           	*/

#define PARSER_CBUS_CFG_REG_P0    	  	0x000
#define PARSER_CBUS_CFG_REG_P1            	0x002
#define PARSER_CBUS_MAX_HDR_SIZE_REG_P0 	0x004
#define PARSER_CBUS_MAX_HDR_SIZE_REG_P1    	0x006
#define PARSER_CBUS_CNTRS_EN_REG_P0	  	0x008
#define PARSER_CBUS_CNTRS_EN_REG_P1   	  	0x00A
#define PARSER_CBUS_HDR_HOLE1_LEN_REG_P0	0x00C
#define PARSER_CBUS_HDR_HOLE1_LEN_REG_P1   	0x00E
#define PARSER_CBUS_HDR_HOLE2_LEN_REG_P0	0x010
#define PARSER_CBUS_HDR_HOLE2_LEN_REG_P1	0x012
#define PARSER_CBUS_HDR_DEST_REG_P0 	  	0x014
#define PARSER_CBUS_HDR_DEST_REG_P1 	  	0x016
#define PARSER_CBUS_DATA_DEST_REG_P0 	  	0x018
#define PARSER_CBUS_DATA_DEST_REG_P1 	  	0x01A

/*      Status Register                         */
#define PARSER_CBUS_STATUS_REG_P0 	  	0x01C
#define PARSER_CBUS_STATUS_REG_P1          	0x01E

/*      Counter Registers                       */
#define PARSER_CBUS_PKT_CNTR_P0      	  	0x020
#define PARSER_CBUS_PKT_CNTR_P1            	0x022
#define PARSER_CBUS_BRDCAST_CNTR_P0       	0x024
#define PARSER_CBUS_BRDCAST_CNTR_P1         	0x026
#define PARSER_CBUS_RCVDBYTES_CNTR_P0    	0x028
#define PARSER_CBUS_RCVDBYTES_CNTR_P1    	0x02A
#define PARSER_CBUS_RCVDBYTES_CNTR_P2 	  	0x02C
#define PARSER_CBUS_RCVDBYTES_CNTR_P3    	0x02E
#define PARSER_CBUS_TCP_CNTR_P0          	0x030
#define PARSER_CBUS_TCP_CNTR_P1          	0x032
#define PARSER_CBUS_UDP_CNTR_P0          	0x034
#define PARSER_CBUS_UDP_CNTR_P1          	0x036
#define PARSER_CBUS_AHPKT_CNTR_P0       	0x040
#define PARSER_CBUS_AHPKT_CNTR_P1        	0x042
#define PARSER_CBUS_ESPPKT_CNTR_P0     	  	0x044
#define PARSER_CBUS_ESPPKT_CNTR_P1    	  	0x046
#define PARSER_CBUS_IPV6_CNTR_P0        	0x048
#define PARSER_CBUS_IPV6_CNTR_P1         	0x04A
#define PARSER_CBUS_IPV4_CNTR_P0         	0x04C
#define PARSER_CBUS_IPV4_CNTR_P1         	0x04E

/*      MAC address table Registers             */
#define	PARSER_MAX_NUM_MACADDR_LIST		8

#define PARSER_CBUS_PRG_ETHADR0_P0    		0x06C
#define PARSER_CBUS_PRG_ETHADR0_P1     		0x06E
#define PARSER_CBUS_PRG_ETHADR0_P2        	0x070
#define PARSER_CBUS_PRG_ETHADR1_P0          	0x074
#define PARSER_CBUS_PRG_ETHADR1_P1        	0x076
#define PARSER_CBUS_PRG_ETHADR1_P2        	0x078
#define PARSER_CBUS_PRG_ETHADR2_P0        	0x07C
#define PARSER_CBUS_PRG_ETHADR2_P1        	0x07E
#define PARSER_CBUS_PRG_ETHADR2_P2        	0x080
#define PARSER_CBUS_PRG_ETHADR3_P0        	0x084
#define PARSER_CBUS_PRG_ETHADR3_P1        	0x086
#define PARSER_CBUS_PRG_ETHADR3_P2        	0x088
#define PARSER_CBUS_PRG_ETHADR4_P0        	0x08C
#define PARSER_CBUS_PRG_ETHADR4_P1        	0x08E
#define PARSER_CBUS_PRG_ETHADR4_P2        	0x090
#define PARSER_CBUS_PRG_ETHADR5_P0        	0x094
#define PARSER_CBUS_PRG_ETHADR5_P1        	0x096
#define PARSER_CBUS_PRG_ETHADR5_P2        	0x098
#define PARSER_CBUS_PRG_ETHADR6_P0        	0x09C
#define PARSER_CBUS_PRG_ETHADR6_P1        	0x09E
#define PARSER_CBUS_PRG_ETHADR6_P2        	0x0A0
#define PARSER_CBUS_PRG_ETHADR7_P0        	0x0A4
#define PARSER_CBUS_PRG_ETHADR7_P1        	0x0A6
#define PARSER_CBUS_PRG_ETHADR7_P2        	0x0A8

/*      IP address table Registers              */
#define PARSER_CBUS_PRG_IPADR0_P0      		0x0AC
#define PARSER_CBUS_PRG_IPADR0_P1         	0x0AE
#define PARSER_CBUS_PRG_IPADR0_P2         	0x0B0
#define PARSER_CBUS_PRG_IPADR0_P3         	0x0B2
#define PARSER_CBUS_PRG_IPADR0_P4         	0x0B4
#define PARSER_CBUS_PRG_IPADR0_P5         	0x0B6
#define PARSER_CBUS_PRG_IPADR0_P6         	0x0B8
#define PARSER_CBUS_PRG_IPADR0_P7         	0x0BA
#define PARSER_CBUS_PRG_IPADR0_P8         	0x0BC
#define PARSER_CBUS_PRG_IPADR1_P0         	0x0C0
#define PARSER_CBUS_PRG_IPADR1_P1         	0x0C2
#define PARSER_CBUS_PRG_IPADR1_P2         	0x0C4
#define PARSER_CBUS_PRG_IPADR1_P3         	0x0C6
#define PARSER_CBUS_PRG_IPADR1_P4         	0x0C8
#define PARSER_CBUS_PRG_IPADR1_P5         	0x0CA
#define PARSER_CBUS_PRG_IPADR1_P6         	0x0CC
#define PARSER_CBUS_PRG_IPADR1_P7         	0x0CE
#define PARSER_CBUS_PRG_IPADR1_P8         	0x0D0
#define PARSER_CBUS_PRG_IPADR2_P0         	0x0D4
#define PARSER_CBUS_PRG_IPADR2_P1         	0x0D6
#define PARSER_CBUS_PRG_IPADR2_P2         	0x0D8
#define PARSER_CBUS_PRG_IPADR2_P3         	0x0DA
#define PARSER_CBUS_PRG_IPADR2_P4         	0x0DC
#define PARSER_CBUS_PRG_IPADR2_P5         	0x0DE
#define PARSER_CBUS_PRG_IPADR2_P6         	0x0E0
#define PARSER_CBUS_PRG_IPADR2_P7         	0x0E2
#define PARSER_CBUS_PRG_IPADR2_P8         	0x0E4
#define PARSER_CBUS_PRG_IPADR3_P0         	0x0E8
#define PARSER_CBUS_PRG_IPADR3_P1         	0x0EA
#define PARSER_CBUS_PRG_IPADR3_P2         	0x0EC
#define PARSER_CBUS_PRG_IPADR3_P3         	0x0EE
#define PARSER_CBUS_PRG_IPADR3_P4         	0x0F0
#define PARSER_CBUS_PRG_IPADR3_P5         	0x0F2
#define PARSER_CBUS_PRG_IPADR3_P6         	0x0F4
#define PARSER_CBUS_PRG_IPADR3_P7         	0x0F6
#define PARSER_CBUS_PRG_IPADR3_P8         	0x0F8

/*      Vendor ID table registers               */
#define PARSER_CBUS_VID_0_P0      	  	0x0FC
#define PARSER_CBUS_VID_0_P1              	0x0FE
#define PARSER_CBUS_VID_1_P0              	0x100
#define PARSER_CBUS_VID_1_P1              	0x102
#define PARSER_CBUS_VID_2_P0              	0x104
#define PARSER_CBUS_VID_2_P1              	0x106
#define PARSER_CBUS_VID_3_P0              	0x108
#define PARSER_CBUS_VID_3_P1              	0x10A
#define PARSER_CBUS_VID_4_P0              	0x10C
#define PARSER_CBUS_VID_4_P1              	0x10E
#define PARSER_CBUS_VID_5_P0              	0x110
#define PARSER_CBUS_VID_5_P1              	0x112
#define PARSER_CBUS_VID_6_P0              	0x114
#define PARSER_CBUS_VID_6_P1              	0x116
#define PARSER_CBUS_VID_7_P0              	0x118
#define PARSER_CBUS_VID_7_P1              	0x11A

/*      Parser output   registers
        Only available for port AP.
        Cannot access these from Host.          */
#define PARSER_CBUS_OUTPUT_0_P0                 0x11C
#define PARSER_CBUS_OUTPUT_0_P1                 0x11E
#define PARSER_CBUS_OUTPUT_1_P0                 0x120
#define PARSER_CBUS_OUTPUT_1_P1                 0x122
#define PARSER_CBUS_OUTPUT_2_P0                 0x124
#define PARSER_CBUS_OUTPUT_2_P1                 0x126
#define PARSER_CBUS_OUTPUT_3_P0                 0x128
#define PARSER_CBUS_OUTPUT_3_P1                 0x12A
#define PARSER_CBUS_OUTPUT_4_P0                 0x12C
#define PARSER_CBUS_OUTPUT_4_P1                 0x12E

/*      CRC engine Registers                    */
#define PARSER_CBUS_PRG_CRC_P0     	  	0x130
#define PARSER_CBUS_PRG_CRCBE_P0          	0x132
#define PARSER_CBUS_PRG_CRCDATA_P0        	0x134
#define PARSER_CBUS_PRG_CRCDATA_P1        	0x136

/*      Configuration Register continued        */
#define PARSER_CBUS_MAX_BUF_LMT_P0		0x138
#define PARSER_CBUS_MAX_BUF_LMT_P1		0x13A
#define PARSER_CBUS_DEF_VLANID_P0		0x13C
#define PARSER_CBUS_DEF_VLANID_P1		0x13E
#define PARSER_CBUS_CLASS_BUF_STATUS_P0		0x140
#define PARSER_CBUS_CLASS_BUF_STATUS_P1		0x142


#if 0
#if defined(MAC1_AP) && (MAC1_AP)
	#define	PARSER_CBUS_BASE_ADDR		0xABCD
#elif MAC2_AP
	#define PARSER_CBUS_BASE_ADDR   	0xABCD
#elif MAC3_AP
	#define PARSER_CBUS_BASE_ADDR           0xABCD
#else
	Parser: Parserhw.h Error
#endif        
#endif


#define _PARSER_CBUS_ADDR(reg) ( PARSER_CBUS ## reg ## + PARSER_CBUS_BASE_ADDR )





/*_____________________________________Header Gaurd_____________________________________*/
#endif	//_PARSERHW_H_











