/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/

/****************************************************************************************
 *                                                                                      *
 * File name    : parsertag.h                                                         	*
 * Author       : Ikanos Communications  [ jpattar@ikanos.com ] 			*
 * Version      :                                                                       *
 * Date         : 08th Oct 2009                                                         *
 * Description  : This file contains the parser's output. When packet received, the   	*
 *                parser gives this output tag.     					*
 *                                                                                      *
 ****************************************************************************************/

/*_____________________________________Header Gaurd_____________________________________*/

#ifndef _PARSERTAG_H_
#define _PARSERTAG_H_

/*_______________________________________Includes_______________________________________*/
#include "parserhw.h"


/*________________________________________MACROS________________________________________*/

/*
 * Following are the flag setting of parser tag.
 * Flag1, flag2 (s/w uderstanding) are with respect to 16bits.
 * Flag3 is of char type i.e. 8bit.
 */

/* First 16bits of parser tag */
#define	PARSERTAG_IPV4_BIT			15
#define	PARSERTAG_IPV6_BIT			14
#define PARSERTAG_PPPDATA_BIT			13
#define PARSERTAG_TCP_BIT			12
#define PARSERTAG_UDP_BIT			11
#define PARSERTAG_AH_BIT			10
#define PARSERTAG_ESP_BIT			9
#define PARSERTAG_TUNNELIPV6_BIT		8
#define PARSERTAG_TUNNELIPV4_BIT		7
#define PARSERTAG_MULTICAST_BIT			6
#define PARSERTAG_VLANTAG1_BIT			5
#define PARSERTAG_VLANTAG2_BIT			4
#define PARSERTAG_FRAGMENT_BIT			3
#define PARSERTAG_TCPACK_BIT			2
#define PARSERTAG_TCPURGENT_BIT			1
#define PARSERTAG_IP_OPTION_BIT			0

/* second 16bits of parser tag */
#define PARSERTAG_ARP_RARP_BIT			15
#define PARSERTAG_BROADCAST_BIT			14
#define PARSERTAG_PPPCONTROL_BIT		13
#define PARSERTAG_TCPSYNFIN_BIT			12
#define PARSERTAG_IPCHECKSUM_ERROR_BIT		11
#define	PARSERTAG_UNKNOWN_ETHTYPE_BIT		10
#define PARSERTAG_IGMP_BIT			9
#define PARSERTAG_ICMP_BIT			8
#define PARSERTAG_TTL_BIT			7
//Just for completeness
#define	PARSERTAG_DEST_IP_MATCH_BIT2		6
#define PARSERTAG_DEST_IP_MATCH_BIT1            5
#define PARSERTAG_DEST_IP_MATCH_BIT0            4
#define	PARSERTAG_DEST_MAC_MATCH_BIT3		3
#define PARSERTAG_DEST_MAC_MATCH_BIT2           2
#define PARSERTAG_DEST_MAC_MATCH_BIT1           1
#define PARSERTAG_DEST_MAC_MATCH_BIT0           0


/* third 8bits of parser tag */
#define	PARSERTAG_RESERVED_BIT0			7
#define PARSERTAG_RESERVED_BIT1			6
#define PARSERTAG_FLOWLABEL_NOTZERO_BIT		5
#define	PARSERTAG_SNAP_BIT			4
#define	PARSERTAG_VENDORTAG_MATCH_BIT3		3
#define PARSERTAG_VENDORTAG_MATCH_BIT2		2
#define PARSERTAG_VENDORTAG_MATCH_BIT1		1
#define PARSERTAG_VENDORTAG_MATCH_BIT0		0

/*
 * Parser status register bits.
 */
#define PARSERTAG_STATUS_HDRWRT_BIT		PARSER_STATUS_HDRWRT_BIT
#define PARSERTAG_STATUS_DMABLKDONE_BIT 	PARSER_STATUS_DMABLKDONE_BIT
#define PARSERTAG_STATUS_EOP_BIT		PARSER_STATUS_EOP_BIT
#define PARSERTAG_STATUS_DATADMADONE_BIT	PARSER_STATUS_DATADMADONE_BIT
#define	PARSERTAG_STATUS_PKTEOF_BIT        	PARSER_STATUS_PKTEOF_BIT
#define PARSERTAG_STATUS_ABORTED_BIT		PARSER_STATUS_ABORTED_BIT
#define PARSERTAG_STATUS_CRCERROR_BIT		PARSER_STATUS_CRCERROR_BIT
#define PARSERTAG_STATUS_OVERRUN_BIT		PARSER_STATUS_OVERRUN_BIT
#define PARSERTAG_STATUS_OVERRUN_CNT_BIT0	PARSER_STATUS_OVERRUN_CNT_BIT0
#define PARSERTAG_STATUS_OVERRUN_CNT_BIT1	PARSER_STATUS_OVERRUN_CNT_BIT1
#define PARSERTAG_STATUS_OVERRUN_CNT_BIT2 	PARSER_STATUS_OVERRUN_CNT_BIT2
#define PARSERTAG_STATUS_OVERRUN_CNT_BIT3	PARSER_STATUS_OVERRUN_CNT_BIT3
#define PARSERTAG_STATUS_FILTER_BIT 		PARSER_STATUS_FILTER_BIT


#define PARSERTAG_STATUS_ERRORS			PARSER_STATUS_ERRORS





/* destination IP match: Register Reg2 contains result. Reg1 should
   point to flag2 of parser tag */
#define	PARSERTAG_DEST_IPMATCH( Reg1, Reg2 ) 	lbl	Reg2, 0x70 \
						and	Reg2, Reg1 \
						lsr	Reg2	   \
						lsr	Reg2 	   \
						lsr	Reg2 	   \
						lsr	Reg2


/* destination MAC match: These bits will be used to find whether to
   go with routing or bridging. Reg1 should point to MAC fields */
#define	PARSERTAG_DEST_MACMATCH( Reg1, Reg2 )	lbl	Reg2, 0xF  \
						and	Reg2, Reg1

/* Vendor Tag: Note - the apcode should do lb instruction instead lw */
#define	PARSERTAG_VENDORTAG_MATCH( Reg1, Reg2 )	lbl	Reg2, 0xF  \
						and	Reg2, Reg1


/* L3 Hash type */
#define PARSERTAG_L3HASH_TYPE( Reg1, Reg2 )	lbl	Reg2, 0xF0 \
						and	Reg1, Reg2 \
						lsr     Reg2       \
                                                lsr     Reg2       \
                                                lsr     Reg2       \
                                                lsr     Reg2

/* L4 Hash type */
#define PARSERTAG_L2HASH_TYPE( Reg1, Reg2 )     lbl     Reg2, 0xF  \
                                                and     Reg2, Reg1


/*
 * NOTATION: 	    Following masks bits has following understanding.
 * if bit is set 1   --> Send that packet to host or drop the packet.
 * if bit is clear 0 --> Proceed next, contiue with bridging/routing.
 */

#if defined(IPV6) && (IPV6)
#define CONDITIONAL_IPV6_BITS            0 << PARSERTAG_IPV6_BIT | 0 << PARSERTAG_TUNNELIPV6_BIT | 0 << PARSERTAG_TUNNELIPV4_BIT
#else
#define CONDITIONAL_IPV6_BITS            1 << PARSERTAG_IPV6_BIT | 1 << PARSERTAG_TUNNELIPV6_BIT | 1 << PARSERTAG_TUNNELIPV4_BIT
#endif

#define PARSERTAG_FLAG1_HOSTBITS_MASK  	( 				  \
                                          0 << PARSERTAG_IPV4_BIT       | \
                                          0 << PARSERTAG_PPPDATA_BIT 	| \
            				  0 << PARSERTAG_TCP_BIT	| \
                                      	  0 << PARSERTAG_UDP_BIT        | \
					  1 << PARSERTAG_AH_BIT         | \
                                          1 << PARSERTAG_ESP_BIT        | \
                                          0 << PARSERTAG_MULTICAST_BIT  | \
                                          0 << PARSERTAG_VLANTAG1_BIT   | \
                                          0 << PARSERTAG_VLANTAG2_BIT   | \
                                          0 << PARSERTAG_FRAGMENT_BIT   | \
                                          0 << PARSERTAG_TCPACK_BIT     | \
                                          0 << PARSERTAG_TCPURGENT_BIT  | \
                                          1 << PARSERTAG_IP_OPTION_BIT  | \
                                          CONDITIONAL_IPV6_BITS )
                                       

#define	PARSERTAG_FLAG2_HOSTBITS_MASK	(				  	  \
                                          1 << PARSERTAG_ARP_RARP_BIT		| \
                                          1 << PARSERTAG_BROADCAST_BIT		| \
                                          1 << PARSERTAG_PPPCONTROL_BIT		| \
                                          1 << PARSERTAG_TCPSYNFIN_BIT		| \
                                          1 << PARSERTAG_IPCHECKSUM_ERROR_BIT	| \
                                          1 << PARSERTAG_UNKNOWN_ETHTYPE_BIT	| \
                                          1 << PARSERTAG_IGMP_BIT		| \
                                          1 << PARSERTAG_ICMP_BIT		| \
                                          0 << PARSERTAG_TTL_BIT		| \
                                          0 << PARSERTAG_DEST_IP_MATCH_BIT2	| \
                                          0 << PARSERTAG_DEST_IP_MATCH_BIT1	| \
                                          0 << PARSERTAG_DEST_IP_MATCH_BIT0	| \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT3	| \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT2	| \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT1	| \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT0	  \
					)

#define PARSERTAG_FLAG3_HOSTBITS_MASK 	(                                         \
                                          0 << PARSERTAG_RESERVED_BIT0		| \
                                          0 << PARSERTAG_RESERVED_BIT1		| \
                                          1 << PARSERTAG_FLOWLABEL_NOTZERO_BIT  | \
                                          1 << PARSERTAG_SNAP_BIT		| \
                                          1 << PARSERTAG_VENDORTAG_MATCH_BIT3	| \
                                          1 << PARSERTAG_VENDORTAG_MATCH_BIT2	| \
                                          1 << PARSERTAG_VENDORTAG_MATCH_BIT1	| \
                                          1 << PARSERTAG_VENDORTAG_MATCH_BIT0	  \
					)


#define PARSERTAG_FLAG2_ROUTE_TO_HOSTBITS_MASK   (                                \
                                          1 << PARSERTAG_ARP_RARP_BIT           | \
                                          1 << PARSERTAG_BROADCAST_BIT          | \
                                          1 << PARSERTAG_PPPCONTROL_BIT         | \
                                          1 << PARSERTAG_TCPSYNFIN_BIT          | \
                                          1 << PARSERTAG_IPCHECKSUM_ERROR_BIT   | \
                                          1 << PARSERTAG_UNKNOWN_ETHTYPE_BIT    | \
                                          1 << PARSERTAG_IGMP_BIT               | \
                                          1 << PARSERTAG_ICMP_BIT               | \
                                          1 << PARSERTAG_TTL_BIT                | \
                                          0 << PARSERTAG_DEST_IP_MATCH_BIT2     | \
                                          0 << PARSERTAG_DEST_IP_MATCH_BIT1     | \
                                          0 << PARSERTAG_DEST_IP_MATCH_BIT0     | \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT3    | \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT2    | \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT1    | \
                                          0 << PARSERTAG_DEST_MAC_MATCH_BIT0      \
                                        )


#define PARSERTAG_FLAG2_DESTMAC_MATCHBITS_MASK (                                  \
                                          1 << PARSERTAG_DEST_MAC_MATCH_BIT3    | \
                                          1 << PARSERTAG_DEST_MAC_MATCH_BIT2    | \
                                          1 << PARSERTAG_DEST_MAC_MATCH_BIT1    | \
                                          1 << PARSERTAG_DEST_MAC_MATCH_BIT0      \
                                        )

#define PARSERTAG_FLAG1_MULTICAST_MASK  (1<< PARSERTAG_MULTICAST_BIT)	


#define PARSERTAG_FLAG1_PACKETTYPEBITS_MASK  	( 				          \
											  1 << PARSERTAG_TCP_BIT		| \
											  1 << PARSERTAG_UDP_BIT		| \
											  1 << PARSERTAG_AH_BIT 		| \
											  1 << PARSERTAG_ESP_BIT		  \
						) 

				
/*__________________________________struct parserTag_s__________________________________*/

typedef struct parserTag_s
{
	void			*cluster;
	unsigned short		flag1;
	unsigned short		flag2;
	unsigned char		ipOff;
	unsigned char		protoOff;
	unsigned char		flag3;
	unsigned char		pppOff;
	unsigned char		outerIpOff;
	unsigned char		hashType;
	unsigned short		hash3;
	unsigned short		hash1;
	unsigned short		hash2;
	unsigned short		statusBits;
	unsigned short		totalLength;
}parserTag_t;

/* Parser tag offsets: All offsets are from ParserTag */

#define PARSERTAG_CLUSTER_MSB_OFFSET		typedef parserTag_t.cluster
#define PARSERTAG_CLUSTER_LSB_OFFSET   		typedef parserTag_t.cluster + 2
#define	PARSERTAG_FLAG1_OFFSET			typedef parserTag_t.flag1
#define PARSERTAG_FLAG2_OFFSET			typedef parserTag_t.flag2
#define PARSERTAG_IP_OFFSET                     typedef parserTag_t.ipOff
#define PARSERTAG_PROTO_OFFSET                  typedef parserTag_t.protoOff
#define PARSERTAG_FLAG3_OFFSET			typedef parserTag_t.flag3
#define PARSERTAG_PPP_OFFSET                    typedef parserTag_t.pppOff
#define PARSERTAG_OUTER_IP_OFFSET               typedef parserTag_t.outerIpOff
#define PARSERTAG_HASHTYPE_OFFSET		typedef parserTag_t.hashType
#define PARSERTAG_L3HASH_OFFSET                 typedef parserTag_t.hash3
#define PARSERTAG_L2HASH1_OFFSET                typedef parserTag_t.hash1
#define PARSERTAG_L2HASH2_OFFSET                typedef parserTag_t.hash2
#define PARSERTAG_STATUSBITS_OFFSET             typedef parserTag_t.statusBits
#define PARSERTAG_TOTALLENGTH_OFFSET            typedef parserTag_t.totalLength



/*_____________________________________Header Gaurd_____________________________________*/
#endif  //_PARSERTAG_H_ Header Gaurd end

