
/****************************************************************************************
 *                                                                                      *
 * Copyright (C) 2007 Ikanos Communications Inc. All rights reserved                    *
 *                                                                                      *
 * The  information  and  source  code contained  herein  is the  exclusive  property   *
 * of Ikanos Communications and may not be disclosed, examined or reproduced in whole   *
 * or in part without  the explicit written authorization from Ikanos Communications.   *
 *                                                                                      *
 ****************************************************************************************/


#ifndef _SCHEDULER_H_
#define _SCHEDULER_H_

typedef struct apSchedQueue_s // This structure includes any of the queue dependent params of Scheduler
{
    unsigned short minBytesInt;
    unsigned short maxBytesInt;
    unsigned char  minBytesFrac;
    unsigned char  maxBytesFrac;
}apSchedQueue_t;

typedef struct apSchedConfig_s
{
    unsigned short  totalBytesInt;
    unsigned short  totalBytesFrac;
    unsigned short  schedClkCntrl;
    unsigned char   donateBW;
    unsigned char   receiveBW;
    unsigned char	tcpAckPriority;
    unsigned char   schedCntrl;
#define AP_ETH_Q_TO_TX		8
    apSchedQueue_t   txScheduler[AP_ETH_Q_TO_TX];

}apSchedConfig_t;

/* This file will have the definitions of all the registers of the Scheduler HW block at TX
 * (QUEUEING DMA)
 *
 *
 *
 */
#define SCHEDULER_BASE_ADDR             0x1000 

#define SCH_HEAD_OF_Q0_PKT_SIZE_OFFSET                  0x0000
#define SCH_HEAD_OF_Q1_PKT_SIZE_OFFSET                  0x0001
#define SCH_HEAD_OF_Q2_PKT_SIZE_OFFSET                  0x0002
#define SCH_HEAD_OF_Q3_PKT_SIZE_OFFSET                  0x0003
#define SCH_HEAD_OF_Q4_PKT_SIZE_OFFSET                  0x0004
#define SCH_HEAD_OF_Q5_PKT_SIZE_OFFSET                  0x0005
#define SCH_HEAD_OF_Q6_PKT_SIZE_OFFSET                  0x0006
#define SCH_HEAD_OF_Q7_PKT_SIZE_OFFSET                  0x0007
#define SCH_PQ_CREDIT_CNTR_INT_OFFSET                   0x0008
#define SCH_PQ_CREDIT_CNTR_FRAC_OFFSET                  0x0009
#define SCH_TOTAL_BYTES_INT_OFFSET                              0x000A
#define SCH_TOTAL_BYTES_FRAC_OFFSET                             0x000B
#define SCH_CLK_CONTROL_OFFSET                                  0x000C
#define SCH_Q0_MIN_BW_INT_OFFSET                                0x000D
#define SCH_Q1_MIN_BW_INT_OFFSET                                0x000E
#define SCH_Q2_MIN_BW_INT_OFFSET                                0x000F
#define SCH_Q3_MIN_BW_INT_OFFSET                                0x0010
#define SCH_Q4_MIN_BW_INT_OFFSET                                0x0011
#define SCH_Q5_MIN_BW_INT_OFFSET                                0x0012
#define SCH_Q6_MIN_BW_INT_OFFSET                                0x0013
#define SCH_Q7_MIN_BW_INT_OFFSET                                0x0014
#define SCH_Q0_MIN_BW_FRAC_OFFSET                               0x0015
#define SCH_Q1_MIN_BW_FRAC_OFFSET                               0x0016
#define SCH_Q2_MIN_BW_FRAC_OFFSET                               0x0017
#define SCH_Q3_MIN_BW_FRAC_OFFSET                               0x0018
#define SCH_Q4_MIN_BW_FRAC_OFFSET                               0x0019
#define SCH_Q5_MIN_BW_FRAC_OFFSET                               0x001A
#define SCH_Q6_MIN_BW_FRAC_OFFSET                               0x001B
#define SCH_Q7_MIN_BW_FRAC_OFFSET                               0x001C
#define SCH_Q0_MAX_BW_INT_OFFSET                                0x001D
#define SCH_Q1_MAX_BW_INT_OFFSET                                0x001E
#define SCH_Q2_MAX_BW_INT_OFFSET                                0x001F
#define SCH_Q3_MAX_BW_INT_OFFSET                                0x0020
#define SCH_Q4_MAX_BW_INT_OFFSET                                0x0021
#define SCH_Q5_MAX_BW_INT_OFFSET                                0x0022
#define SCH_Q6_MAX_BW_INT_OFFSET                                0x0023
#define SCH_Q7_MAX_BW_INT_OFFSET                                0x0024
#define SCH_Q0_MAX_BW_FRAC_OFFSET                               0x0025
#define SCH_Q1_MAX_BW_FRAC_OFFSET                               0x0026
#define SCH_Q2_MAX_BW_FRAC_OFFSET                               0x0027
#define SCH_Q3_MAX_BW_FRAC_OFFSET                               0x0028
#define SCH_Q4_MAX_BW_FRAC_OFFSET                               0x0029
#define SCH_Q5_MAX_BW_FRAC_OFFSET                               0x002A
#define SCH_Q6_MAX_BW_FRAC_OFFSET                               0x002B
#define SCH_Q7_MAX_BW_FRAC_OFFSET                               0x002C
#define SCH_Q0_CREDIT_CNTR_INT_OFFSET                           0x002D
#define SCH_Q1_CREDIT_CNTR_INT_OFFSET                           0x002E
#define SCH_Q2_CREDIT_CNTR_INT_OFFSET                           0x002F
#define SCH_Q3_CREDIT_CNTR_INT_OFFSET                           0x0030
#define SCH_Q4_CREDIT_CNTR_INT_OFFSET                           0x0031
#define SCH_Q5_CREDIT_CNTR_INT_OFFSET                           0x0032
#define SCH_Q6_CREDIT_CNTR_INT_OFFSET                           0x0033
#define SCH_Q7_CREDIT_CNTR_INT_OFFSET                           0x0034
#define SCH_Q0_CREDIT_CNTR_FRAC_OFFSET                  	0x0035
#define SCH_Q1_CREDIT_CNTR_FRAC_OFFSET                  	0x0036
#define SCH_Q2_CREDIT_CNTR_FRAC_OFFSET                  	0x0037
#define SCH_Q3_CREDIT_CNTR_FRAC_OFFSET                  	0x0038
#define SCH_Q4_CREDIT_CNTR_FRAC_OFFSET                  	0x0039
#define SCH_Q5_CREDIT_CNTR_FRAC_OFFSET                  	0x003A
#define SCH_Q6_CREDIT_CNTR_FRAC_OFFSET                  	0x003B
#define SCH_Q7_CREDIT_CNTR_FRAC_OFFSET                  	0x003C
#define SCH_DONATE_BW_OFFSET                                    0x003D
#define SCH_RECEIVE_BW_OFFSET                                   0x003E
#define SCH_NEXT_Q_TO_SERVICE_OFFSET                            0x003F
#define SCH_CONTROL_OFFSET                                      0x0040
#define SCH_Q_ACTIVE_OFFSET                                     0x0041


#define  _SCHED_ADDR(reg) ( SCH_ ## reg ## _OFFSET + SCHEDULER_BASE_ADDR )

#define SCH_QUEUE_INVALID_BIT    				7
#define	SCHED_CREDIT_CNTR_UPDATE_DISABLE_BIT			15

#define SCH_ETHERNET_IPG_VALUE					20 + 4	/* 20 bytes IPG + 4 bytes CRC  */	
#define ETHERNET_CRC_LEN					4	/* 4 bytes */


#endif
